(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[    404568,      13235]
NotebookOptionsPosition[    340496,      11461]
NotebookOutlinePosition[    362279,      11933]
CellTagsIndexPosition[    361208,      11896]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Counting", "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 6}}],

Cell[CellGroupData[{

Cell["Introduction", "Section"],

Cell[TextData[{
 "This chapter presents a variety of techniques that are available in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " for counting a diverse collection of discrete objects, including \
combinations and permutations of finite sets. Objects can be counted using \
formulae or by using algorithms to list the objects and then directly \
counting the size of the list."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["6.1 The Basics of Counting", "Section",
 CellTags->"Section1"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to perform the computations needed to solve basic counting \
problems. We will begin by looking at some examples. We will discuss \
computations involving large integers. Then we'll see how the principles of \
counting can be used to count the number of operations used by a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function. This section concludes by using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions to solve counting problems by enumerating all the \
possibilities."
}], "Text"],

Cell[CellGroupData[{

Cell["Basic Examples", "Subsection",
 CellTags->"Section1a"],

Cell[TextData[{
 "We begin with two basic examples to demonstrate the use of some useful ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions."
}], "Text"],

Cell[CellGroupData[{

Cell["Counting One-to-one Functions", "Subsubsection"],

Cell[TextData[{
 "Recall Example 7 from Section 6.1 of the text. That example calculated that \
the number of one-to-one functions from a set with ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " elements to a set with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements is "
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"n", "(", 
    RowBox[{"n", "-", "1"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"n", "-", "2"}], ")"}], 
   RowBox[{"\[CenterEllipsis]", "(", 
    RowBox[{"n", "-", "m", "+", "1"}], ")"}]}], TraditionalForm]], "Equation"],

Cell["Note that we can rewrite this using product notation as", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Product]", 
     StyleBox[
      RowBox[{"i", "=", "0"}], "TI"], 
     RowBox[{"m", "-", "1"}],
     LimitsPositioning->False], "n"}], "-", "i", " "}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "For small values of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", it is easy to enter this product in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". For instance, the expression below computes the number of one-to-one \
functions from a set of 4 elements to a set of 20 elements."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"20", "*", "19", "*", "18", "*", "17"}]], "Input"],

Cell[BoxData["116280"], "Output"]
}, Open  ]],

Cell[TextData[{
 "For larger values of ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ", it is more convenient to use the ",
 StyleBox[ButtonBox["Product",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Product"], "FunctionLink"],
 " function. The ",
 StyleBox[ButtonBox["Product",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Product"], "FunctionLink"],
 " function is used to multiply a sequence of values. Its syntax is identical \
to that of ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 ". The first argument is an expression in terms of a variable (e.g., ",
 StyleBox["i", "CodeVar"],
 ") that evaluates to the values that are to be multiplied together. The \
second argument can take a variety of forms. One usage has the form ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["i", "CodeVar"], ",", 
   StyleBox["a", "CodeVar"], ",", 
   StyleBox["b", "CodeVar"], "}"}]], "myCode"],
 " which indicates that the variable ",
 StyleBox["i", "CodeVar"],
 " is to range from a minimum of ",
 StyleBox["a", "CodeVar"],
 " to a maximum of ",
 StyleBox["b", "CodeVar"],
 ". If the minimum value is to be 1, then ",
 StyleBox["a", "CodeVar"],
 " can be omitted. "
}], "Text"],

Cell["\<\
For example, we can recompute the number of one-to-one functions from a set \
of 20 elements to a set of 4 elements as follows.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Product", "[", 
  RowBox[{
   RowBox[{"20", "-", "i"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "3"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["116280"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The second argument indicates that the index variable will be assigned the \
integers 0, 1, 2 and 3. These are then substituted into the first argument, \
",
 Cell[BoxData[
  RowBox[{"20", "-", "i"}]], "myCode"],
 ", producing the values 20, 19, 18, and 17, which are multiplied together."
}], "Text"],

Cell["\<\
We can easily compute the number of one-to-one functions from a set of 12 \
elements to a set of 300 elements.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Product", "[", 
  RowBox[{
   RowBox[{"300", "-", "i"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "11"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["425270752192695317567218560000"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Computer Passwords", "Subsubsection"],

Cell["\<\
Example 16 from Section 6.1 describes a computer system in which each user \
has a password that must be between six and eight characters long and \
consisting of uppercase letters or digits. Also, each password must contain \
at least one digit. \
\>", "Text"],

Cell["\<\
The solution to the example describes how to calculate this. For each \
password length, 6, 7, or 8, the number of passwords are\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"P6", "=", 
  RowBox[{
   RowBox[{"36", "^", "6"}], "-", 
   RowBox[{"26", "^", "6"}]}]}]], "Input"],

Cell[BoxData["1867866560"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"P7", "=", 
  RowBox[{
   RowBox[{"36", "^", "7"}], "-", 
   RowBox[{"26", "^", "7"}]}]}]], "Input"],

Cell[BoxData["70332353920"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"P8", "=", 
  RowBox[{
   RowBox[{"36", "^", "8"}], "-", 
   RowBox[{"26", "^", "8"}]}]}]], "Input"],

Cell[BoxData["2612282842880"], "Output"]
}, Open  ]],

Cell["Thus the total number of possible passwords is ", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"P", "=", 
  RowBox[{"P6", "+", "P7", "+", "P8"}]}]], "Input"],

Cell[BoxData["2684483063360"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can use the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function to perform this calculation in one step."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"36", "^", "i"}], "-", 
    RowBox[{"26", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "6", ",", "8"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["2684483063360"], "Output"]
}, Open  ]],

Cell["\<\
This makes it easy to compute the number of valid passwords for larger \
ranges. For instance, it is common to require passwords to be between 8 and \
16 characters. If we retain the rules that the password be uppercase letters \
or numbers and include at least one number, then the total number of possible \
passwords is calculated with the expression below.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"36", "^", "i"}], "-", 
    RowBox[{"26", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "8", ",", "16"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["8140698334757962630580480"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Working with Large Integers", "Subsection",
 CellTags->"Section1b"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s computational engine is able to work with arbitrarily large integers, \
subject only to the limitations imposed by the computer's memory and speed. "
}], "Text"],

Cell[CellGroupData[{

Cell["DNA", "Subsubsection"],

Cell[TextData[{
 "In Example 11, the text provides a brief description of DNA and concludes \
that there are at least ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["4", 
    SuperscriptBox["10", "8"]], TraditionalForm]]],
 " different possible sequences of bases in the DNA for complex organisms."
}], "Text"],

Cell[TextData[{
 "To have ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " compute this value, we just enter the expression and wait for it to \
complete the computation. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DNAsequences", "=", 
  RowBox[{"4", "^", 
   RowBox[{"10", "^", "8"}]}]}]], "Input"],

Cell[BoxData[
 InterpretationBox[
  TagBox[
   PanelBox[GridBox[{
      {
       StyleBox[
        StyleBox[
         DynamicBox[ToBoxes[
           FEPrivate`FrontEndResource["FEStrings", "sizeExplanation"], 
           StandardForm],
          ImageSizeCache->{278., {2., 8.}}],
         StripOnInput->False,
         DynamicUpdating->True], "Panel",
        StripOnInput->False,
        Background->None]},
      {
       ItemBox[
        TagBox[
         RowBox[{
         "13576763067144082109859764943426215110923900940995100861638392126957\
291410917608911489552935725380527448262970047404815772410095615122011337654446\
", 
          RowBox[{"\[LeftSkeleton]", "60205707", "\[RightSkeleton]"}], 
          "4347580097059138134536253777721694042210623817508491267308464323788\
014197939109068927095524265411013142858298323361401629914610165163562737871093\
76"}],
         Short[#, 5]& ],
        Background->GrayLevel[1],
        BaseStyle->{Deployed -> False},
        Frame->True,
        FrameStyle->GrayLevel[0, 0.2],
        StripOnInput->False]},
      {
       RowBox[{
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowLess"], 
             StandardForm],
            ImageSizeCache->{49., {1., 8.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 10, 23501633677001016999, 5/2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowMore"], 
             StandardForm],
            ImageSizeCache->{52., {1., 8.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 10, 23501633677001016999, 5 2],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeShowAll"], 
             StandardForm],
            ImageSizeCache->{82., {2., 8.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>OutputSizeLimit`ButtonFunction[
          Identity, 10, 23501633677001016999, Infinity],
         Enabled->True,
         Evaluator->Automatic,
         Method->"Queued"], "\[ThinSpace]", 
        ButtonBox[
         StyleBox[
          StyleBox[
           DynamicBox[ToBoxes[
             FEPrivate`FrontEndResource["FEStrings", "sizeChangeLimit"], 
             StandardForm],
            ImageSizeCache->{74., {1., 8.}}],
           StripOnInput->False,
           DynamicUpdating->True], "Panel",
          StripOnInput->False,
          Background->None],
         Appearance->Automatic,
         ButtonFunction:>FrontEndExecute[{
            FrontEnd`SetOptions[
            FrontEnd`$FrontEnd, 
             FrontEnd`PreferencesSettings -> {"Page" -> "Evaluation"}], 
            FrontEnd`FrontEndToken["PreferencesDialog"]}],
         Evaluator->None,
         Method->"Preemptive"]}]}
     },
     DefaultBaseStyle->"Column",
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxDividers->{
      "Columns" -> {{False}}, "ColumnsIndexed" -> {}, "Rows" -> {{False}}, 
       "RowsIndexed" -> {}},
     GridBoxItemSize->{
      "Columns" -> {{Automatic}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[1.2]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    DefaultBaseStyle->{},
    FrameMargins->5],
   Deploy,
   DefaultBaseStyle->"Deploy"],
  Out[10]]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " reports that the result is very large. The output shows the first several \
digits and the last several digits, with the number in between indicating the \
number of digits that were omitted. This does not imply that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has not computed the entire value, it only means that displaying the \
integer would require excessive space."
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " has computed and stored the exact value and we can use this value in \
further computations. For example, we can find the last three digits of the \
number by computing the result modulo 1000."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Mod", "[", 
  RowBox[{"DNAsequences", ",", "1000"}], "]"}]], "Input"],

Cell[BoxData["376"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can determine the number of digits in the result by applying the ",
 StyleBox[ButtonBox["IntegerLength",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerLength"], "FunctionLink"],
 " function. This function is slightly more efficient than computing the base \
10 logarithm of the integer. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerLength", "[", "DNAsequences", "]"}]], "Input"],

Cell[BoxData["60206000"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Remember that the approximation ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["4", 
    SuperscriptBox["10", "8"]], TraditionalForm]]],
 " was a lower bound. In other words, the minimum number of possible \
sequences of bases in the DNA of a complex organism has over 60 million \
digits. "
}], "Text"],

Cell["\<\
Suppose you wanted to print this number. Using a typical fixed-width 12-point \
font and 1-inch margins, you can fit about 64 digits in each line and 45 \
lines on a page. With these parameters, it would require \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{
   RowBox[{"IntegerLength", "[", "DNAsequences", "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"64", "*", "45"}], ")"}]}], "]"}]], "Input"],

Cell[BoxData["20904.86111111111`"], "Output"]
}, Open  ]],

Cell["pages to print the entire number.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Symbol Names in Mathematica", "Subsubsection"],

Cell["\<\
Example 15 in Section 6.1 of the text calculated that in one version of the \
programming language BASIC, there were 957 different names for variables. \
\>", "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is extremely flexible with regards to symbol names. For simplicity, we \
will say that a symbol consists of a letter possibly followed by additional \
letters or numbers. ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " considers uppercase distinct from lowercase. There are 52 uppercase or \
lowercase letters that can be used as the first character. With the ten \
digits included, there are 62 possibilities for each character following the \
first. "
}], "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not impose a maximum length on variable names. For this example, \
however, we will consider only variable names up to a reasonable length of 15 \
characters. "
}], "Text"],

Cell[TextData[{
 "How many possible symbols are there? We need to compute the sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     StyleBox[
      RowBox[{"i", "=", "0"}], "TI"], "14",
     LimitsPositioning->False], 
    RowBox[{"52", "\[CenterDot]", 
     SuperscriptBox["62", "i"]}]}], TraditionalForm]]],
 ". Apply the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function to calculate this value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"52", "*", 
    RowBox[{"62", "^", "i"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "14"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["655464010775997815815360444"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We see that even limiting ourselves to a maximum of 15 characters, there \
are over 650 septillion distinct ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " symbols. (The number above is slightly inaccurate because it does not \
exclude ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " keywords and other protected symbols that you are not allowed to assign \
values to. Of course, those are relatively insignificant.)"
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Counting Operations in a Function", "Subsection",
 CellTags->"Section1c"],

Cell[TextData[{
 "Next we'll consider an example of counting the number of operations \
performed by a function. In Example 9 in Section 6.1 of the textbook, it is \
shown that the number of times that the innermost statement in a nested ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop is executed is the product of the number of iterations of each loop. \
"
}], "Text"],

Cell[TextData[{
 "As an example of this, we'll consider the ",
 StyleBox["makePostage", "Input"],
 " function from Section 5.1 of this manual. Recall that the purpose of this \
function was to determine the numbers of stamps of two given denominations \
that are required to make a given amount of postage. Here is the definition \
again."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"makePostage", "[", 
   RowBox[{"stampA_Integer", ",", "stampB_Integer", ",", "postage_Integer"}], 
   "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"a", ",", "b"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Catch", "[", "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"a", "=", "0"}], ",", 
       RowBox[{"a", "\[LessEqual]", 
        RowBox[{"Floor", "[", 
         RowBox[{"postage", "/", "stampA"}], "]"}]}], ",", 
       RowBox[{"a", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"b", "=", "0"}], ",", 
         RowBox[{"b", "\[LessEqual]", 
          RowBox[{"Floor", "[", 
           RowBox[{"postage", "/", "stampB"}], "]"}]}], ",", 
         RowBox[{"b", "++"}], ",", "\[IndentingNewLine]", 
         RowBox[{"If", "[", 
          RowBox[{
           RowBox[{
            RowBox[{
             RowBox[{"stampA", "*", "a"}], "+", 
             RowBox[{"stampB", "*", "b"}]}], "\[Equal]", "postage"}], ",", 
           RowBox[{"Throw", "[", 
            RowBox[{"{", 
             RowBox[{"a", ",", "b"}], "}"}], "]"}]}], "]"}]}], 
        "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "We will count the number of multiplications and additions that this \
function requires in the worst case. The ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 "/",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " pair means that once the function has found a way to make the desired \
amount of postage, execution is immediately terminated. This means that \
knowing the number of iterations used for a particular input value is \
equivalent to knowing the output of the function. If there was a formula for \
that, we wouldn't need the function. By considering the worst-case scenario, \
we can get an idea of the complexity of the algorithm without having to \
execute the function."
}], "Text"],

Cell[TextData[{
 "The worst-case scenario, that is, the situation that requires the most \
number of iterations of the loop, occurs when the desired postage cannot be \
made. In this case, the outer loop variable will range from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftFloor]", 
    FractionBox["postage", "stampA"], "\[RightFloor]"}], TraditionalForm]]],
 " and the inner loop will range from 0 to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftFloor]", 
    FractionBox["postage", "stampB"], "\[RightFloor]"}], TraditionalForm]]],
 ". Thus the number of times the if statement is executed is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[LeftFloor]", 
       FractionBox["postage", "stampA"], "\[RightFloor]"}], "+", "1"}], ")"}], 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"\[LeftFloor]", 
       FractionBox["postage", "stampB"], "\[RightFloor]"}], "+", "1"}], 
     ")"}]}], TraditionalForm]]],
 ". Therefore, in the worst case, the ",
 StyleBox["makePostage", "Input"],
 " function requires that number of additions and twice as many \
multiplications."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Counting by Listing All Possibilities", "Subsection",
 CellTags->"Section1d"],

Cell[TextData[{
 "At the end of Section 6.1, the text discusses using tree diagrams to solve \
counting problems. Tree diagrams provide a visual way to organize information \
so you can be sure that you arrive at all possible results. We will not, in \
this section, implement trees, as that is the focus of Chapter 11. The goal \
of a tree diagram is to list all of the possibilities. In this subsection we \
will consider two problems that can be solved by using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions to list all possibilities."
}], "Text"],

Cell[CellGroupData[{

Cell["Subsets", "Subsubsection"],

Cell["\<\
For the first example, we consider the following question: how many subsets \
of the set of the integers 1 through 10 have sums less than 15? (This is \
similar to Exercise 67 in Section 6.1.) \
\>", "Text"],

Cell["\<\
To solve this problem, we'll consider all of the possible subsets and count \
those that satisfy the condition. \
\>", "Text"],

Cell[TextData[{
 "In order to generate all of the possible subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 ", we'll use the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function, first introduced in Section 2.1 of this manual."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function accepts a list (representing a set) as an argument and produces \
the list of all subsets. Here is an example of ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " applied to the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We will use ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " to solve the problem of counting the number of subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 " whose sum is less than 15. Instead of displaying all subsets, we'll \
instead test their sum using the ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 " function."
}], "Text"],

Cell[TextData[{
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 ", when applied to a list and a Boolean-valued function, returns the sublist \
consisting of all elements of the original list that cause the function to \
return ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 ". In our case, the first argument will be the output from ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 ". The second argument will be the pure function that adds the elements in \
the subset and compares the result to the target of 15. "
}], "Text"],

Cell[TextData[{
 "To add the elements of a list, we will ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") the addition operator ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 " (",
 StyleBox["+", "Input"],
 ") to the set, as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apply", "[", 
  RowBox[{"Plus", ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["10"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " function has the effect of replacing the head of the second argument with \
the first argument. In the example above, the expression ",
 Cell[BoxData[
  RowBox[{"{", "1", ",", "2", ",", "3", ",", "4", "}"}]], "myCode"],
 " has head ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " replaces ",
 StyleBox[ButtonBox["List",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/List"], "FunctionLink"],
 " with ",
 StyleBox[ButtonBox["Plus",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Plus"], "FunctionLink"],
 ", transforming the expression into ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"Plus", "["}], "1", ",", "2", ",", "3", ",", "4", "]"}]], "myCode"],
 ", which is then evaluated and outputs the sum. ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " can be used any time you need to use the elements of a list as the \
arguments to a function."
}], "Text"],

Cell["\<\
To compare the sum with 15, we just need to add the inequality to the \
expression.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Apply", "[", 
   RowBox[{"Plus", ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]}], "]"}], "<", 
  "15"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "And to turn this example into a pure function, we replace the specific set \
with a ",
 StyleBox[ButtonBox["Slot",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Slot"], "FunctionLink"],
 " (",
 StyleBox["#", "Input"],
 ") and put an ampersand at the end to indicate its status as a ",
 StyleBox[ButtonBox["Function",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Function"], "FunctionLink"],
 " (&)."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Apply", "[", 
    RowBox[{"Plus", ",", "#"}], "]"}], "<", "15"}], "&"}]], "myCode"],

Cell[TextData[{
 "With this as the second argument to ",
 StyleBox[ButtonBox["Select",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Select"], "FunctionLink"],
 ", and an application of ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " as the first argument, we obtain the list of all subsets with the \
specified sum. As an example, we list the subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 " whose sum is less than 5."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Select", "[", 
  RowBox[{
   RowBox[{"Subsets", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "3"}], "}"}], "]"}], ",", 
   RowBox[{
    RowBox[{
     RowBox[{"Apply", "[", 
      RowBox[{"Plus", ",", "#"}], "]"}], "<", "5"}], "&"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The original question was to count the number of subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 " whose sum is at most 15. So all that remains is to apply ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " in order to get a count of the number of subsets. We can generalize a bit \
and create a function that accepts a set of integers and a target value and \
counts the number of subsets of the given set whose elements sum to a value \
less than the target."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"subsetSumCount", "[", 
   RowBox[{
    RowBox[{"S", ":", 
     RowBox[{"{", "__Integer", "}"}]}], ",", "target_Integer"}], "]"}], ":=", 
  
  RowBox[{"Length", "[", 
   RowBox[{"Select", "[", 
    RowBox[{
     RowBox[{"Subsets", "[", "S", "]"}], ",", 
     RowBox[{
      RowBox[{
       RowBox[{"Apply", "[", 
        RowBox[{"Plus", ",", "#"}], "]"}], "<", "target"}], "&"}]}], "]"}], 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying this function to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 " and 15 will answer the original question."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetSumCount", "[", 
  RowBox[{
   RowBox[{"Range", "[", "10", "]"}], ",", "15"}], "]"}]], "Input"],

Cell[BoxData["99"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Bit Strings", "Subsubsection"],

Cell["\<\
For the second example, we'll consider a problem similar to Example 21. How \
many bit strings of length 10 do not have three consecutive ones? \
\>", "Text"],

Cell["\<\
We could use an approach similar to the previous example and produce all bit \
strings of length 10 and then count the number that do not contain three \
consecutive 1s. However, the solution to Example 21 of Section 6.1, and \
especially Figure 2, suggests a more efficient solution. Instead of creating \
all the possible bit strings, we can build them in such a way as to only \
create those that satisfy the limitation on the number of consecutive 1s. \
\>", "Text"],

Cell["\<\
To solve this problem, we will use a recursive algorithm. The basis step will \
be the set consisting of all bit strings of length 2 (these cannot have three \
consecutive 1s). In the recursive step, the new version of the set will be \
constructed as follows. For each bit string in the previous set, we will \
append a 0. And we will append a 1 to all of the bit strings whose last two \
bits are not both 1. \
\>", "Text"],

Cell["\<\
For this problem, we will model bit strings as lists of 0s and 1s. The \
algorithm described above will be implemented as two functions. The first \
function will be responsible for extending a particular bit string. That is, \
given a bit string (a list of 0s and 1s), it will return either the two bit \
strings obtained by appending a 0 and by appending a 1, or it will return the \
single bit string formed by appending a 0 if appending a 1 would result in \
three consecutive 1s. The second function will apply the first to build the \
entire set of admissable bit strings.\
\>", "Text"],

Cell["\<\
First we implement the function that extends a single bit string. The \
parameter to this function will be a bit string, that is a list of 0s and 1s. \
The function will first create a new bit string by appending a zero to the \
input.\
\>", "Text"],

Cell[TextData[{
 "Then the function will test the last two elements of the original bit \
string to determine if they are both ones. We will accomplish this test by \
extracting the last two entries with the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator. We use the fact that negative integers indicate the position \
from the end of the list: ",
 Cell["L[[-1]]", "myCode"],
 " is the last element,",
 " ",
 Cell["L[[-2]]", "myCode"],
 " is the next to last element, and so on. We use a ",
 StyleBox[ButtonBox["Span",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Span"], "FunctionLink"],
 " (",
 StyleBox[";;", "Input"],
 "), i.e., ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"L", "[", 
    RowBox[{"[", 
     RowBox[{"-", "2"}]}]}], ";;", 
   RowBox[{
    RowBox[{
     RowBox[{"-", "1"}], "]"}], "]"}]}]], "myCode"],
 ", so that we obtain the sublist of the last two elements. We can then \
compare the result against the list ",
 Cell[BoxData[
  RowBox[{"{", "1", ",", "1", "}"}]], "myCode"],
 ". If those lists are equal, that is, the last two bits are both 1, then the \
function only returns the list obtained by adding 0. Otherwise, it will \
create the list with 1 added and return both extended bit strings as a \
sequence."
}], "Text"],

Cell["Here is the implementation.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"addBit", "[", "L_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"L0", ",", "L1"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"L0", "=", 
      RowBox[{"Append", "[", 
       RowBox[{"L", ",", "0"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"L", "[", 
         RowBox[{"[", 
          RowBox[{
           RowBox[{"-", "2"}], ";;", 
           RowBox[{"-", "1"}]}], "]"}], "]"}], "\[Equal]", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}]}], ",", 
       RowBox[{"Return", "[", 
        RowBox[{"{", "L0", "}"}], "]"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"L1", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"L", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Return", "[", 
         RowBox[{"{", 
          RowBox[{"L0", ",", "L1"}], "}"}], "]"}]}]}], "\[IndentingNewLine]", 
      "]"}]}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Let's test this function: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], TraditionalForm]]],
 " should produce only ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
   TraditionalForm]]],
 ", while applying the function to that result should produce ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "0"}], "}"}], 
   TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"addBit", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "1", ",", "1"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"addBit", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "0"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "0", ",", "1", ",", "1", ",", "0", ",", "1"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now we write the main function. It will accept as input a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " representing the length of the bit strings to be output. In case this \
value is 2, the output is the four bit strings of length 2. For values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " larger than 2, it will recursively call itself on ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " and store the result of the recursion as ",
 StyleBox["S", "Input"],
 ". It then initializes a new list ",
 StyleBox["T", "Input"],
 " to the empty set. Finally, it loops through the set ",
 StyleBox["S", "Input"],
 ", applying ",
 StyleBox["addBit", "Input"],
 " to each member and adding the result to ",
 StyleBox["T", "Input"],
 "."
}], "Text"],

Cell["Here is the implementation.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findBitStrings", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"S", ",", "s", ",", 
      RowBox[{"T", "=", 
       RowBox[{"{", "}"}]}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"n", "\[Equal]", "2"}], ",", "\[IndentingNewLine]", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"0", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"0", ",", "1"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "0"}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"1", ",", "1"}], "}"}]}], "}"}], ",", "\[IndentingNewLine]", 
      
      RowBox[{
       RowBox[{"S", "=", 
        RowBox[{"findBitStrings", "[", 
         RowBox[{"n", "-", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"Do", "[", 
        RowBox[{
         RowBox[{"T", "=", 
          RowBox[{"Join", "[", 
           RowBox[{"T", ",", 
            RowBox[{"addBit", "[", "s", "]"}]}], "]"}]}], ",", 
         RowBox[{"{", 
          RowBox[{"s", ",", "S"}], "}"}]}], "]"}], ";", "\[IndentingNewLine]",
        "T"}]}], "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Applying the function to 10 and using ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " on the output will give us the number of bit strings of length 10 that do \
not include three successive 1s."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"findBitStrings", "[", "10", "]"}], "]"}]], "Input"],

Cell[BoxData["504"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.2 The Pigeonhole Principle", "Section",
 CellTags->"Section2"],

Cell[TextData[{
 "In this section we will see how ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can be used to help explore two problems related to the pigeonhole \
principle: finding consecutive entries in a sequence with a given sum, and \
finding increasing and decreasing subsequences."
}], "Text"],

Cell[TextData[{
 "Before considering those two problems, however, recall the ",
 StyleBox[ButtonBox["Ceiling",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Ceiling"], "FunctionLink"],
 " function. This function calculates the ceiling of an expression. For \
example, the solution to Example 8 in the text indicates that the minimum \
number of area codes needed to assign different phone numbers to 25 million \
phones is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[LeftCeiling]", 
    FractionBox["25000000", "8000000"], "\[RightCeiling]"}], 
   TraditionalForm]]],
 ". In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", this is computed by the following expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ceiling", "[", 
  RowBox[{"25000000", "/", "8000000"}], "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Consecutive Entries with a Given Sum", "Subsection",
 CellTags->"Section2a"],

Cell["\<\
Example 10 in Section 6.2 describes the solution to the following problem. In \
a month with 30 days, a baseball team plays at least one game per day but at \
most 45 games during the month. There then must be a period of consecutive \
days during which the team plays exactly 14 games.\
\>", "Text"],

Cell[TextData[{
 "The problem can be generalized. Given a sequence of ",
 Cell[BoxData[
  FormBox["d", TraditionalForm]]],
 " positive integers whose sum is at most ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 ", there must be a consecutive subsequence with sum ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 " for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"T", "<", 
    RowBox[{"S", "-", "d"}]}], TraditionalForm]]],
 ". We leave it to the reader to prove this assertion."
}], "Text"],

Cell[TextData[{
 "We will write a function that, given a sequence and target sum ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ", will find the consecutive terms whose sum is ",
 Cell[BoxData[
  FormBox["T", TraditionalForm]]],
 ". Our solution will be based on the approach used in Example 10."
}], "Text"],

Cell[TextData[{
 "First, we will calculate the numbers ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "d"]}], TraditionalForm]]],
 " with each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "i"], TraditionalForm]]],
 " equal to the sum of the first ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " terms in the sequence. These values will be stored as a list, ",
 StyleBox["A", "Input"],
 ". We will calculate these sums using the observation that each one is equal \
to the previous sum plus the next entry in the sequence."
}], "Text"],

Cell[TextData[{
 "Then we will calculate ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "i"], "+", "T"}], TraditionalForm]]],
 " for each ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " and use the ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " functions to check to see if this value is in ",
 StyleBox["A", "Input"],
 ". Both functions requires two arguments: the first is the list to be \
searched and the second is the element being sought. ",
 StyleBox[ButtonBox["MemberQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/MemberQ"], "FunctionLink"],
 " returns ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " or ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " depending on whether the object is or is not a member of the list. The ",
 StyleBox[ButtonBox["Position",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Position"], "FunctionLink"],
 " function returns a list of position specifications. Consider the example \
below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<b\>\"", ",", "\"\<a\>\"", ",", "\"\<a\>\"", ",", "\"\<c\>\""}], 
    "}"}], ",", "\"\<a\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "6", "}"}], ",", 
   RowBox[{"{", "7", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the result is a list of three sublists. Each element of the main \
list indicates the position at which an \[OpenCurlyDoubleQuote]a\
\[CloseCurlyDoubleQuote] occurs, namely positions 1, 6, and 7. The reason \
that the location specifications are enclosed in their own lists is for the \
case when the desired element is found within sublists of the original list. \
For example, in the example below, the character \[OpenCurlyDoubleQuote]c\
\[CloseCurlyDoubleQuote] is found both in the third element of the sublist at \
position 2 as well as in position 4.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Position", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", 
     RowBox[{"{", 
      RowBox[{
      "\"\<b\>\"", ",", "\"\<a\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
      "}"}], ",", "\"\<d\>\"", ",", "\"\<c\>\"", ",", "\"\<a\>\""}], "}"}], 
   ",", "\"\<c\>\""}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", "4", "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Our list ",
 StyleBox["A", "Input"],
 " will be flat, that is, with no sublists. Also, we are only interested in \
finding the first match, since that will tell us the location of a sublist \
with the desired sum. To find the position of the first match, we use the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") specification ",
 StyleBox["[[1,1]]", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Finally, if ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "i"], "+", "T"}], TraditionalForm]]],
 " is found in the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], ",", 
    SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "d"]}], TraditionalForm]]],
 ", say at position ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 ", then we know that ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 " through ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " are the positions of the consecutive subsequence with the desired sum. The \
function will output the starting and ending positions as well as the \
subsequence."
}], "Text"],

Cell[TextData[{
 "Here is the function. Note that the first line uses the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to update the value of ",
 StyleBox["a", "Input"],
 ", which serves as the sum of the list elements, at the same time it \
populates the list ",
 StyleBox["A", "Input"],
 " with those values."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findSubSum", "[", 
   RowBox[{
    RowBox[{"L", ":", 
     RowBox[{"{", "__Integer", "}"}]}], ",", "T_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"A", ",", "i", ",", "j", ",", 
      RowBox[{"a", "=", "0"}], ",", "p"}], "}"}], ",", "\[IndentingNewLine]", 
    
    RowBox[{
     RowBox[{"A", "=", 
      RowBox[{"Table", "[", 
       RowBox[{
        RowBox[{"a", "=", 
         RowBox[{"a", "+", 
          RowBox[{"L", "[", 
           RowBox[{"[", "i", "]"}], "]"}]}]}], ",", 
        RowBox[{"{", 
         RowBox[{"i", ",", 
          RowBox[{"Length", "[", "L", "]"}]}], "}"}]}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Catch", "[", "\[IndentingNewLine]", 
      RowBox[{"For", "[", 
       RowBox[{
        RowBox[{"i", "=", "1"}], ",", 
        RowBox[{"i", "\[LessEqual]", 
         RowBox[{"Length", "[", "L", "]"}]}], ",", 
        RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"MemberQ", "[", 
           RowBox[{"A", ",", 
            RowBox[{
             RowBox[{"A", "[", 
              RowBox[{"[", "i", "]"}], "]"}], "+", "T"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"j", "=", 
            RowBox[{
             RowBox[{"Position", "[", 
              RowBox[{"A", ",", 
               RowBox[{
                RowBox[{"A", "[", 
                 RowBox[{"[", "i", "]"}], "]"}], "+", "T"}]}], "]"}], "[", 
             RowBox[{"[", 
              RowBox[{"1", ",", "1"}], "]"}], "]"}]}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Throw", "[", 
            RowBox[{"{", 
             RowBox[{"i", ",", "j", ",", 
              RowBox[{"L", "[", 
               RowBox[{"[", 
                RowBox[{"i", ";;", "j"}], "]"}], "]"}]}], "}"}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}], 
      "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We apply the function to the following sequence, representing the number of \
games a baseball team played on each day of a 30-day month: \
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
  "2", ",", "1", ",", "3", ",", "1", ",", "1", ",", "3", ",", "1", ",", "1", 
   ",", "1", ",", "1", ",", "3", ",", "1", ",", "3", ",", "1", ",", "1", ",", 
   "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "3", 
   ",", "1", ",", "1", ",", "3", ",", "1", ",", "3", ",", "1", ",", "1"}], 
  TraditionalForm]], "Equation"],

Cell["\<\
As in Example 10, we'll find the consecutive days during which the team \
played 14 games. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findSubSum", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "2", ",", "1", ",", "3", ",", "1", ",", "1", ",", "3", ",", "1", ",", "1",
      ",", "1", ",", "1", ",", "3", ",", "1", ",", "3", ",", "1", ",", "1", 
     ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", 
     ",", "3", ",", "1", ",", "1", ",", "3", ",", "1", ",", "3", ",", "1", 
     ",", "1"}], "}"}], ",", "14"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "13", ",", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "3", ",", "1", ",", "1", ",", "1", ",", "1", ",", "3", ",", "1",
      ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Strictly Increasing Subsequences", "Subsection",
 CellTags->"Section2b"],

Cell[TextData[{
 "Theorem 3 of Section 6.2 asserts that every sequence of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " distinct real numbers contains a subsequence of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " that is either strictly increasing or strictly decreasing. We will develop \
a function that will find a longest strictly increasing subsequence."
}], "Text"],

Cell[CellGroupData[{

Cell["The Patience Algorithm", "Subsubsection"],

Cell["\<\
To find the longest increasing subsequence, we will use a greedy strategy \
based on \[OpenCurlyDoubleQuote]Patience sorting\[CloseCurlyDoubleQuote] (the \
name refers to the solitaire card game also called Klondike). The idea is as \
follows. Imagine that the numbers in the sequence are written on cards. The \
cards are placed in a \[OpenCurlyDoubleQuote]deck\[CloseCurlyDoubleQuote] in \
the order they appear in the sequence and with the first element of the \
sequence on top. Now play a \[OpenCurlyDoubleQuote]game\
\[CloseCurlyDoubleQuote] using the deck of cards based on the following rules.\
\>", "Text"],

Cell["\<\
The cards are \[OpenCurlyDoubleQuote]dealt\[CloseCurlyDoubleQuote] one at a \
time onto a series of piles on the table. Initially there are no piles. The \
top card (the first element in the sequence) is the first card dealt and \
forms the first pile. To play the next card, check to see if it is less than \
or greater than the first card. If the second card (the second element of the \
sequence) is less than the first, then it is placed on the first pile, on top \
of the first card. If the second card is greater than the first, then it \
starts a new pile to the right of the first.\
\>", "Text"],

Cell["\<\
The \[OpenCurlyDoubleQuote]game\[CloseCurlyDoubleQuote] continues in this \
way. At each step, the table has on it a series of piles. To play the next \
card, you compare the value on the card to the card on top of the first pile. \
If the card to be played has a value smaller than the number showing on the \
first pile, then the new card is placed on top of the first pile. Otherwise, \
you look at the second pile. If the card being played is smaller than the \
value on the second pile, it is placed on top of the second pile. Continue in \
this fashion until either the card has been played or, if it is larger than \
the top card on every existing pile, then it begins a new pile to the right \
of all the others.\
\>", "Text"],

Cell["\<\
An illustration is in order. Consider the sequence 12, 18, 7, 11, 16, 3, 20, \
17. \
\>", "Text"],

Cell[TextData[Cell[BoxData[
 RowBox[{GridBox[{
    {Cell["Step 1: Play the first entry, 12, as the first pile"], 
     TagBox[GridBox[{
        {
         ItemBox["12",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], "", "", ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 2: Play the second entry, 18. Since ",
      Cell[BoxData[
       FormBox[
        RowBox[{"18", ">", "12"}], TraditionalForm]]],
      ", 18 starts a new pile."
     }]], 
     TagBox[GridBox[{
        {"12", 
         ItemBox["18",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], "", ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 3: Play 7. Checking the first pile, note that ",
      Cell[BoxData[
       FormBox[
        RowBox[{"7", "<", "12"}], TraditionalForm]]],
      ", so 7 is played on the first pile."
     }]], 
     TagBox[GridBox[{
        {
         ItemBox["7",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], "", "", ""},
        {"12", "18", "", ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 4: Play 11. Checking the first pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"11", ">", "7"}], TraditionalForm]]],
      ", so do not play 11 on first pile. Checking the second pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"11", "<", "18"}], TraditionalForm]]],
      ", so play 11 on the second pile."
     }]], 
     TagBox[GridBox[{
        {"7", 
         ItemBox["11",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], "", ""},
        {"12", "18", "", ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 5: Play 16. Checking the first pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"16", ">", "7"}], TraditionalForm]]],
      " so do not play 16 on the first pile. Checking the second pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"16", ">", "11"}], TraditionalForm]]],
      ", so 16 begins a third pile."
     }]], 
     TagBox[GridBox[{
        {"7", "11", "", ""},
        {"12", "18", 
         ItemBox["16",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 6: Play 3. Checking the first pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"3", "<", "7"}], TraditionalForm]]],
      ", so 3 is played on the first pile."
     }]], 
     TagBox[GridBox[{
        {
         ItemBox["3",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False], "", "", ""},
        {"7", "11", "", ""},
        {"12", "18", "16", ""}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 7: Play 20. Checking the first pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"20", ">", "3"}], TraditionalForm]]],
      ". Checking the second pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"20", ">", "11"}], TraditionalForm]]],
      ". Checking the third pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"20", ">", "16"}], TraditionalForm]]],
      ", so 20 starts a new pile."
     }]], 
     TagBox[GridBox[{
        {"3", "", "", ""},
        {"7", "11", "", ""},
        {"12", "18", "16", 
         ItemBox["20",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False]}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]},
    {Cell[TextData[{
      "Step 8: Play 17. Checking the first pile, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"17", ">", "3"}], TraditionalForm]]],
      ". Checking the second, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"17", ">", "11"}], TraditionalForm]]],
      ". Checking the third, ",
      Cell[BoxData[
       FormBox[
        RowBox[{"17", ">", "16"}], TraditionalForm]]],
      ", but ",
      Cell[BoxData[
       FormBox[
        RowBox[{"17", "<", "20"}], TraditionalForm]]],
      ", so 17 is played on the fourth pile."
     }]], 
     TagBox[GridBox[{
        {"3", "", "", ""},
        {"7", "11", "", 
         ItemBox["17",
          Background->RGBColor[0.8, 0.6, 0.8],
          StripOnInput->False]},
        {"12", "18", "16", "20"}
       },
       AutoDelete->False,
       FrameStyle->GrayLevel[1],
       GridBoxFrame->{"Columns" -> {{True}}, "Rows" -> {{True}}},
       GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
       
       GridBoxSpacings->{"Columns" -> {{3}}, "Rows" -> {{1}}}],
      "Grid"]}
   },
   BaseStyle->{FontFamily -> "Times"},
   GridBoxAlignment->{"Columns" -> {{Left}}, "Rows" -> {{Top}}},
   GridBoxDividers->{
    "Columns" -> {{True}}, "ColumnsIndexed" -> {}, "Rows" -> {{True}}, 
     "RowsIndexed" -> {}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}], 
  "\[NoBreak]"}]]]], "Grid"],

Cell["\<\
Once the \[OpenCurlyDoubleQuote]game\[CloseCurlyDoubleQuote] is complete, the \
length of the longest strictly increasing subsequence is equal to the number \
of piles.\
\>", "Text"],

Cell["\<\
Now that all of the cards are played, we obtain a strictly increasing \
subsequence by backtracking. The top card on the final pile is 17, so 17 will \
be the last entry in the subsequence. When 17 was placed on the pile, the top \
card on the pile before it was 16 (step 8), so 16 precedes 17 in the \
subsequence. When 16 was placed on the third pile, the top card on the second \
pile was 11 (step 5), so 11 precedes 16. And when 11 was placed on the second \
pile, the top card on the first pile was 7, so 7 is first in the subsequence. \
Thus, 7, 11, 16, 17 is a strictly increasing subsequence of maximal length.\
\>", "Text"],

Cell[TextData[{
 "You are encouraged to \[OpenCurlyDoubleQuote]play\[CloseCurlyDoubleQuote] \
through this approach a few times with your own sequences to ensure that you \
understand the process before continuing on to the implementation of the \
procedure below. You can apply the ",
 StyleBox["findIncreasing", "Input"],
 " function defined below to make sure that you are arriving at the same \
result. Be sure to keep track of what was on top of the previous pile when \
each number is played, as you need that information in the backtracking stage."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Implementing the Algorithm", "Subsubsection"],

Cell["\<\
Now we will implement the Patience algorithm. The given sequence will be \
input to the function as a list. Within the function, we need to track three \
kinds of information. \
\>", "Text"],

Cell[TextData[{
 "First, we need to know which \[OpenCurlyDoubleQuote]step\
\[CloseCurlyDoubleQuote] we're in, that is, which card is being played. This \
will be represented by the variable to a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop ranging from 1 to the size of the list."
}], "Text"],

Cell[TextData[{
 "Second, we need to know what cards are on the piles. Specifically, we need \
to know the top card of each pile. This will be represented as a list, ",
 StyleBox["piles", "Input"],
 ". When a card is placed on top of an existing pile, we can replace the \
current value in that position with the syntax ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"piles", "[", 
    RowBox[{"[", "i", "]"}], "]"}], "=", "x"}]], "myCode"],
 ". When a new pile is added to the list, we extend the list via ",
 StyleBox[ButtonBox["AppendTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/AppendTo"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "Third, in order to backtrack and recover the longest increasing sequence, \
we need to store, for each member of the sequence, the value that was on the \
top of the pile to the left of the entry's pile. For this, we will use a \
indexed variable, ",
 StyleBox["pointers", "Input"],
 ", whose indices will be the members of the sequence and whose values will \
be set to the previous pile's top card. (We use the name ",
 StyleBox["pointers", "Input"],
 " for this variable because of the similarity to the \
\[OpenCurlyDoubleQuote]linked list\[CloseCurlyDoubleQuote] structure used in \
many programming languages.) For those numbers played in the first pile, the \
value in the variable will be set to ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ". Note that since pointers will be an indexed variable, rather than a list, \
we use a single pair of brackets around its indices, not the double brackets \
used with ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") and lists."
}], "Text"],

Cell[TextData[{
 "The algorithm will consist of two stages. The first stage will be the game \
stage. We begin with a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop with loop variable ",
 StyleBox["step", "Input"],
 " running from 1 to the length of the input sequence ",
 StyleBox["S", "Input"],
 ". Within this main loop, two tasks are performed."
}], "Text"],

Cell[TextData[{
 "The first thing that happens within the ",
 StyleBox["step", "Input"],
 " loop is determining on which pile to play the current card. Note that the \
value of the current card is accessed by ",
 StyleBox["S[[step]]", "Input"],
 ". To determine the proper location for the current card, we do the \
following."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "Open a ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 " block. A variable ",
 StyleBox["whichpile", "Input"],
 " is set to the output of this ",
 StyleBox[ButtonBox["Catch",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Catch"], "FunctionLink"],
 "."
}], "Item1Numbered"],

Cell[TextData[{
 "Use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop from 1 to the current number of piles. Within the loop, compare the \
current card to the top of each pile. If the current card is smaller than the \
value in a pile, ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " that pile index, causing the loop to be short-circuited and assigning that \
value to ",
 StyleBox["whichpile", "Input"],
 ". If the ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop terminates, ",
 StyleBox[ButtonBox["Throw",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Throw"], "FunctionLink"],
 " ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 " to indicate that the card cannot be placed on any existing pile."
}], "Item1Numbered"],

Cell[TextData[{
 "Next, check the value of ",
 StyleBox["whichpile", "Input"],
 ". If it is ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", that means a pile was not found for the current card, and thus the ",
 StyleBox["piles", "Input"],
 " list must be extended to create a new pile for this card. Otherwise, the \
value of ",
 StyleBox["whichpile", "Input"],
 " is the appropriate pile and we update the corresponding entry in ",
 StyleBox["piles", "Input"],
 " to indicate that the latest card is placed on top in that position."
}], "Item1Numbered"]
}, Open  ]],

Cell[TextData[{
 "The second task within the ",
 StyleBox["step", "Input"],
 " loop is to update the ",
 StyleBox["pointers", "Input"],
 " variable. This also depends on the value of ",
 StyleBox["whichpile", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[TextData[{
 "If ",
 StyleBox["whichpile", "Input"],
 " is 1 or if ",
 StyleBox["piles", "Input"],
 " only contains 1 entry, then the latest card was played on the first pile \
and the associated value should be ",
 StyleBox["Null", "Input"],
 "."
}], "Item1"],

Cell[TextData[{
 "If ",
 StyleBox["whichpile", "Input"],
 " is ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", then the value associated with the latest card is ",
 StyleBox["piles[[-2]]", "Input"],
 ", the top card on what had been the last pile but is now the next to last \
pile. (Note that the previous condition, that ",
 StyleBox["whichpile", "Input"],
 " is 1 or ",
 StyleBox["piles", "Input"],
 " has only 1 entry will ensure that this second condition is only tested if ",
 StyleBox["piles", "Input"],
 " has at least 2 entries and thus ",
 StyleBox["piles[[-2]]", "Input"],
 " is a valid selection.)"
}], "Item1"],

Cell[TextData[{
 "Otherwise, the value is ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"piles", "[", "["}], "whichpile", "-", "1", 
   RowBox[{"]", "]"}]}]], "myCode"],
 "."
}], "Item1"]
}, Open  ]],

Cell[TextData[{
 "That concludes the game stage. The second stage is the backtracking stage, \
which is much simpler. First, access the top card of the last pile with ",
 StyleBox["piles[[-1]]", "Input"],
 ", and initialize the maximal increasing list, ",
 StyleBox["iList", "Input"],
 ", to the list consisting of this value."
}], "Text"],

Cell[TextData[{
 "Then we extend ",
 StyleBox["iList", "Input"],
 " on the left with the entry in the ",
 StyleBox["pointers", "Input"],
 " variable associated to that value. Since we\[CloseCurlyQuote]re building \
the list from right to left, ",
 StyleBox["iList[[1]]", "Input"],
 " always contains the most recently added number. So ",
 StyleBox["pointers[iList[[1]]]", "Input"],
 " is the new value, which is added via ",
 StyleBox[ButtonBox["PrependTo",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PrependTo"], "FunctionLink"],
 ". Since the cards played in the first pile were associated with ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 ", we can use a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop with condition ",
 Cell[BoxData[
  RowBox[{
   RowBox[{"pointers", "[", 
    RowBox[{"iList", "[", 
     RowBox[{"[", "1", "]"}], "]"}]}], "=!=", "Null"}]], "myCode"],
 " to fill the ",
 StyleBox["iList", "Input"],
 ". At the conclusion of the loop, the function returns ",
 StyleBox["iList", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "Note throughout that whenever making a comparison which may be a comparison \
between an integer and the symbol Null, you must use ",
 StyleBox[ButtonBox["SameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SameQ"], "FunctionLink"],
 " (",
 StyleBox["=\[VeryThinSpace]=\[VeryThinSpace]=", "Input"],
 ") or ",
 StyleBox[ButtonBox["UnsameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/UnsameQ"], "FunctionLink"],
 " (",
 StyleBox["=!=", "Input"],
 "), not ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["=\[VeryThinSpace]=", "Input"],
 ") and ",
 StyleBox[ButtonBox["Unequal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Unequal"], "FunctionLink"],
 " (",
 StyleBox["!=", "Input"],
 ")."
}], "Text"],

Cell["Here, finally, is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"findIncreasing", "[", 
   RowBox[{"S", ":", 
    RowBox[{"{", "__Integer", "}"}]}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"piles", "=", 
       RowBox[{"{", "}"}]}], ",", "pointers", ",", "step", ",", "whichpile", 
      ",", "p", ",", "iList"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"(*", 
     RowBox[{"game", " ", "stage"}], "*)"}], "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"step", "=", "1"}], ",", 
       RowBox[{"step", "\[LessEqual]", 
        RowBox[{"Length", "[", "S", "]"}]}], ",", 
       RowBox[{"step", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{"(*", 
        RowBox[{"play", " ", "the", " ", "card"}], "*)"}], 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"whichpile", "=", 
         RowBox[{"Catch", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"For", "[", 
            RowBox[{
             RowBox[{"p", "=", "1"}], ",", 
             RowBox[{"p", "\[LessEqual]", 
              RowBox[{"Length", "[", "piles", "]"}]}], ",", 
             RowBox[{"p", "++"}], ",", "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{
                RowBox[{"S", "[", 
                 RowBox[{"[", "step", "]"}], "]"}], "<", 
                RowBox[{"piles", "[", 
                 RowBox[{"[", "p", "]"}], "]"}]}], ",", 
               RowBox[{"Throw", "[", "p", "]"}]}], "]"}]}], 
            "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
           RowBox[{"Throw", "[", "Null", "]"}]}], "\[IndentingNewLine]", 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"whichpile", "===", "Null"}], ",", "\[IndentingNewLine]", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"piles", ",", 
            RowBox[{"S", "[", 
             RowBox[{"[", "step", "]"}], "]"}]}], "]"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"piles", "[", 
            RowBox[{"[", "whichpile", "]"}], "]"}], "=", 
           RowBox[{"S", "[", 
            RowBox[{"[", "step", "]"}], "]"}]}]}], "\[IndentingNewLine]", 
         "]"}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", 
         RowBox[{"update", " ", "pointers"}], "*)"}], "\[IndentingNewLine]", 
        RowBox[{"Which", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"whichpile", "===", "1"}], "||", 
           RowBox[{
            RowBox[{"Length", "[", "piles", "]"}], "\[Equal]", "1"}]}], ",", 
          RowBox[{
           RowBox[{"pointers", "[", 
            RowBox[{"S", "[", 
             RowBox[{"[", "step", "]"}], "]"}], "]"}], "=", "Null"}], ",", 
          "\[IndentingNewLine]", 
          RowBox[{"whichpile", "===", "Null"}], ",", 
          RowBox[{
           RowBox[{"pointers", "[", 
            RowBox[{"S", "[", 
             RowBox[{"[", "step", "]"}], "]"}], "]"}], "=", 
           RowBox[{"piles", "[", 
            RowBox[{"[", 
             RowBox[{"-", "2"}], "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
          "True", ",", 
          RowBox[{
           RowBox[{"pointers", "[", 
            RowBox[{"S", "[", 
             RowBox[{"[", "step", "]"}], "]"}], "]"}], "=", 
           RowBox[{"piles", "[", 
            RowBox[{"[", 
             RowBox[{"whichpile", "-", "1"}], "]"}], "]"}]}]}], 
         "\[IndentingNewLine]", "]"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     
     RowBox[{"(*", 
      RowBox[{"ends", " ", "main", " ", "For", " ", "loop"}], "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"(*", 
      RowBox[{"backtracking", " ", "stage"}], "*)"}], "\[IndentingNewLine]", 
     RowBox[{"iList", "=", 
      RowBox[{"{", 
       RowBox[{"piles", "[", 
        RowBox[{"[", 
         RowBox[{"-", "1"}], "]"}], "]"}], "}"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"pointers", "[", 
         RowBox[{"iList", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "]"}], "=!=", "Null"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"PrependTo", "[", 
        RowBox[{"iList", ",", 
         RowBox[{"pointers", "[", 
          RowBox[{"iList", "[", 
           RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "iList"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Example 12 from the text involved the sequence ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "8", ",", "11", ",", "9", ",", "1", ",", "4", ",", "6", ",", "12", ",", 
    "10", ",", "5", ",", "7"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"findIncreasing", "[", 
  RowBox[{"{", 
   RowBox[{
   "8", ",", "11", ",", "9", ",", "1", ",", "4", ",", "6", ",", "12", ",", 
    "10", ",", "5", ",", "7"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "4", ",", "5", ",", "7"}], "}"}]], "Output"]
}, Open  ]],

Cell["This is one of the four sequences given in the text.", "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Connection to the Pigeonhole Principle", "Subsubsection"],

Cell[TextData[{
 "Recall that Theorem 3 asserted that every sequence of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " distinct real numbers contains a subsequence of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " that is either strictly increasing or strictly decreasing. It may appear \
that the Patience algorithm has no connection to this theorem or to the \
pigeonhole principle."
}], "Text"],

Cell[TextData[{
 "However, the Patience algorithm does in fact suggest a proof of Theorem 3 \
via the pigeonhole principle. When the Patience algorithm is executed on a \
list of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " distinct real numbers, either there are at least ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " stacks or there are at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " stacks. If there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " stacks, then there is a strictly increasing subsequence of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "On the other hand, assume that there are at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " stacks. Take the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " values to be the pigeons and the stacks the pigeonholes. When ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["n", "2"], "+", "1"}], TraditionalForm]]],
 " objects are placed in ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " boxes, then there is a box containing at least ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[LeftCeiling]", 
     FractionBox[
      RowBox[{
       SuperscriptBox["n", "2"], "+", "1"}], "n"], "\[RightCeiling]"}], "=", 
    RowBox[{
     RowBox[{"\[LeftCeiling]", 
      RowBox[{
       FractionBox[
        SuperscriptBox["n", "2"], "n"], "+", 
       FractionBox["1", "n"]}], "\[RightCeiling]"}], "=", 
     RowBox[{
      RowBox[{"n", "+", 
       RowBox[{"\[LeftCeiling]", 
        FractionBox["1", "n"], "\[RightCeiling]"}]}], "=", 
      RowBox[{"n", "+", "1"}]}]}]}], TraditionalForm]]],
 " objects (Theorem 2). Hence some stack has ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " values. But the rules of the game ensure that each stack is a strictly \
decreasing subsequence, since one value is placed on top of another in a \
stack only when the second value is lesser than, and appears in the sequence \
later than, the lower value."
}], "Text"],

Cell[TextData[{
 "In short, either there are ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " stacks and hence an increasing sequence of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " or there is a stack of size ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 " and hence a decreasing sequence of length ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "1"}], TraditionalForm]]],
 "."
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.3 Permutations and Combinations", "Section",
 CellTags->"Section3"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes many functions pertaining to counting and generating \
combinatorial structures."
}], "Text"],

Cell[CellGroupData[{

Cell["Permutations", "Subsection",
 CellTags->"Section3a"],

Cell["\<\
We begin by looking at commands related to permutations of objects. \
\>", "Text"],

Cell["\<\
We have seen in previous chapters the use of the exclamation mark for \
factorial.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"6", "!"}]], "Input"],

Cell[BoxData["720"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Factorial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Factorial"], "FunctionLink"],
 " (",
 StyleBox["!", "Input"],
 ") function can be used instead of the exclamation mark if you prefer. \
Otherwise they are equivalent."
}], "Text"],

Cell[TextData[{
 "To compute the number of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinct objects, you can use the formula ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"P", "(", 
     RowBox[{"n", ",", "r"}], ")"}], "=", 
    FractionBox[
     RowBox[{"n", "!"}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", "-", "r"}], ")"}], "!"}]]}], TraditionalForm]]],
 ", given as Corollary 1 in Section 6.3 of the textbook. We can define a \
function in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", which we call ",
 StyleBox["numPerm", "Input"],
 ", based on this formula. Recall that the formula requires that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "\[LessEqual]", "r", "\[LessEqual]", "n"}], TraditionalForm]]],
 ". We will include that as a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") in the definition of the function. "
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"numPerm", "[", 
    RowBox[{"n_Integer", ",", "r_Integer"}], "]"}], "/;", 
   RowBox[{"0", "\[LessEqual]", "r", "\[LessEqual]", "n"}]}], ":=", 
  RowBox[{
   RowBox[{"n", "!"}], "/", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"n", "-", "r"}], ")"}], "!"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Then the number of 4-permutations of a set with 7 distinct objects, that \
is, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"P", "(", 
    RowBox[{"7", ",", "4"}], ")"}], TraditionalForm]]],
 " is computed with the following expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"numPerm", "[", 
  RowBox[{"7", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["840"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Listing Permutations", "Subsubsection"],

Cell[TextData[{
 "To obtain a list of all permutations, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function."
}], "Text"],

Cell[TextData[{
 "The first argument is a list containing the objects to be permuted. If this \
is the only argument given, then the function returns the list of all \
permutations of those objects. For example, to list the permutations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 ", you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If you wish to list the permutations of the integers from 1 to a specified \
maximum, you can combine the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function with ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " applied to the maximum value. For example, the following produces all \
permutations of the first 4 positive integers."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{"Range", "[", "4", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To produce the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of a list of objects, you only have to provide a second \
argument to ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 ". If you give an integer, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", as the second argument, the function will return the list of all \
permutations with at most ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements. That is, it produces all of the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[LessEqual]", "n"}], TraditionalForm]]],
 ". For example, the following produces all permutations of at most two \
elements of the set of the first five positive integers. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "4", "}"}], ",", 
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "4"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To obtain only the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations for a specific ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", enter the second argument as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "r", "}"}], TraditionalForm]]],
 ". For example, to list all of the 3-permutations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 ", you would enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "4", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "4", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "4", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also provide a range of values for ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " by entering the second argument as ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["min", "CodeVar"], ",", 
   StyleBox["max", "CodeVar"], "}"}]], "myCode"],
 "."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Random Permutations", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " also provides a function, ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 ", that will produce a randomly chosen permutation."
}], "Text"],

Cell[TextData[{
 "Once again, the first argument is a list of the objects to be permuted. If \
no second argument is given, ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 " will output a randomly chosen permutation of all of the elements. The \
following produces a random permutation of the letters \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote] through \
\[OpenCurlyDoubleQuote]e\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomSample", "[", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
    "\"\<e\>\""}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"e\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
   ",", "\<\"d\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can also provide a positive integer as the second argument. In this \
case, ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 " will produce a random permutation of that size. Note that the second \
argument must be less than the size of the list. The following outputs a \
random 3-permutation of the first ten positive integers."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomSample", "[", 
  RowBox[{
   RowBox[{"Range", "[", "10", "]"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"9", ",", "5", ",", "3"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Note that the permutation is selected so that each permutation has the same \
probability of being chosen. \
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Combinations", "Subsection",
 CellTags->"Section3b"],

Cell["\<\
The functions related to combinations are very similar to those for \
permutations.\
\>", "Text"],

Cell[TextData[{
 "To compute the total number of combinations of a set of a specific size, \
use the formula ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "n"], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the number of objects being selected from. For example, the following \
shows that there are 32 subsets of a set of 5 elements."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"2", 
  StyleBox["^", "EquationNumbered"], 
  StyleBox["5", "EquationNumbered"]}]], "Input"],

Cell[BoxData["32"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To compute the number of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combinations of a set with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides the ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function. The following computes ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"52", ",", "5"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"52", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["2598960"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Listing Combinations", "Subsubsection"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function is the combination analog of ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "Given a list as the only argument, ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " outputs all possible subsets of every size."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"{", 
   RowBox[{
   "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], "}"}],
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"d\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"d\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To produce the subsets of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 ", give the argument as ",
 StyleBox["Range[n]", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"Range", "[", "3", "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " accepts a second argument with the same syntax as ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 ". If an integer is given as a second argument, the output will be all \
subsets whose cardinality is at most that integer. For example, the following \
finds all of the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combinations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
    "}"}], TraditionalForm]]],
 ", for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "\[LessEqual]", "2"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\""}], 
    "}"}], ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"c\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"d\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"d\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"d\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "If the second argument is provided as a list containing a single integer, \
then the output will be the subsets whose cardinality is equal to that \
integer. For example, you obtain the 3-combinations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 " as shown below.",
 Cell[BoxData[
  FormBox[Cell[""], TraditionalForm]]]
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "With a list of two integers as the second argument, you obtain the subsets \
whose cardinalities are between the two values. The integers must be given \
with the smaller value first, or the output will be the empty list. The \
following calculates the proper, non-empty subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "4", "}"}], ",", 
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4", ",", "5"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "With a third argument, you can limit the number of results that are \
included. A positive integer given as the third argument will cause ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " to output at most that number of subsets. Note that the second argument \
must be present to be able to use this option. If you wish to use the third \
argument, but provide no restriction on the size of the subsets considered, \
you can give ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " as the second argument. The following outputs the first 10 subsets of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", "All", ",", "10"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "1", "}"}], ",", 
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "4", "}"}], ",", 
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can obtain a single subset by entering ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["n", "CodeVar"], "}"}]], "myCode"],
 " as the third argument. This will output the list containing the ",
 Cell[BoxData["n"], "CodeVar"],
 "th subset in the canonical order used by ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " when listing subsets. You can also give a pair ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["n", "CodeVar"], ",", 
   StyleBox["m", "CodeVar"], "}"}]], "myCode"],
 " to list the subsets from the ",
 StyleBox["n", "CodeVar"],
 "th through the ",
 StyleBox["m", "CodeVar"],
 "th."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", "All", ",", 
   RowBox[{"{", "7", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "5", "]"}], ",", "All", ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "7"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "2", "}"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "4", "}"}], ",", 
   RowBox[{"{", "5", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Random Combinations", "Subsubsection"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " does not explicitly provide a function for selecting random combinations. \
To randomly select an ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combination for a particular ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", one approach is to apply ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " to a random permutation obtained with ",
 StyleBox[ButtonBox["RandomSample",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomSample"], "FunctionLink"],
 ". For example, the following produces a random 3-combination of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sort", "[", 
  RowBox[{"RandomSample", "[", 
   RowBox[{
    RowBox[{"Range", "[", "10", "]"}], ",", "3"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "5", ",", "8"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The application of ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " is not strictly necessary, but helps conceptually. The difference between \
an ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutation and an ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combination is that order \[OpenCurlyDoubleQuote]matters\
\[CloseCurlyDoubleQuote] for the permutation and does not for the \
combination. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "1"}], "}"}], TraditionalForm]]],
 " are different as permutations but the same as combinations or sets. By \
applying ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 ", we impose the same order on them, which makes the order they originally \
had irrelevant."
}], "Text"],

Cell[TextData[{
 "A second approach to is to randomly select from the output of ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 ". The function ",
 StyleBox[ButtonBox["RandomChoice",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomChoice"], "FunctionLink"],
 ", applied to a list, will return one randomly chosen element of the list. \
Since ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " outputs a list of subsets, it can be used as the argument to ",
 StyleBox[ButtonBox["RandomChoice",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomChoice"], "FunctionLink"],
 ". The following demonstrates an alternate approach to choosing a random \
3-combination of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 ". Recall that the second argument of {3} limits the output to the subsets \
of size 3."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RandomChoice", "[", 
  RowBox[{"Subsets", "[", 
   RowBox[{
    RowBox[{"Range", "[", "10", "]"}], ",", 
    RowBox[{"{", "3", "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "5", ",", "8"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "A third approach is to first select a random integer using the ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " function, and then use that randomly selected integer in the third \
argument of ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 ". You must give the ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " function the single argument of the form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "n"}], "}"}], TraditionalForm]]],
 " where ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is the number of combinations being selected from. This causes it to \
produce a random integer from 1 to that maximum. In the below, we first \
select a random integer between 1 and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"10", ",", "3"}], ")"}], TraditionalForm]]],
 ", the number of 3-combinations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 ". This value is stored as ",
 StyleBox["whichone", "Input"],
 ". Then we apply ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " with first argument ",
 StyleBox["Range[10]", "Input"],
 ", which produces the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "10"}], "}"}], 
   TraditionalForm]]],
 "; second argument ",
 StyleBox["{3}", "Input"],
 ", indicating that only 3-combinations are allowed; and third argument ",
 StyleBox["{whichone}", "Input"],
 ", which causes the function to return only the single subset at that \
position in the list of all the 3-combinations. Note that since ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " always outputs a list of sets, the result will be nested."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"whichone", "=", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{"{", 
     RowBox[{"1", ",", 
      RowBox[{"Binomial", "[", 
       RowBox[{"10", ",", "3"}], "]"}]}], "}"}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Subsets", "[", 
  RowBox[{
   RowBox[{"Range", "[", "10", "]"}], ",", 
   RowBox[{"{", "3", "}"}], ",", 
   RowBox[{"{", "whichone", "}"}]}], "]"}]}], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"{", 
   RowBox[{"3", ",", "8", ",", "9"}], "}"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The first approach is the most efficient, but can only be applied when \
selecting a random ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combination for a specific value of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". If you wish to select a random subset whose cardinality can vary, the \
first method will select longer combinations more frequently than short ones. \
The second approach, using ",
 StyleBox[ButtonBox["RandomChoice",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomChoice"], "FunctionLink"],
 ", is fairly straightforward and can be easily used to select a random \
combination without the restriction that they all be the same size. This \
approach can also be useful if you want to randomly select a combination from \
a list of combinations that meet some criteria. However, it is inefficient \
and restricts the maximum size of the original set. The third option, using \
",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 ", is reasonably efficient, although it is not quite as fast as the first \
option. It is also fairly flexible, as you are not restricted to a single \
cardinality, provided that you accurately count the number of possible \
combinations when applying ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 ". "
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Circular Permutations", "Subsection",
 CellTags->"Section3c"],

Cell[TextData[{
 "The prelude to Exercises 40 and 41 of Section 6.3 describes circular \
permutations. A circular ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutation of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " people is a seating of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " of those ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " people at a circular table. Moreover, two seatings are considered the same \
if one can be obtained from the other by rotation. "
}], "Text"],

Cell[TextData[{
 "The exercises ask you to compute the number of circular 3-permutations of 5 \
people and to arrive at a formula for that number. In this subsection, we'll \
write a function to list all of the circular ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " people. Having such a function can help you more easily explore the \
concept and test your formula. "
}], "Text"],

Cell[CellGroupData[{

Cell["Rotating a Permutation", "Subsubsection"],

Cell["\<\
The key to listing all circular permutations is to devise a way to test \
whether two circular permutations are equal. According to the definition, two \
circular permutations are considered to be equal if one can be obtained from \
the other by a rotation. If we use a list to represent a permutation, a \
rotation will consist of moving the first element to the end of the list (or \
the last to the front). \
\>", "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " includes the functions ",
 StyleBox[ButtonBox["RotateLeft",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RotateLeft"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["RotateRight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RotateRight"], "FunctionLink"],
 " that manipulate lists in exactly this way. ",
 StyleBox[ButtonBox["RotateLeft",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RotateLeft"], "FunctionLink"],
 " moves the first element to the end of the list and ",
 StyleBox[ButtonBox["RotateRight",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RotateRight"], "FunctionLink"],
 " moves the last element to the beginning, as illustrated below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "4", ",", "5", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateRight", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"5", ",", "1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Both of these functions can also accept an integer as a second optional \
argument to move more than one element at a time. For example, the following \
moves the first two elements of the list to the end.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], ",", "2"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4", ",", "5", ",", "1", ",", "2"}], "}"}]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Equality of Circular Permutations", "Subsubsection"],

Cell[TextData[{
 "Note that repeatedly rotating a permutation will eventually result in the \
original. Specifically, for an ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutation, after ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " rotations, the list will return to its original state. This is illustrated \
below with an example \
\[OpenCurlyDoubleQuote]seating.\[CloseCurlyDoubleQuote]"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<Abe\>\"", ",", "\"\<Carol\>\"", ",", "\"\<Barbara\>\""}], 
   "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Carol\"\>", ",", "\<\"Barbara\"\>", ",", "\<\"Abe\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Barbara\"\>", ",", "\<\"Abe\"\>", ",", "\<\"Carol\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RotateLeft", "[", "%", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"Abe\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Barbara\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This observation indicates that, given two ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations, we can test to see if they are the same by rotating one of \
them ",
 Cell[BoxData[
  FormBox[
   RowBox[{"r", "-", "1"}], TraditionalForm]]],
 " times. The function below returns true if the two input lists represent \
the same circular permutation and false otherwise. It first checks equality \
without performing rotation. Then, using a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop, it rotates the second list, checking for equality after each \
rotation."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"CPEqualQ", "[", 
   RowBox[{"L1_List", ",", "L2_List"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"Ltest", "=", "L2"}]}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"L1", "\[Equal]", "Ltest"}], ",", 
       RowBox[{"Return", "[", "True", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"For", "[", 
      RowBox[{
       RowBox[{"i", "=", "1"}], ",", 
       RowBox[{"i", "\[LessEqual]", 
        RowBox[{
         RowBox[{"Length", "[", "Ltest", "]"}], "-", "1"}]}], ",", 
       RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"Ltest", "=", 
         RowBox[{"RotateLeft", "[", "Ltest", "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"L1", "\[Equal]", "Ltest"}], ",", 
          RowBox[{"Return", "[", "True", "]"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     RowBox[{"Return", "[", "False", "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We can use this function to confirm equality of circular permutations. For \
example,\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CPEqualQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\"\<Charles\>\"", ",", "\"\<Helen\>\"", ",", "\"\<Dean\>\""}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\"\<Helen\>\"", ",", "\"\<Dean\>\"", ",", "\"\<Charles\>\""}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Listing All Circular Permutations", "Subsubsection"],

Cell[TextData[{
 "Now we are prepared to write a function that lists all circular \
permutations. First, we'll use ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " to generate all ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " people. We will initialize the list of all distinct circular permutations \
to the first element of the list of all permutations. That first permutation \
is then removed from the list of all permutations. Recall that ",
 StyleBox[ButtonBox["Delete",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Delete"], "FunctionLink"],
 " applied to a list and an index returns the list obtained by removing the \
element at the given location from the list."
}], "Text"],

Cell[TextData[{
 "Within a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop conditioned on the list of permutations being nonempty, consider the \
first element in the list of all permutations. Use ",
 StyleBox["CPEqualQ", "Input"],
 " and a loop to see if the first element is identical to any of the members \
of the set of circular permutations. If not, add it to the set of circular \
permutations. In either case, it is deleted from the list of all \
permutations. This continues until the list of all permutations has been \
emptied."
}], "Text"],

Cell[TextData[{
 "Here is the implementation. Our function will accept a list of \
\[OpenCurlyDoubleQuote]people\[CloseCurlyDoubleQuote] to be seated as the \
first argument and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", the number that can be seated at the table, as the second argument."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"circularPermutations", "[", 
   RowBox[{"S_List", ",", "r_Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"allP", ",", "allCP", ",", "isnew", ",", "p"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"allP", "=", 
      RowBox[{"Permutations", "[", 
       RowBox[{"S", ",", 
        RowBox[{"{", "r", "}"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"allCP", "=", 
      RowBox[{"{", 
       RowBox[{"allP", "[", 
        RowBox[{"[", "1", "]"}], "]"}], "}"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"allP", "=", 
      RowBox[{"Delete", "[", 
       RowBox[{"allP", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"allP", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"isnew", "=", 
         RowBox[{"Catch", "[", "\[IndentingNewLine]", 
          RowBox[{
           RowBox[{"Do", "[", 
            RowBox[{
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"CPEqualQ", "[", 
                RowBox[{
                 RowBox[{"allP", "[", 
                  RowBox[{"[", "1", "]"}], "]"}], ",", "p"}], "]"}], ",", 
               RowBox[{"Throw", "[", "False", "]"}]}], "]"}], ",", 
             RowBox[{"{", 
              RowBox[{"p", ",", "allCP"}], "}"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Throw", "[", "True", "]"}]}], "\[IndentingNewLine]", 
          "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{"isnew", ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"allCP", ",", 
            RowBox[{"allP", "[", 
             RowBox[{"[", "1", "]"}], "]"}]}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"allP", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"allP", ",", "1"}], "]"}]}]}]}], "\[IndentingNewLine]", 
      "]"}], ";", "\[IndentingNewLine]", "allCP"}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Note that the ",
 StyleBox["isnew", "Input"],
 " Boolean is used to track whether or not the current first member of ",
 StyleBox["allP", "Input"],
 " is new or not."
}], "Text"],

Cell[TextData[{
 "The following computes the possible circular 3-permutations of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "\"\<Abe\>\"", ",", "\"\<Barbara\>\"", ",", "\"\<Carol\>\"", ",", 
     "\"\<Dean\>\"", ",", "\"\<Eve\>\""}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"circularPermutations", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<Abe\>\"", ",", "\"\<Barbara\>\"", ",", "\"\<Carol\>\"", ",", 
     "\"\<Dean\>\"", ",", "\"\<Eve\>\""}], "}"}], ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Barbara\"\>", ",", "\<\"Carol\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Barbara\"\>", ",", "\<\"Dean\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Barbara\"\>", ",", "\<\"Eve\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Barbara\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Dean\"\>"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Eve\"\>"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Barbara\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Carol\"\>"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Eve\"\>"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Barbara\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Carol\"\>"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Abe\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Dean\"\>"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Dean\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Carol\"\>", ",", "\<\"Eve\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Carol\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Eve\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Carol\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Barbara\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Dean\"\>"}], 
    "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Carol\"\>", ",", "\<\"Dean\"\>", ",", "\<\"Eve\"\>"}], "}"}],
    ",", 
   RowBox[{"{", 
    RowBox[{"\<\"Carol\"\>", ",", "\<\"Eve\"\>", ",", "\<\"Dean\"\>"}], 
    "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"circularPermutations", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<Abe\>\"", ",", "\"\<Barbara\>\"", ",", "\"\<Carol\>\"", ",", 
      "\"\<Dean\>\"", ",", "\"\<Eve\>\""}], "}"}], ",", "3"}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["20"], "Output"]
}, Open  ]],

Cell[TextData[{
 "It is left to the reader to experiment with other starting sets and values \
of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " to determine a formula. Note that the functions in this subsection were \
written using a very naive approach. There are simpler and more efficient \
approaches, but those would give away the key idea used to create the formula."
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.4 Binomial Coefficients and Identities", "Section",
 CellTags->"Section4"],

Cell[TextData[{
 "In this section we will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to compute binomial coefficients, to generate Pascal's triangle, and to \
verify identities."
}], "Text"],

Cell[CellGroupData[{

Cell["The Binomial Theorem", "Subsection",
 CellTags->"Section4a"],

Cell[TextData[{
 "Recall from the previous section that the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " can be used to compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", GridBox[{
      {"n"},
      {"k"}
     }], ")"}], TraditionalForm]]],
 ", which is another notation for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"n", ",", "k"}], ")"}], TraditionalForm]]],
 ". The ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function is in fact more general than the binomial coefficients described \
in the textbook, as it will compute coefficients that appear in Newton's \
generalized binomial theorem. The generalization is beyond the scope of this \
manual, but be aware that the function may return values even for inputs that \
you might expect should cause an error."
}], "Text"],

Cell["\<\
Here, we will consider questions such as Examples 2 through 4 from section \
6.4 of the text. \
\>", "Text"],

Cell[TextData[{
 "First consider the problem of expanding ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "+", "y"}], ")"}], "5"], TraditionalForm]]],
 ". In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", this can be done easily with ",
 StyleBox[ButtonBox["Expand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Expand"], "FunctionLink"],
 ". The ",
 StyleBox[ButtonBox["Expand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Expand"], "FunctionLink"],
 " function requires one argument, an algebraic expression. It returns the \
result of expanding the expression, that is, of distributing products over \
sums."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "+", "y"}], ")"}], "^", "5"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "5"], "+", 
  RowBox[{"5", " ", 
   SuperscriptBox["x", "4"], " ", "y"}], "+", 
  RowBox[{"10", " ", 
   SuperscriptBox["x", "3"], " ", 
   SuperscriptBox["y", "2"]}], "+", 
  RowBox[{"10", " ", 
   SuperscriptBox["x", "2"], " ", 
   SuperscriptBox["y", "3"]}], "+", 
  RowBox[{"5", " ", "x", " ", 
   SuperscriptBox["y", "4"]}], "+", 
  SuperscriptBox["y", "5"]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now consider the question of finding the coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "18"], 
    SuperscriptBox["y", "12"]}], TraditionalForm]]],
 " in the expansion of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "+", "y"}], ")"}], "30"], TraditionalForm]]],
 ". The binomial theorem tells us that this coefficient is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", GridBox[{
      {"30"},
      {"12"}
     }], ")"}], TraditionalForm]]],
 ". The ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function with first argument 30 and second 12 will produce this value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"30", ",", "12"}], "]"}]], "Input"],

Cell[BoxData["86493225"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Thus, the expansion of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "+", "y"}], ")"}], "30"], TraditionalForm]]],
 " contains the term ",
 Cell[BoxData[
  FormBox[
   RowBox[{"86493225", 
    SuperscriptBox["x", "18"], 
    SuperscriptBox["y", "12"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Finding the coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "12"], 
    SuperscriptBox["y", "13"]}], TraditionalForm]]],
 " in the expansion of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", "x"}], "-", 
      RowBox[{"3", "y"}]}], ")"}], "25"], TraditionalForm]]],
 " requires that we include the coefficients of ",
 Cell[BoxData[
  FormBox["x", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["y", TraditionalForm]]],
 " in the computation. As explained in the solution to Example 4 of the text, \
the expansion is"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", "x"}], "+", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "3"}], "y"}], ")"}]}], ")"}], "25"], "=", 
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"j", "=", "0"}], "25",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"25"},
        {"j"}
       }], ")"}], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"2", "x"}], ")"}], 
      RowBox[{"25", "-", "j"}]], 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "3"}], "y"}], ")"}], "j"]}]}]}], 
  TraditionalForm]], "Equation"],

Cell[TextData[{
 "The coefficient of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["x", "12"], 
    SuperscriptBox["y", "13"]}], TraditionalForm]]],
 " is found by taking ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "=", "13"}], TraditionalForm]]],
 ":"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {"25"},
      {"13"}
     }], ")"}], 
   SuperscriptBox["2", "12"], 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "3"}], ")"}], "13"]}], TraditionalForm]], "Equation"],

Cell["This is", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Binomial", "[", 
   RowBox[{"25", ",", "13"}], "]"}], "*", 
  RowBox[{"2", "^", "12"}], "*", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}], "^", "13"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"-", "33959763545702400"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Pascal\[CloseCurlyQuote]s Triangle", "Subsection",
 CellTags->"Section4b"],

Cell[TextData[{
 "As we've seen, it is very easy to compute binomial coefficients with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". To compute row ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " of Pascal's triangle, we apply the ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function with the second argument ranging from 0 to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This can be done with the ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " function. For example, the 25th row of Pascal's triangle is shown below."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"Binomial", "[", 
    RowBox[{"25", ",", "n"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"n", ",", "0", ",", "25"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "1", ",", "25", ",", "300", ",", "2300", ",", "12650", ",", "53130", ",", 
   "177100", ",", "480700", ",", "1081575", ",", "2042975", ",", "3268760", 
   ",", "4457400", ",", "5200300", ",", "5200300", ",", "4457400", ",", 
   "3268760", ",", "2042975", ",", "1081575", ",", "480700", ",", "177100", 
   ",", "53130", ",", "12650", ",", "2300", ",", "300", ",", "25", ",", "1"}],
   "}"}]], "Output"]
}, Open  ]],

Cell["\<\
When calculating a single binomial coefficient or an isolated row, applying \
the formula may be the most efficient approach. But if you wish to build a \
sizable portion of Pascal's triangle, making use of Pascal's identity \
(Theorem 2 of Section 6.4) and the symmetry property (Corollary 2 of Section \
6.3) can be more effective. \
\>", "Text"],

Cell[TextData[{
 "In this subsection, we'll write two functions for computing binomial \
coefficients. The first will simply apply the formula ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"n", "!"}], 
    RowBox[{
     RowBox[{"k", "!"}], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", "-", "k"}], ")"}], "!"}]}]], TraditionalForm]]],
 ". The second will be a recursive function making use of Pascal's identity \
and symmetry. Then we'll compare the performance of the two functions in \
building Pascal's triangle."
}], "Text"],

Cell[TextData[{
 "The first function will be a straightforward application of the formula. We \
name it ",
 StyleBox["binomialF", "Input"],
 " (for formula)."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"binomialF", "[", 
    RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
   RowBox[{"0", "\[LessEqual]", "k", "\[LessEqual]", "n"}]}], ":=", 
  RowBox[{
   RowBox[{"n", "!"}], "/", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"k", "!"}], "*", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"n", "-", "k"}], ")"}], "!"}]}], ")"}]}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell["A Recursive Function", "Subsubsection"],

Cell[TextData[{
 "The second function will be called ",
 StyleBox["binomialR", "Input"],
 " (for recursive). Recall Pascal's identity:"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {
       RowBox[{"n", "+", "1"}]},
      {"k"}
     }], ")"}], "=", 
   RowBox[{
    RowBox[{"(", GridBox[{
       {"n"},
       {
        RowBox[{"k", "-", "1"}]}
      }], ")"}], "+", 
    RowBox[{"(", GridBox[{
       {"n"},
       {"k"}
      }], ")"}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "Rewriting this in terms of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 ", we have"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {"n"},
      {"k"}
     }], ")"}], "=", 
   RowBox[{
    RowBox[{"(", GridBox[{
       {
        RowBox[{"n", "-", "1"}]},
       {
        RowBox[{"k", "-", "1"}]}
      }], ")"}], "+", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"n", "-", "1"}]},
       {"k"}
      }], ")"}]}]}], TraditionalForm]], "Equation"],

Cell["\<\
Also recall that the binomial coefficients are symmetric, that is,\
\>", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {"n"},
      {"k"}
     }], ")"}], "=", 
   RowBox[{"(", GridBox[{
      {"n"},
      {
       RowBox[{"n", "-", "k"}]}
     }], ")"}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "With these facts in mind, our recursive function will work as follows. It \
will be designed to store values in an indexed variable with the same name as \
the function, ",
 StyleBox["binomialR", "Input"],
 ". The body of the function will consist of a ",
 StyleBox[ButtonBox["Which",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Which"], "FunctionLink"],
 " statement. In case the second argument is 0, the result is 1. That forms \
the basis case of the recursion. The second possibility is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "k"}], ">", "n"}], TraditionalForm]]],
 ". In this case, we make use of symmetry and call ",
 StyleBox["binomialR", "Input"],
 " on ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "k"}], TraditionalForm]]],
 ". Finally, if neither of those conditions are met, we apply Pascal's \
identity, making recursive calls to ",
 StyleBox["binomialR", "Input"],
 ". Here is the implementation."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"binomialR", "[", 
    RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
   RowBox[{"0", "\[LessEqual]", "k", "\[LessEqual]", "n"}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{"Which", "[", 
     RowBox[{
      RowBox[{"k", "\[Equal]", "0"}], ",", 
      RowBox[{
       RowBox[{"binomialR", "[", 
        RowBox[{"n", ",", "k"}], "]"}], "=", "1"}], ",", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"2", "*", "k"}], ">", "n"}], ",", 
      RowBox[{
       RowBox[{"binomialR", "[", 
        RowBox[{"n", ",", "k"}], "]"}], "=", 
       RowBox[{"binomialR", "[", 
        RowBox[{"n", ",", 
         RowBox[{"n", "-", "k"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
      "True", ",", 
      RowBox[{
       RowBox[{"binomialR", "[", 
        RowBox[{"n", ",", "k"}], "]"}], "=", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{
          RowBox[{"n", "-", "1"}], ",", 
          RowBox[{"k", "-", "1"}]}], "]"}], "+", 
        RowBox[{"binomialR", "[", 
         RowBox[{
          RowBox[{"n", "-", "1"}], ",", "k"}], "]"}]}]}]}], 
     "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[TextData[{
 "We use our two functions to build Pascal's triangle using a ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with two loop variables. We use the ",
 StyleBox[ButtonBox["Column",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Column"], "FunctionLink"],
 " function with ",
 StyleBox[ButtonBox["Center",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Center"], "FunctionLink"],
 " as second argument to give the table its usual triangular form."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Column", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"binomialF", "[", 
      RowBox[{"n", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "0", ",", "n"}], "}"}]}], "]"}], ",", "Center"}], 
  "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", "1", "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "3", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "4", ",", "6", ",", "4", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "5", ",", "10", ",", "10", ",", "5", ",", "1"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Center}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Column", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"binomialR", "[", 
      RowBox[{"n", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", "5"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "0", ",", "n"}], "}"}]}], "]"}], ",", "Center"}], 
  "]"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", "1", "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "2", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "3", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "4", ",", "6", ",", "4", ",", "1"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{"1", ",", "5", ",", "10", ",", "10", ",", "5", ",", "1"}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Center}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Comparing Performance", "Subsubsection"],

Cell["Now we'll compare the performance of the two functions.", "Text"],

Cell[TextData[{
 "First, we'll use each of them to compute the first 1000 rows of Pascal's \
triangle and compare the time it takes. We use ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " and redefine ",
 StyleBox["binomialR", "Input"],
 " in order to remove any values already stored so that the comparison is \
fair."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"binomialF", "[", 
      RowBox[{"n", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", "1000"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "0", ",", "n"}], "}"}]}], "]"}], ";"}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "9.22489099999999950796336634084582328796`6.985561156579211", ",", "Null"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "binomialR", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"binomialR", "[", 
     RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
    RowBox[{"0", "\[LessEqual]", "k", "\[LessEqual]", "n"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"k", "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"2", "*", "k"}], ">", "n"}], ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", 
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", 
          RowBox[{"n", "-", "k"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       "True", ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", 
        RowBox[{
         RowBox[{"binomialR", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", 
           RowBox[{"k", "-", "1"}]}], "]"}], "+", 
         RowBox[{"binomialR", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", "k"}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Timing", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"binomialR", "[", 
      RowBox[{"n", ",", "k"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"n", ",", "0", ",", "1000"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"k", ",", "0", ",", "n"}], "}"}]}], "]"}], ";"}], 
  "\[IndentingNewLine]", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "4.5178549999999999542410478170495480299`6.675532201459716", ",", "Null"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
You see that the difference between the two performance of the two functions \
is substantial. \
\>", "Text"],

Cell[TextData[{
 "However, if we compute some isolated values, the situation is reversed. We \
will generate some random values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " between 100 and 200 and random values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " between 0 and the corresponding value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " function applied to an integer outputs a random integer between 0 and that \
value, and given a pair ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], TraditionalForm]]],
 ", it returns a randomly selected integer between ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 ". With a positive integer as a second argument, the function will produce \
that many random integers in the selected range."
}], "Text"],

Cell[TextData[{
 "To produce 100 random values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", we simply call ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " on the range ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"100", ",", "200"}], "}"}], TraditionalForm]]],
 " and the number 100 and assign the resulting list of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " values to the symbol ",
 StyleBox["randomNs", "Input"],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"randomNs", "=", 
   RowBox[{"RandomInteger", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"100", ",", "200"}], "}"}], ",", "100"}], "]"}]}], 
  ";"}]], "Input"],

Cell[TextData[{
 "To produce the list of values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", we use ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ". Each value of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " will be found by applying ",
 StyleBox[ButtonBox["RandomInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/RandomInteger"], "FunctionLink"],
 " to the value found in ",
 StyleBox["randomNs", "Input"],
 ". That way, the values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " will be between 0 and the corresponding value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"randomKs", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"RandomInteger", "[", 
      RowBox[{"randomNs", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "100"}], "}"}]}], "]"}]}], ";"}]], "Input"],

Cell[TextData[{
 "We can time the performance of ",
 StyleBox["binomialF", "Input"],
 " simply by applying the function to the pairs of values."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Timing", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"binomialF", "[", 
      RowBox[{
       RowBox[{"randomNs", "[", 
        RowBox[{"[", "i", "]"}], "]"}], ",", 
       RowBox[{"randomKs", "[", 
        RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "100"}], "}"}]}], "]"}], ";"}], "\[IndentingNewLine]",
   "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.00100799999999999999843180997771696639`3.0240604453891304", ",", 
   "Null"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To check the performance of the recursive function, we must again ",
 StyleBox[ButtonBox["Clear",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Clear"], "FunctionLink"],
 " and redefine the function, as it currently stores the entire table up to \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "1000"}], TraditionalForm]]],
 ". We also must override ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s recursion limit, ",
 StyleBox[ButtonBox["$RecursionLimit",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/$RecursionLimit"], "FunctionLink"],
 ", to avoid errors based on the depth of the recursion we are attempting. \
This was not necessary when computing the table through row 1000 above, \
because each row was computed in turn and thus each individual computation \
had to look only one row down to find a known value. We set the recursion \
limit inside of a ",
 StyleBox[ButtonBox["Block",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Block"], "FunctionLink"],
 " to localize the override."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"Clear", "[", "binomialR", "]"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"binomialR", "[", 
     RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
    RowBox[{"0", "\[LessEqual]", "k", "\[LessEqual]", "n"}]}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", "}"}], ",", "\[IndentingNewLine]", 
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"k", "\[Equal]", "0"}], ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", "1"}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"2", "*", "k"}], ">", "n"}], ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", 
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", 
          RowBox[{"n", "-", "k"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
       "True", ",", 
       RowBox[{
        RowBox[{"binomialR", "[", 
         RowBox[{"n", ",", "k"}], "]"}], "=", 
        RowBox[{
         RowBox[{"binomialR", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", 
           RowBox[{"k", "-", "1"}]}], "]"}], "+", 
         RowBox[{"binomialR", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", "k"}], "]"}]}]}]}], 
      "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{"Block", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"$RecursionLimit", "=", "Infinity"}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"Timing", "[", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"binomialR", "[", 
        RowBox[{
         RowBox[{"randomNs", "[", 
          RowBox[{"[", "i", "]"}], "]"}], ",", 
         RowBox[{"randomKs", "[", 
          RowBox[{"[", "i", "]"}], "]"}]}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", "100"}], "}"}]}], "]"}], ";"}], 
    "\[IndentingNewLine]", "]"}]}], "\[IndentingNewLine]", "]"}]}], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "0.07430599999999999705213582501528435387`4.891623796511053", ",", "Null"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The reason for the difference in relative performance is that when \
generating Pascal's triangle, all of the values beginning with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "0"}], TraditionalForm]]],
 " needed to be calculated. On the other hand when calculating isolated \
values, the recursive function still had to calculate all of the results for \
lower values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", which the non-recursive function did not need to compute."
}], "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Verifying Identities", "Subsection",
 CellTags->"Section4c"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can help verify identities involving the binomial coefficients. If you \
enter an expression using the ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function and include symbolic arguments, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will try to simplify the expression algebraically. First, we ensure that \
",
 StyleBox["n", "Input"],
 " and ",
 StyleBox["k", "Input"],
 " are unassigned."
}], "Text"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"n", ",", "k"}], "]"}]], "Input"],

Cell[TextData[{
 "For example, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " can transform ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"n", ",", "3"}], ")"}], TraditionalForm]]],
 " into an algebraic expression."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"n", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "6"], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "2"}], "+", "n"}], ")"}], " ", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"-", "1"}], "+", "n"}], ")"}], " ", "n"}]], "Output"]
}, Open  ]],

Cell["\<\
More general expressions, however, will be left unsimplified.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"n", ",", "k"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"n", ",", "k"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "However, we can force ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to unwind this general expression into its formula with the ",
 StyleBox[ButtonBox["FunctionExpand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FunctionExpand"], "FunctionLink"],
 " function. ",
 StyleBox[ButtonBox["FunctionExpand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FunctionExpand"], "FunctionLink"],
 " is a way to insist that certain complex functions, such as ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 ", are expanded when doing so might allow further simplification. Observe \
what happens:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionExpand", "[", 
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", "k"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{"Gamma", "[", 
   RowBox[{"1", "+", "n"}], "]"}], 
  RowBox[{
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "+", "k"}], "]"}], " ", 
   RowBox[{"Gamma", "[", 
    RowBox[{"1", "-", "k", "+", "n"}], "]"}]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 "A complete description of the gamma function is beyond the scope of this \
manual. Suffice it to say that for positive integers, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", "n", ")"}], "=", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"n", "-", "1"}], ")"}], "!"}]}], TraditionalForm]]],
 ". Because we are concerned only with the domain of positive integers, we \
can transform the expression above into its more typical form by applying ",
 StyleBox[ButtonBox["ReplaceAll",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplaceAll"], "FunctionLink"],
 " (",
 StyleBox["/.", "Input"],
 ") and the rule \[CapitalGamma](n)=(n-1)!."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FunctionExpand", "[", 
   RowBox[{"Binomial", "[", 
    RowBox[{"n", ",", "k"}], "]"}], "]"}], "/.", 
  RowBox[{
   RowBox[{"Gamma", "[", "n_", "]"}], "\[Rule]", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"n", "-", "1"}], ")"}], "!"}]}]}]], "Input"],

Cell[BoxData[
 FractionBox[
  RowBox[{"n", "!"}], 
  RowBox[{
   RowBox[{"k", "!"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", "k"}], "+", "n"}], ")"}], "!"}]}]]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Verifying Symmetry", "Subsubsection"],

Cell[TextData[{
 "As a first example, we\[CloseCurlyQuote]ll verify the identity ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"C", "(", 
     RowBox[{"n", ",", "k"}], ")"}], "=", 
    RowBox[{"C", "(", 
     RowBox[{"n", ",", 
      RowBox[{"n", "-", "k"}]}], ")"}]}], TraditionalForm]]],
 ", the symmetry identity."
}], "Text"],

Cell["\<\
Assign names to the left and right hand sides of the identity. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"left", "=", 
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", "k"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"n", ",", "k"}], "]"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"right", "=", 
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", 
    RowBox[{"n", "-", "k"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"Binomial", "[", 
  RowBox[{"n", ",", 
   RowBox[{
    RowBox[{"-", "k"}], "+", "n"}]}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that if you simply try to apply ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 "), ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will simply echo the expressions, indicating that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " is not able to confirm that the expressions are equal or not."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"left", "\[Equal]", "right"}]], "Input"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", "k"}], "]"}], "\[Equal]", 
  RowBox[{"Binomial", "[", 
   RowBox[{"n", ",", 
    RowBox[{
     RowBox[{"-", "k"}], "+", "n"}]}], "]"}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Also, ",
 StyleBox[ButtonBox["SameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SameQ"], "FunctionLink"],
 " (",
 StyleBox["===", "Input"],
 ") will return ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 ", since the expressions are not identical."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"left", "===", "right"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The above illustrate that a great deal of care is required when using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to verify identities. A negative result from ",
 StyleBox[ButtonBox["SameQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SameQ"], "FunctionLink"],
 " (",
 StyleBox["===", "Input"],
 ") indicates that the expressions are not identical, while they still may be \
algebraically equivalent. And a failure of",
 " ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") generally means that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs more direction about how to proceed."
}], "Text"],

Cell[TextData[{
 "In particular, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " needs to be told to expand the ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 " function. We saw one way to this above, using ",
 StyleBox[ButtonBox["FunctionExpand",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FunctionExpand"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["FullSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullSimplify"], "FunctionLink"],
 " is another function that will tell ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to delve into the definition of a function and attempt to perform \
algebraic simplification. Applying ",
 StyleBox[ButtonBox["FullSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullSimplify"], "FunctionLink"],
 " to the ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") test yields the correct result."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"left", "\[Equal]", "right"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["A Second Identity", "Subsubsection"],

Cell["Exercise 26 in Section 6.4 asks you to prove the identity: ", "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"k", "=", "1"}], "n",
     LimitsPositioning->False], 
    RowBox[{
     RowBox[{"(", GridBox[{
        {"n"},
        {"k"}
       }], ")"}], 
     RowBox[{"(", GridBox[{
        {"n"},
        {
         RowBox[{"k", "-", "1"}]}
       }], ")"}]}]}], " ", "=", " ", 
   RowBox[{
    RowBox[{
     RowBox[{"(", GridBox[{
        {
         RowBox[{
          RowBox[{"2", "n"}], "+", "2"}]},
        {
         RowBox[{"n", "+", "1"}]}
       }], ")"}], "/", "2"}], "-", 
    RowBox[{"(", GridBox[{
       {
        RowBox[{"2", "n"}]},
       {"n"}
      }], ")"}]}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will verify this identity for us."
}], "Text"],

Cell["\<\
First, we will give the right hand side of the identity a name.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"right2", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"Binomial", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"2", "n"}], "+", "2"}], ",", 
      RowBox[{"n", "+", "1"}]}], "]"}], "/", "2"}], "-", 
   RowBox[{"Binomial", "[", 
    RowBox[{
     RowBox[{"2", "n"}], ",", "n"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   RowBox[{"Binomial", "[", 
    RowBox[{
     RowBox[{"2", " ", "n"}], ",", "n"}], "]"}]}], "+", 
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"Binomial", "[", 
    RowBox[{
     RowBox[{"2", "+", 
      RowBox[{"2", " ", "n"}]}], ",", 
     RowBox[{"1", "+", "n"}]}], "]"}]}]}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "As you may have expected, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " simply echoed the expression"
}], "Text"],

Cell[TextData[{
 "The left hand side is a summation, so we apply the ",
 StyleBox[ButtonBox["Sum",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sum"], "FunctionLink"],
 " function."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"left2", "=", 
  RowBox[{"Sum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Binomial", "[", 
      RowBox[{"n", ",", "k"}], "]"}], "*", 
     RowBox[{"Binomial", "[", 
      RowBox[{"n", ",", 
       RowBox[{"k", "-", "1"}]}], "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "1", ",", "n"}], "}"}]}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 FractionBox[
  RowBox[{
   SuperscriptBox["4", "n"], " ", "n", " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "1"}], "+", 
        RowBox[{"2", " ", "n"}]}], ")"}]}], ")"}], "!"}]}], 
  RowBox[{
   SqrtBox["\[Pi]"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", "n"}], ")"}], "!"}]}]]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " automatically simplified the summation and returned a closed formula."
}], "Text"],

Cell[TextData[{
 "To verify the identity, apply ",
 StyleBox[ButtonBox["FullSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullSimplify"], "FunctionLink"],
 " to the expression identifying the two sides of the formula. Note that ",
 StyleBox[ButtonBox["FullSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullSimplify"], "FunctionLink"],
 " is required, using ",
 StyleBox[ButtonBox["Equal",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Equal"], "FunctionLink"],
 " (",
 StyleBox["==", "Input"],
 ") alone will not produce a truth value."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", 
  RowBox[{"left2", "\[Equal]", "right2"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Keep in mind when using ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to check identities that it will only report true when the two sides of \
the expression are algebraically equivalent. If it does report true, you can \
be fairly confident that the identity does hold, though a truly convincing \
proof requires that you explicitly show the algebraic manipulations or \
provide a combinatorial argument justifying the equivalence."
}], "Text"],

Cell[TextData[{
 "If ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " reports false or just echoes the expression, however, even after using ",
 StyleBox[ButtonBox["FullSimplify",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FullSimplify"], "FunctionLink"],
 ", you can not be certain whether the identity is false or if it is true but \
more manipulation is needed to get ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to recognize it. To use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to demonstrate that a purported identity is false, you would need to find \
a counterexample by computing the values of both expressions and finding \
inputs that result in different values. (Refer to Section 1.7 for examples of \
finding counterexamples.)"
}], "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.5 Generalized Permutations and Combinations", "Section",
 CellTags->"Section5"],

Cell[TextData[{
 "In this section we will introduce a variety of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions related to permutations and combinations with repetition allowed \
and related to distributing objects in boxes where the objects and the boxes \
may or may not be distinguishable."
}], "Text"],

Cell[CellGroupData[{

Cell["Permutations with Repetition", "Subsection",
 CellTags->"Section5a"],

Cell[TextData[{
 "Recall from Theorem 1 that the number of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["n", "r"], TraditionalForm]]],
 " if repetition is allowed."
}], "Text"],

Cell["\<\
For example, the number of strings of length 5 that can be formed from the 26 \
uppercase letters of the English alphabet is\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"26", "^", "5"}]], "Input"],

Cell[BoxData["11881376"], "Output"]
}, Open  ]],

Cell["\<\
As a second example, we compute the number of ways that four elements can be \
selected in order from a set with three elements when repetition is allowed.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"3", "^", "4"}]], "Input"],

Cell[BoxData["81"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Recall from the previous section that the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function accepts a list as its first argument. In case the list contains \
repeated elements, those elements are treated as identical, but are allowed \
to repeat in the permutations."
}], "Text"],

Cell["For example, consider the expression below.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "2"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Given a list of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " not necessarily distinct objects and no second argument, the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function produces all of the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "-permutations of the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects. Note that 1 appeared twice in the input and thus appears twice in \
the results, while 2 appeared once in the input and so appears once in the \
output."
}], "Text"],

Cell["\<\
With a second argument, you can specify the length of the permutations, just \
as was described in the previous section.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell["\<\
Since 1 appeared twice in the input list, it was allowed to appear twice in \
the permutations, but 2 appeared only one time in the input and thus was not \
allowed to repeat.\
\>", "Text"],

Cell[TextData[{
 "This means that if you want to list the ways that four elements can be \
selected in order from a set with three elements when repetition is allowed, \
you can use the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function, so long as you repeat the elements in the input. In order to \
generate all ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects with repetition allowed, you must use as input the list consisting \
of the ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects each repeated ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " times. If an object is repeated fewer than ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " times in the input list, then that will limit the number of times it is \
allowed to repeat in the results."
}], "Text"],

Cell[TextData[{
 "To form 4-permutations with repetition allowed of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 ", we apply the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function after using ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " to build its argument. "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " function takes two arguments. The first is the object to be repeated and \
the second is the number of times the object is to be repeated. The output is \
the list consisting of that number of copies of the element. For example, the \
following creates the list consisting of four copies of the character \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote]."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConstantArray", "[", 
  RowBox[{"\"\<a\>\"", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>"}],
   "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " function accepts a number of lists and outputs the list obtained by \
merging them. The following produces the list of four \
\[OpenCurlyDoubleQuote]a\[CloseCurlyDoubleQuote]s and four \
\[OpenCurlyDoubleQuote]b\[CloseCurlyDoubleQuote]s and four \
\[OpenCurlyDoubleQuote]c\[CloseCurlyDoubleQuote]s."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Join", "[", 
  RowBox[{
   RowBox[{"ConstantArray", "[", 
    RowBox[{"\"\<a\>\"", ",", "4"}], "]"}], ",", 
   RowBox[{"ConstantArray", "[", 
    RowBox[{"\"\<b\>\"", ",", "4"}], "]"}], ",", 
   RowBox[{"ConstantArray", "[", 
    RowBox[{"\"\<c\>\"", ",", "4"}], "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
   ",", "\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
   ",", "\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "You can make this more compact, especially for large numbers of symbols, by \
applying ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " with a table variable ranging over the desired elements. Remember that if \
the second argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " is of the form ",
 Cell[BoxData[
  RowBox[{"{", 
   StyleBox["variable", "CodeVar"], ",", 
   StyleBox["list", "CodeVar"], "}"}]], "myCode"],
 ", then the table variable will be assigned each element of the list in \
turn. Using ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " as the first argument to ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", with the table variable as the array element, we can obtain the list of \
\[OpenCurlyDoubleQuote]constant arrays.\[CloseCurlyDoubleQuote]"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{"ConstantArray", "[", 
    RowBox[{"i", ",", "4"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", 
     RowBox[{"{", 
      RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}]}], 
    "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Since ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 " expects several lists as arguments, not a single list of lists, we need to \
use the ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") operator. ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ")",
 " ",
 "has the effect of taking the elements of a list and giving them as the \
arguments to a function. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Join", " ", "@@", 
  RowBox[{"Table", "[", 
   RowBox[{
    RowBox[{"ConstantArray", "[", 
     RowBox[{"i", ",", "4"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", 
      RowBox[{"{", 
       RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}]}], 
     "}"}]}], "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
   ",", "\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
   ",", "\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This is the list we give to the ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function, along with the size specification {4}, to obtain permutations of \
length 4."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"abcRepeated", "=", 
  RowBox[{"Permutations", "[", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Join", " ", "@@", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"ConstantArray", "[", 
        RowBox[{"i", ",", "4"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"{", 
          RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}]}], 
        "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", "4", "}"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>", 
     ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"c\"\>", ",", "\<\"c\"\>", 
     ",", "\<\"c\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "abcRepeated", "]"}]], "Input"],

Cell[BoxData["81"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the size of the list produced by ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " agrees with the answer given by the formula ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["n", "r"], TraditionalForm]]],
 "."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Combinations with Repetition", "Subsection",
 CellTags->"Section5b"],

Cell[TextData[{
 "Combinations with repetition can be handled in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " in much the same way as permutations with repetition are."
}], "Text"],

Cell[TextData[{
 "Theorem 2 of Section 6.5, asserts that the number of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combinations of a set of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects when repetition of elements is allowed is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{
     RowBox[{"n", "+", "r", "-", "1"}], ",", "r"}], ")"}], 
   TraditionalForm]]],
 ". This suggests the following useful function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CRep", "[", 
    RowBox[{"n_Integer", ",", "r_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", 
    RowBox[{"r", "\[GreaterEqual]", "0"}]}]}], ":=", 
  RowBox[{"Binomial", "[", 
   RowBox[{
    RowBox[{"n", "+", "r", "-", "1"}], ",", "r"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
Thus we can compute, for example, the number of ways to select five bills \
from a cash box with seven types of bills (Example 3) as follows. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CRep", "[", 
  RowBox[{"7", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["462"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can also make use of the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function similar to how ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " was used above."
}], "Text"],

Cell[TextData[{
 "Consider Example 2 from Section 6.5. In this example, we are given a bowl \
of apples, oranges, and pears and are to select four pieces of fruit from the \
bowl provided that it contains at least four pieces of each kind of fruit. We \
have two ways to solve this problem with ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". "
}], "Text"],

Cell[TextData[{
 "First, we can use the ",
 StyleBox["CRep", "Input"],
 " function we created."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CRep", "[", 
  RowBox[{"3", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["15"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The other approach is to use ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " to list all the options. We form the argument to ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " in the same way as in the last subsection. However, the difference between \
",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " is that ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " treats repeated objects as distinct. This means that combinations will be \
repeated. For example, consider the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Subsets", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<b\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The subsets ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "b", "}"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}], TraditionalForm]]],
 " appear twice because ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " considered the two \[OpenCurlyDoubleQuote]b\[CloseCurlyDoubleQuote]s in \
the input to be different from each other. So one ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "b", "}"}], TraditionalForm]]],
 " in the output is from the first \[OpenCurlyDoubleQuote]b\
\[CloseCurlyDoubleQuote] in the input, and the other ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "b", "}"}], TraditionalForm]]],
 " is the second \[OpenCurlyDoubleQuote]b\[CloseCurlyDoubleQuote] in the \
input. To eliminate these redundancies, we can apply ",
 StyleBox[ButtonBox["DeleteDuplicates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DeleteDuplicates"], "FunctionLink"],
 ", a function that removes duplicate elements from a list."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeleteDuplicates", "[", 
  RowBox[{"Subsets", "[", 
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<b\>\""}], "}"}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "}"}], ",", 
   RowBox[{"{", "\<\"a\"\>", "}"}], ",", 
   RowBox[{"{", "\<\"b\"\>", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"b\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell["\<\
To answer the question about the bowl of fruit, we enter the following.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeleteDuplicates", "[", 
  RowBox[{"Subsets", "[", 
   RowBox[{
    RowBox[{"Join", "@@", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"ConstantArray", "[", 
        RowBox[{"i", ",", "4"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"{", 
          RowBox[{
          "\"\<apple\>\"", ",", "\"\<pear\>\"", ",", "\"\<orange\>\""}], 
          "}"}]}], "}"}]}], "]"}]}], ",", "\[IndentingNewLine]", 
    RowBox[{"{", "4", "}"}]}], "\[IndentingNewLine]", "]"}], 
  "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"apple\"\>", 
     ",", "\<\"apple\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"apple\"\>", 
     ",", "\<\"pear\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"apple\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"pear\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"apple\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"pear\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"pear\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"pear\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"pear\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"apple\"\>", ",", "\<\"orange\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"pear\"\>", ",", "\<\"pear\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"pear\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"pear\"\>", ",", "\<\"pear\"\>", ",", "\<\"pear\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"pear\"\>", ",", "\<\"pear\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"pear\"\>", ",", "\<\"orange\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"orange\"\>", ",", "\<\"orange\"\>", ",", "\<\"orange\"\>", 
     ",", "\<\"orange\"\>"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input"],

Cell[BoxData["15"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Pay careful attention to the difference between ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 ". ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " considered repeated elements to be identical, while ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " treats them as distinct."
}], "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Permutations with Indistinguishable Objects", "Subsection",
 CellTags->"Section5c"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " handles permutations with indistinguishable objects in the same way as \
when repetition is allowed. The ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " function accepts a list of objects as its first argument. If objects in \
the list are repeated, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " considers them as indistinguishable and produces the permutations \
appropriately."
}], "Text"],

Cell[TextData[{
 "For example, to solve Example 7, finding the number of different strings \
that can be made from the letters of the word SUCCESS, we use the list ",
 Cell[BoxData[
  FormBox[
   StyleBox[
    RowBox[{"{", 
     RowBox[{
     "\"\<S\>\"", ",", "\"\<U\>\"", ",", "\"\<C\>\"", ",", "\"\<C\>\"", ",", 
      "\"\<E\>\"", ",", "\"\<S\>\"", ",", "\"\<S\>\""}], "}"}], "Input"], 
   TraditionalForm]]],
 " as the argument to ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 ". Since we are only interested in the number of permutations, we apply ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " as well."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Permutations", "[", 
   RowBox[{"{", 
    RowBox[{
    "\"\<S\>\"", ",", "\"\<U\>\"", ",", "\"\<C\>\"", ",", "\"\<C\>\"", ",", 
     "\"\<E\>\"", ",", "\"\<S\>\"", ",", "\"\<S\>\""}], "}"}], "]"}], 
  "]"}]], "Input"],

Cell[BoxData["420"], "Output"]
}, Open  ]],

Cell["\<\
Observe that this gives the same result as the formula given in Theorem 3. \
\>", "Text"],

Cell[TextData[{
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " makes it easy to go a bit further than Theorem 3, which is restricted to \
the situation when you are permuting all ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects. To find the number of ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-permutations of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects where some objects are indistinguishable, you give ",
 StyleBox["{r}", "Input"],
 " as the second argument."
}], "Text"],

Cell["\<\
For example, the strings of length 3 that can be made from the letters of the \
word SUCCESS can be found as follows. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Permutations", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<S\>\"", ",", "\"\<U\>\"", ",", "\"\<C\>\"", ",", "\"\<C\>\"", ",", 
     "\"\<E\>\"", ",", "\"\<S\>\"", ",", "\"\<S\>\""}], "}"}], ",", 
   RowBox[{"{", "3", "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"U\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"U\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"U\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"C\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"C\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"C\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"C\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"E\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"E\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"E\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"S\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"S\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"S\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"S\"\>", ",", "\<\"S\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"S\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"S\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"S\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"C\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"C\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"C\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"E\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"U\"\>", ",", "\<\"E\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"S\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"S\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"S\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"S\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"U\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"U\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"U\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"C\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"C\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"C\"\>", ",", "\<\"E\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"E\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"E\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"C\"\>", ",", "\<\"E\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"S\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"S\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"S\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"U\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"U\"\>", ",", "\<\"C\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"C\"\>", ",", "\<\"S\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"C\"\>", ",", "\<\"U\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"E\"\>", ",", "\<\"C\"\>", ",", "\<\"C\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Applying ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " returns the number of such strings."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "%", "]"}]], "Input"],

Cell[BoxData["43"], "Output"]
}, Open  ]],

Cell[TextData[{
 "A related question is the number of strings with 3 or more letters that can \
be made from the letters of the word SUCCESS. To find this, we only have to \
change the second argument of ",
 StyleBox[ButtonBox["Permutations",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Permutations"], "FunctionLink"],
 " to ",
 StyleBox["{3,7}", "Input"],
 ", indicating that the length of the permutations should be allowed to range \
from 3 to 7 (the number of letters in SUCCESS)."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"Permutations", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "\"\<S\>\"", ",", "\"\<U\>\"", ",", "\"\<C\>\"", ",", "\"\<C\>\"", ",", 
      "\"\<E\>\"", ",", "\"\<S\>\"", ",", "\"\<S\>\""}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "7"}], "}"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["1247"], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Distinguishable Objects and Distinguishable Boxes", "Subsection",
 CellTags->"Section5d"],

Cell[TextData[{
 "Example 8 asks how many ways there are to distribute hands of 5 cards to \
each of four players from a deck of 52 cards. There are several ways to \
compute this value in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "."
}], "Text"],

Cell[TextData[{
 "First, we can use the expression in terms of combinations, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"C", "(", 
     RowBox[{"52", ",", "5"}], ")"}], 
    RowBox[{"C", "(", 
     RowBox[{"47", ",", "5"}], ")"}], 
    RowBox[{"C", "(", 
     RowBox[{"42", ",", "5"}], ")"}], 
    RowBox[{"C", "(", 
     RowBox[{"37", ",", "5"}], ")"}]}], TraditionalForm]]],
 ", by using ",
 StyleBox[ButtonBox["Binomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Binomial"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Binomial", "[", 
   RowBox[{"52", ",", "5"}], "]"}], "*", 
  RowBox[{"Binomial", "[", 
   RowBox[{"47", ",", "5"}], "]"}], "*", 
  RowBox[{"Binomial", "[", 
   RowBox[{"42", ",", "5"}], "]"}], "*", 
  RowBox[{"Binomial", "[", 
   RowBox[{"37", ",", "5"}], "]"}]}]], "Input"],

Cell[BoxData["1478262843475644020034240"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Second, we can use the formula from Theorem 4: ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"52", "!"}], 
    RowBox[{
     RowBox[{"5", "!"}], "\[CenterDot]", 
     RowBox[{"5", "!"}], "\[CenterDot]", 
     RowBox[{"5", "!"}], "\[CenterDot]", 
     RowBox[{"5", "!"}], "\[CenterDot]", 
     RowBox[{"32", "!"}]}]], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"52", "!"}], "/", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"5", "!"}], "*", 
    RowBox[{"5", "!"}], "*", 
    RowBox[{"5", "!"}], "*", 
    RowBox[{"5", "!"}], "*", 
    RowBox[{"32", "!"}]}], ")"}]}]], "Input"],

Cell[BoxData["1478262843475644020034240"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Finally, this same value can be computed using the ",
 StyleBox[ButtonBox["Multinomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Multinomial"], "FunctionLink"],
 " function. Recall Theorem 4 in the text asserts that the number of ways to \
place ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinguishable objects into boxes so that ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "1"], TraditionalForm]]],
 " objects are placed in box 1, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", 
    SubscriptBox["2", " "]], TraditionalForm]]],
 " objects are put in box 2, etc., is ",
 Cell[BoxData[
  FormBox[
   FractionBox[
    RowBox[{"n", "!"}], 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["n", "1"], "!"}], "\[CenterDot]", 
      RowBox[{
       SubscriptBox["n", "2"], "!"}]}], 
     RowBox[{
      SubscriptBox["\[CenterEllipsis]n", "k"], "!"}]}]], TraditionalForm]]],
 ". The ",
 StyleBox[ButtonBox["Multinomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Multinomial"], "FunctionLink"],
 " function takes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], ",", 
    SubscriptBox["n", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["n", "k"]}], TraditionalForm]]],
 " as arguments and applies the formula with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " computed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["n", "1"], "+", 
    SubscriptBox["n", "2"], "+", "\[CenterEllipsis]", "+", 
    SubscriptBox["n", "k"]}], TraditionalForm]]],
 ". So to compute the answer to Example 8, you enter the following."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Multinomial", "[", 
  RowBox[{"5", ",", "5", ",", "5", ",", "5", ",", "32"}], "]"}]], "Input"],

Cell[BoxData["1478262843475644020034240"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[{
 "Revising the ",
 StyleBox["Multinomial", "Input"],
 " Function"
}], "Subsubsection"],

Cell[TextData[{
 "It is common in questions about distributing distinguishable objects into \
distinguishable boxes that you want to distribute only some of the objects. \
In Example 8, for instance, not all of the cards are dealt to players. The ",
 StyleBox[ButtonBox["Multinomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Multinomial"], "FunctionLink"],
 " function requires that you include the remainder of the cards as an \
argument. Conceptually, you can think of making one more box to hold the \
objects that are not placed in any of the other boxes."
}], "Text"],

Cell[TextData[{
 "This is such a common occurrence, however, that it seems more natural to \
forget about this \[OpenCurlyDoubleQuote]discard box\[CloseCurlyDoubleQuote] \
and instead include the total number of objects. We will write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that will use the formula from Theorem 4 but will require the \
total number of objects as the first argument and calculate the size of the \
discard box."
}], "Text"],

Cell[TextData[{
 "We would like our function to, like ",
 StyleBox[ButtonBox["Multinomial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Multinomial"], "FunctionLink"],
 ", accept any number of arguments, rather than requiring the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["n", "i"], TraditionalForm]]],
 " to be collected in a list. That is, we would like to be able to compute \
the answer to Example 8 as follows."
}], "Text"],

Cell[BoxData[
 RowBox[{"myMultinomial", "[", 
  RowBox[{"52", ",", "5", ",", "5", ",", "5", ",", "5"}], "]"}]], "myCode"],

Cell["\<\
The first argument will be the total number of objects, and it should be \
followed by at least one integer indicating the sizes of the boxes.\
\>", "Text"],

Cell[TextData[{
 "To allow the function to accept arbitrary numbers of argument, we use the \
",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 " (",
 StyleBox["__", "Input"],
 ") pattern, entered with two underscores. In the past, \
we\[CloseCurlyQuote]ve used this inside braces as a way to specify that an \
argument must be a list of a certain kind, as in ",
 StyleBox["a:{__Integer}", "Input"],
 ". This will be similar, but without the braces."
}], "Text"],

Cell["The function definition will begin as shown below.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myMultinomial", "[", 
   RowBox[{"n_Integer", ",", "L__Integer"}], "]"}], ":="}]], "myCode"],

Cell[TextData[{
 "The difficulty arises when working with the sequence of integers matched by \
",
 StyleBox["L", "Input"],
 ". In particular, ",
 StyleBox["L", "Input"],
 " does not represent a list, it represents a sequence of arguments. So \
functions that accept a list argument, such as ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 ", will raise an error, as illustrated below."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"numberArgs", "[", "L__", "]"}], ":=", 
  RowBox[{"Length", "[", "L", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"numberArgs", "[", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"Length", "::", "argx"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"\[NoBreak]\\!\\(Length\\)\[NoBreak] called with \[NoBreak]\
\\!\\(4\\)\[NoBreak] arguments; 1 argument is expected. \\!\\(\\*ButtonBox[\\\
\"\[RightSkeleton]\\\", ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/argx\\\", ButtonNote -> \
\\\"Length::argx\\\"]\\)\"\>"}]], "Message", "MSG"],

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The way to deal with this is to wrap ",
 StyleBox["L", "Input"],
 " in braces before applying the function."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"numberArgs2", "[", "L__", "]"}], ":=", 
  RowBox[{"Length", "[", 
   RowBox[{"{", "L", "}"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"numberArgs2", "[", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["4"], "Output"]
}, Open  ]],

Cell[TextData[{
 "On the other hand, functions that normally accepts arbitrary numbers of \
arguments and must usually be used in conjunction with ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " to operate on a list, can be given the sequence directly."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sumArgs", "[", "L__", "]"}], ":=", 
  RowBox[{"Plus", "[", "L", "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sumArgs", "[", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["10"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Now that we know how to work with the ",
 StyleBox[ButtonBox["BlankSequence",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/BlankSequence"], "FunctionLink"],
 ", writing the ",
 StyleBox["myMultinomial", "Input"],
 " function is straightforward. We compute the size of the discard box by \
subtracting the first argument from the sum of the rest. The ",
 StyleBox[ButtonBox["Factorial",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Factorial"], "FunctionLink"],
 " function automatically threads over lists, so we can apply it to the list \
formed from ",
 StyleBox["L", "Input"],
 " and the discard box size. We need to use ",
 StyleBox[ButtonBox["Apply",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Apply"], "FunctionLink"],
 " (",
 StyleBox["@@", "Input"],
 ") with ",
 StyleBox[ButtonBox["Times",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Times"], "FunctionLink"],
 " in order to multiply the factorials, and then divide ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "!"}], TraditionalForm]]],
 " by that result."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myMultinomial", "[", 
   RowBox[{"n_Integer", ",", "L__Integer"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"discard", ",", "denomList", ",", "denom"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"discard", "=", 
      RowBox[{"n", "-", 
       RowBox[{"Plus", "[", "L", "]"}]}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"denomList", "=", 
      RowBox[{"Factorial", "[", 
       RowBox[{"{", 
        RowBox[{"L", ",", "discard"}], "}"}], "]"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"denom", "=", 
      RowBox[{"Times", "@@", "denomList"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"n", "!"}], "/", "denom"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"myMultinomial", "[", 
  RowBox[{"52", ",", "5", ",", "5", ",", "5", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["1478262843475644020034240"], "Output"]
}, Open  ]],

Cell["\<\
The reader is encouraged to eliminate the local variables and write a \
\[OpenCurlyDoubleQuote]one line\[CloseCurlyDoubleQuote] version of this \
function.\
\>", "Text"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Indistinguishable Objects and Distinguishable Boxes", "Subsection",
 CellTags->"Section5e"],

Cell["\<\
The text describes the correspondence between questions about placing \
indistinguishable objects into distinguishable boxes and about combination \
with repetition questions.\
\>", "Text"],

Cell[TextData[{
 "Example 9 asks how many ways 10 indistinguishable balls can be placed in 8 \
bins. We can use the ",
 StyleBox["CRep", "Input"],
 " function written earlier."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CRep", "[", 
  RowBox[{"8", ",", "10"}], "]"}]], "Input"],

Cell[BoxData["19448"], "Output"]
}, Open  ]],

Cell["\<\
It may seem that the arguments were reversed. Keep in mind that the \
connection to combinations with repetition is that you are selecting 10 bins \
from the 8 available bins, with repetition allowed.\
\>", "Text"],

Cell[CellGroupData[{

Cell["Compositions and Weak Compositions", "Subsubsection"],

Cell[TextData[{
 "We can also use the ",
 StyleBox[ButtonBox["Compositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/Compositions"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["NumberOfCompositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/NumberOfCompositions"], 
  "FunctionLink"],
 " functions to answer questions of this kind. A ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 "-composition of a positive integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a way of writing ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as the sum of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " positive integers where the order of the summands matters. For example, 4 \
has three distinct 2-compositions: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"3", "+", "1"}], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "+", "2"}], TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "+", "3"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "A weak composition is similar, but the terms in the sum are allowed to be \
0. Thus 4 has five distinct weak 2-compositions: ",
 Cell[BoxData[
  FormBox[
   RowBox[{"4", "+", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"0", "+", "4"}], TraditionalForm]]],
 " in addition to the three listed before."
}], "Text"],

Cell[TextData[{
 "Note that the weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-compositions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " correspond to the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-compositions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "r"}], TraditionalForm]]],
 ". For suppose that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "+", 
     SubscriptBox["x", "2"], "+", "\[CenterEllipsis]", "+", 
     SubscriptBox["x", "r"]}], "=", "n"}], TraditionalForm]]],
 " is a weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-composition. Then each ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "i"], TraditionalForm]]],
 " is nonnegative. So"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["x", "1"], "+", "1"}], ")"}], "+", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["x", "2"], "+", "1"}], ")"}], "+", "\[CenterEllipsis]", 
     "+", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["x", "r"], "+", "1"}], ")"}]}], "=", 
    RowBox[{"n", "+", "r"}]}], ","}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "and each ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "i"], "+", "1"}], TraditionalForm]]],
 " is positive, and hence this is a composition of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "r"}], TraditionalForm]]],
 ". Likewise, any ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-composition of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "+", "r"}], TraditionalForm]]],
 " can be transformed into a weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-composition by subtracting 1 from each term."
}], "Text"],

Cell[TextData[{
 "Also note that weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-compositions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " correspond to placing ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " indistinguishable balls into ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " distinguishable bins. Suppose ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "+", 
     SubscriptBox["x", "2"], "+", "\[CenterEllipsis]", "+", 
     SubscriptBox["x", "r"]}], "=", "n"}], TraditionalForm]]],
 " is a weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-composition of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". This can be identified with placing ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "1"], TraditionalForm]]],
 " of the objects into the first bin, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["x", "2"], TraditionalForm]]],
 " objects in the second bin, etc."
}], "Text"],

Cell[TextData[{
 "We now return to ",
 StyleBox[ButtonBox["Compositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/Compositions"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["NumberOfCompositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/NumberOfCompositions"], 
  "FunctionLink"],
 ". These are functions contained in the ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " package, and are not available without loading that package by executing a \
",
 StyleBox[ButtonBox["Needs",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Needs"], "FunctionLink"],
 " statement. "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Needs", "[", "\"\<Combinatorica`\>\"", "]"}]], "Input",
 InitializationCell->True],

Cell[BoxData[
 RowBox[{
  StyleBox[
   RowBox[{"General", "::", "compat"}], "MessageName"], 
  RowBox[{
  ":", " "}], "\<\"Combinatorica Graph and Permutations functionality has \
been superseded by preloaded functionality. The package now being loaded may \
conflict with this. Please see the Compatibility Guide for details.\"\>"}]], \
"Message", "MSG"]
}, Open  ]],

Cell[TextData[{
 "You may safely ignore the compatibility warning in this context. Some of \
the functionality of the ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " package has been superseded by new functionality in the main kernel in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ". However, this does not affect the functions we will be using."
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["Compositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/Compositions"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["NumberOfCompositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/NumberOfCompositions"], 
  "FunctionLink"],
 " functions accept the same arguments: the integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ". ",
 StyleBox[ButtonBox["NumberOfCompositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/NumberOfCompositions"], 
  "FunctionLink"],
 " outputs the number of ",
 StyleBox["weak",
  FontSlant->"Italic"],
 " ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-compositions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", while ",
 StyleBox[ButtonBox["Compositions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/Compositions"], "FunctionLink"],
 " returns a list of them. Observe that the names ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " uses for these functions is slightly at odds with the usual terminology."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfCompositions", "[", 
  RowBox[{"4", ",", "2"}], "]"}]], "Input"],

Cell[BoxData["5"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Compositions", "[", 
  RowBox[{"4", ",", "2"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"0", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "0"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The number of ways to place ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " indistinguishable objects in ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " distinguishable boxes is the same as the number of weak ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-compositions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". Consequently, we can determine the number of ways 10 balls can be placed \
in 8 bins, the same question as before, as follows."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumberOfCompositions", "[", 
  RowBox[{"10", ",", "8"}], "]"}]], "Input"],

Cell[BoxData["19448"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Distinguishable Objects and Indistinguishable Boxes", "Subsection",
 CellTags->"Section5f"],

Cell[TextData[{
 "As described in the text, the number of ways to place ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " distinguishable objects in ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " indistinguishable boxes is given by the Stirling numbers of the second \
kind, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"S", "(", 
    RowBox[{"n", ",", "k"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The function ",
 StyleBox[ButtonBox["StirlingS2",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StirlingS2"], "FunctionLink"],
 " computes the Stirling number of the second kind. This function requires \
two arguments, the number of objects and the number of boxes. For example, \
the expression below computes the number of ways to put seven different \
employees in 4 different offices when each office must not be empty."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StirlingS2", "[", 
  RowBox[{"7", ",", "4"}], "]"}]], "Input"],

Cell[BoxData["350"], "Output"]
}, Open  ]],

Cell["\<\
In order to compute the number of ways to assign the 7 employees to the 4 \
offices and allow empty offices, we must add the number of ways to assign all \
7 employees to 1 office, to 2 offices, to 3 offices, and to 4 offices.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Sum", "[", 
  RowBox[{
   RowBox[{"StirlingS2", "[", 
    RowBox[{"7", ",", "i"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData["715"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generating Assignments of Employees to Offices", "Subsubsection",
 PageBreakWithin->False],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["StirlingS2",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/StirlingS2"], "FunctionLink"],
 " function tells us how many ways there are to place distinguishable objects \
in indistinguishable boxes. In this subsection, we will create a function to \
list these. However, note that the ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " package has a function, similar to the one we will create by hand, called ",
 StyleBox[ButtonBox["SetPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/SetPartitions"], "FunctionLink"],
 "."
}], "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],

Cell[TextData[{
 "To create a function that will list the possible assignments of \
distinguishable objects to boxes, we rely on the following observations. \
First, as indicated in the text, a choice of distinguishable objects to \
indistinguishable boxes can be modeled as a set of subsets. For instance,",
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"A", ",", "C"}], "}"}], ",", 
     RowBox[{"{", "D", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"B", ",", "E"}], "}"}]}], "}"}], TraditionalForm]]],
 " represents the assignment of A and C to one box, D to a box of its own, \
and B and E to another box. The set of subsets must not contain the empty set \
and must be such that the union of the subsets be the entire collection of \
objects."
}], "Text"],

Cell[TextData[{
 "We can produce such assignments recursively. The basis step is that there \
is only one way to assign ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects to 1 box and there are no ways to assign ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "n"}], TraditionalForm]]],
 " (under the requirement that no box be empty). To assign ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[LessEqual]", "n"}], TraditionalForm]]],
 ", proceed as follows."
}], "Text"],

Cell[TextData[{
 "First, find all assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " boxes and update each assignment by placing object ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in a box by itself. In terms of the set representation, given ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["B", "1"], ",", 
     SubscriptBox["B", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["B", 
      RowBox[{"k", "-", "1"}]]}], "}"}], TraditionalForm]]],
 ", we produce ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["B", "1"], ",", 
     SubscriptBox["B", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["B", 
      RowBox[{"k", "-", "1"}]], ",", 
     RowBox[{"{", "n", "}"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Second, find all assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes. For each such assignment ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["B", "1"], ",", 
     SubscriptBox["B", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["B", "k"]}], "}"}], TraditionalForm]]],
 ", produce the following ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " assignments of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects to the ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes:"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      SubscriptBox["B", "1"], "\[Union]", 
      RowBox[{"{", "n", "}"}]}], ",", 
     SubscriptBox["B", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["B", "k"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["B", "1"], ",", 
     RowBox[{
      SubscriptBox["B", "2"], "\[Union]", 
      RowBox[{"{", "n", "}"}]}], ",", 
     SubscriptBox["B", "3"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["B", "k"]}], "}"}], ",", 
   RowBox[{"\[Ellipsis]", 
    RowBox[{"{", 
     RowBox[{
      SubscriptBox["B", "1"], ",", 
      SubscriptBox["B", "2"], ",", "\[Ellipsis]", ",", 
      RowBox[{
       SubscriptBox["B", "k"], "\[Union]", 
       RowBox[{"{", "n", "}"}]}]}], "}"}]}]}], TraditionalForm]], "Equation"],

Cell["\<\
The assignments of objects to boxes produced by the two methods above produce \
all assignments. The following function implements this algorithm.\
\>", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"makeStirling2", "[", 
    RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", 
    RowBox[{"k", ">", "0"}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "A", ",", "k1boxes", ",", "kboxes", ",", "B", ",", "new", ",", "i"}], 
     "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"Which", "[", 
      RowBox[{
       RowBox[{"k", "\[Equal]", "1"}], ",", "\[IndentingNewLine]", 
       RowBox[{"A", "=", 
        RowBox[{"{", 
         RowBox[{"{", 
          RowBox[{"Range", "[", "n", "]"}], "}"}], "}"}]}], ",", 
       "\[IndentingNewLine]", 
       RowBox[{"k", ">", "n"}], ",", "\[IndentingNewLine]", 
       RowBox[{"A", "=", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", "True", ",", 
       "\[IndentingNewLine]", 
       RowBox[{"(*", " ", 
        RowBox[{"the", " ", 
         RowBox[{"recursion", ":"}]}], " ", "*)"}], "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"A", "=", 
         RowBox[{"{", "}"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"n", "-", 
          RowBox[{"1", " ", "objects", " ", "in", " ", "k"}], "-", 
          RowBox[{"1", " ", "boxes"}]}], " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"k1boxes", "=", 
         RowBox[{"makeStirling2", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", 
           RowBox[{"k", "-", "1"}]}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{
           RowBox[{"new", "=", 
            RowBox[{"Union", "[", 
             RowBox[{"B", ",", 
              RowBox[{"{", 
               RowBox[{"{", "n", "}"}], "}"}]}], "]"}]}], ";", 
           RowBox[{"AppendTo", "[", 
            RowBox[{"A", ",", "new"}], "]"}]}], "\[IndentingNewLine]", ",", 
          RowBox[{"{", 
           RowBox[{"B", ",", "k1boxes"}], "}"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"(*", " ", 
         RowBox[{"n", "-", 
          RowBox[{"1", " ", "objects", " ", "in", " ", "k", " ", "boxes"}]}], 
         " ", "*)"}], "\[IndentingNewLine]", 
        RowBox[{"kboxes", "=", 
         RowBox[{"makeStirling2", "[", 
          RowBox[{
           RowBox[{"n", "-", "1"}], ",", "k"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"Do", "[", 
         RowBox[{
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"i", "=", "1"}], ",", 
            RowBox[{"i", "\[LessEqual]", "k"}], ",", 
            RowBox[{"i", "++"}], ",", "\[IndentingNewLine]", 
            RowBox[{
             RowBox[{"new", "=", 
              RowBox[{"ReplacePart", "[", 
               RowBox[{"B", ",", 
                RowBox[{"i", "\[Rule]", 
                 RowBox[{"Append", "[", 
                  RowBox[{
                   RowBox[{"B", "[", 
                    RowBox[{"[", "i", "]"}], "]"}], ",", "n"}], "]"}]}]}], 
               "]"}]}], ";", "\[IndentingNewLine]", 
             RowBox[{"AppendTo", "[", 
              RowBox[{"A", ",", "new"}], "]"}]}]}], "\[IndentingNewLine]", 
           "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"B", ",", "kboxes"}], "}"}]}], "]"}]}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", "A"}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Let us analyze the function. It accepts ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " as parameters and returns the list of all possible assignments of \
distinguishable objects to indistinguishable boxes. The symbol ",
 StyleBox["A", "Input"],
 " stores the list that will ultimately be output."
}], "Text"],

Cell[TextData[{
 "In the case that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "1"}], TraditionalForm]]],
 ", there is only one possible assignment, all objects are assigned to the \
single box. This assignment is represented by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"{", 
     RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], "}"}], 
   TraditionalForm]]],
 ", since an assignment corresponds to a set of subsets. The function sets \
the output symbol ",
 StyleBox["A", "Input"],
 " to the list consisting of this single assignment when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "1"}], TraditionalForm]]],
 ". Recall the ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " function applied to a positive integer produces the list of integers from \
1 to that value."
}], "Text"],

Cell[TextData[{
 "If ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "n"}], TraditionalForm]]],
 ", then there are no valid assignments and the function sets the output \
symbol ",
 StyleBox["A", "Input"],
 " to the empty set."
}], "Text"],

Cell[TextData[{
 "Otherwise, the symbol ",
 StyleBox["A", "Input"],
 " is initialized to the empty set. Recall that there are two recursive \
steps. First expanding on the assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " boxes. And then expanding on the assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes."
}], "Text"],

Cell[TextData[{
 "For the first part, we assign the symbol ",
 StyleBox["k1boxes", "Input"],
 " to the set of assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " boxes. For each such assignment, that is each ",
 StyleBox["B", "Input"],
 " in ",
 StyleBox["k1boxes", "Input"],
 ", we add ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " in its own box, e.g., for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
     RowBox[{"{", "2", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "6"}], "}"}]}], "}"}], TraditionalForm]]],
 ", we would add 7:"
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
      RowBox[{"{", "2", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"4", ",", "6"}], "}"}]}], "}"}], "\[Union]", 
    RowBox[{"{", 
     RowBox[{"{", "7", "}"}], "}"}]}], "=", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
      RowBox[{"{", "2", "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"4", ",", "6"}], "}"}], ",", 
      RowBox[{"{", "7", "}"}]}], "}"}], "."}]}], TraditionalForm]], "Equation"],

Cell[TextData[{
 "This ",
 StyleBox["new", "Input"],
 " assignment is then added to ",
 StyleBox["A", "Input"],
 "."
}], "Text"],

Cell[TextData[{
 "In the second part, we assign ",
 StyleBox["kboxes", "Input"],
 " to the set of assignments of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "1"}], TraditionalForm]]],
 " objects to ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes. For each such assignment ",
 StyleBox["B", "Input"],
 ", we consider each of the ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes in turn and add ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to that box. For instance, the assignment ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "3"}], "}"}], ",", 
     RowBox[{"{", "1", "}"}], ",", 
     RowBox[{"{", "5", "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"4", ",", "6"}], "}"}]}], "}"}], TraditionalForm]]],
 " would generate the four assignments:"
}], "Text"],

Cell[BoxData[{
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3", ",", "7"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", "5", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "6"}], "}"}]}], "}"}], 
  TraditionalForm], "\[LineSeparator]", 
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"1", ",", "7"}], "}"}], ",", 
    RowBox[{"{", "5", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "6"}], "}"}]}], "}"}], 
  TraditionalForm], "\[LineSeparator]", 
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"5", ",", "7"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "6"}], "}"}]}], "}"}], 
  TraditionalForm], "\[LineSeparator]", 
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"2", ",", "3"}], "}"}], ",", 
    RowBox[{"{", "1", "}"}], ",", 
    RowBox[{"{", "5", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"4", ",", "6", ",", "7"}], "}"}]}], "}"}], 
  TraditionalForm]}], "Equation"],

Cell[TextData[{
 "To create these four ",
 StyleBox["new", "Input"],
 " assignments from the initial assignment ",
 StyleBox["B", "Input"],
 ", we use the ",
 StyleBox[ButtonBox["ReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplacePart"], "FunctionLink"],
 " function. In its simplest form, as we use it here, ",
 StyleBox[ButtonBox["ReplacePart",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ReplacePart"], "FunctionLink"],
 " takes two arguments. The first argument is an expression to be \
manipulated, in this case ",
 StyleBox["B", "Input"],
 ", the original list representing an assignment of objects to boxes. The \
second argument is a rule of the form ",
 Cell[BoxData[
  RowBox[{
   StyleBox["index", "CodeVar"], "->", 
   StyleBox["replacement", "CodeVar"]}]], "myCode"],
 ". The ",
 StyleBox["index", "CodeVar"],
 " is the location within ",
 StyleBox["B", "Input"],
 " that is to be substituted with the ",
 StyleBox["replacement", "CodeVar"],
 " expression. In ",
 StyleBox["makeStirling2", "Input"],
 ", we use a ",
 StyleBox[ButtonBox["For",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/For"], "FunctionLink"],
 " loop variable as the index and the replacement expression is obtained from \
",
 StyleBox[ButtonBox["Append",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Append"], "FunctionLink"],
 " to add the value ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to the box at that index."
}], "Text"],

Cell[TextData[{
 "Compare the result of our function to the solution of Example 10 in Section \
6.5 of the text. We use ",
 StyleBox[ButtonBox["Column",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Column"], "FunctionLink"],
 " to put each assignment on its own line."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"makeStirling2", "[", 
   RowBox[{"4", ",", "3"}], "]"}], "//", "Column"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "3", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", "3", "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "3", "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}]}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output"]
}, Open  ]],

Cell["\<\
Except for using the integers 1 through 4 instead of the letters A through D, \
the output above is the same as the six ways listed in the text for placing \
the four employees in 3 offices. \
\>", "Text"],

Cell[TextData[{
 "To produce all 14 ways to assign the 4 employees to 3 offices with each \
office containing any number of employees, we need to loop over the different \
values of ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ". Using ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 " to vary the number of offices results in a list, each element of which is \
a list of results. Using ",
 StyleBox[ButtonBox["Flatten",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Flatten"], "FunctionLink"],
 " with second argument 1 combines this list of lists into a single list of \
results. The 1 as the second argument indicates that only the top level of \
lists are to be flattened. The result can be displayed with ",
 StyleBox[ButtonBox["Column",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Column"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Flatten", "[", 
   RowBox[{
    RowBox[{"Table", "[", 
     RowBox[{
      RowBox[{"makeStirling2", "[", 
       RowBox[{"4", ",", "k"}], "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"k", ",", "1", ",", "3"}], "}"}]}], "]"}], ",", "1"}], "]"}], "//",
   "Column"}]], "Input"],

Cell[BoxData[
 TagBox[GridBox[{
    {
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "3", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2", ",", "4"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "2", "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "4"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "4"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3", ",", "4"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "3", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "2"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", "4", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "3"}], "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", "3", "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"2", ",", "4"}], "}"}], ",", 
       RowBox[{"{", "3", "}"}]}], "}"}]},
    {
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", "1", "}"}], ",", 
       RowBox[{"{", "2", "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"3", ",", "4"}], "}"}]}], "}"}]}
   },
   DefaultBaseStyle->"Column",
   GridBoxAlignment->{"Columns" -> {{Left}}},
   GridBoxItemSize->{"Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
  "Column"]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Indistinguishable Objects and Indistinguishable Boxes", "Subsection",
 CellTags->"Section5g"],

Cell[TextData[{
 "As described in the text, distributing ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " indistinguishable objects into ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " indistinguishable boxes is identical to forming a partition of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " into ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " positive integers. A partition of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " into ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " positive integers is a sum ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", 
    RowBox[{
     SubscriptBox["a", "1"], "+", 
     SubscriptBox["a", "2"], "+", "\[CenterEllipsis]", "+", 
     SubscriptBox["a", "k"]}]}], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "1"], "\[GreaterEqual]", 
    SubscriptBox["a", "2"], "\[GreaterEqual]", "\[CenterEllipsis]", 
    "\[GreaterEqual]", 
    SubscriptBox["a", "k"], ">", "0"}], TraditionalForm]]],
 ". "
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " functions ",
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 " and ",
 StyleBox[ButtonBox["PartitionsP",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PartitionsP"], "FunctionLink"],
 " are used to form and count partitions of integers. With one argument, a \
nonnegative integer ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 StyleBox[ButtonBox["PartitionsP",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PartitionsP"], "FunctionLink"],
 " returns the total number of partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " into as many as ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " boxes. Likewise, ",
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 " applied to one argument returns a list containing lists representing \
partitions of its argument."
}], "Text"],

Cell["\<\
For example, the expressions below compute the number of partitions of 7 and \
lists all the partitions of 7. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartitionsP", "[", "7", "]"}]], "Input"],

Cell[BoxData["15"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", "7", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "7", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}],
     "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The ",
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 " function also accepts a second argument, which specifies the number of \
elements allowed to appear in a partition. For example, to answer the \
question: how many can you distribute 7 indistinguishable balls in up to 3 \
identical boxes, we would give 3 as the second argument to ",
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", 
  RowBox[{"7", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", "7", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"6", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "This second argument, which specifies the length of the partitions, can \
take on many of the usual forms: ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", as we saw, indicates the maximum length; ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "n", "}"}], TraditionalForm]]],
 " limits the output to those partitions of length exactly ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"n", ",", "m"}], "}"}], TraditionalForm]]],
 " produces the partitions from length ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 ". Also, the symbol ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " can be used to list all of the partitions, just as if no second argument \
were given."
}], "Text"],

Cell["\<\
For example, the following produces the partitions of 7 whose lengths are \
between 3 and 5.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", 
  RowBox[{"7", ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"5", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "1", ",", "1"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 " also accepts a third optional argument to control what values may appear \
in the partitions. Note that in order to use the this option, the second \
argument, specifying the length, must be given. To allow unrestricted \
lengths, you should give ",
 StyleBox[ButtonBox["All",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/All"], "FunctionLink"],
 " as the second argument. The third argument has only one form: a list of \
the allowable values. "
}], "Text"],

Cell["\<\
For example, to partition 15 using only 2, 3, and 4, you would enter the \
following.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", 
  RowBox[{"15", ",", "All", ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4"}], "}"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"4", ",", "4", ",", "4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "4", ",", "3", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "3", ",", "3", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "2", ",", "2", ",", "2", ",", "2"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "3", ",", "3", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "3", ",", "2", ",", "2", ",", "2"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2", ",", "2"}],
     "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "By using ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " in the third argument, you can easily specify a maximum allowable value, \
or another range of values. For example, to answer the question: how many \
ways are there to distribute 7 indistinguishable balls in up to 5 identical \
boxes when each box can hold at most 4 objects, you would given ",
 StyleBox["Range[4]", "Input"],
 " as the third argument."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", 
  RowBox[{"7", ",", "5", ",", 
   RowBox[{"Range", "[", "4", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"4", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "1", ",", "1"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To determine the number, without outputting the list, simply apply ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 ". "
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"IntegerPartitions", "[", 
   RowBox[{"7", ",", "5", ",", 
    RowBox[{"Range", "[", "4", "]"}]}], "]"}], "]"}]], "Input"],

Cell[BoxData["9"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that these optional arguments are not available for ",
 StyleBox[ButtonBox["PartitionsP",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PartitionsP"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell["Equivalence of Maximum Length and Maximum Value", "Subsubsection"],

Cell[TextData[{
 "The partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " with at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects in a box are in one-to-one correspondence with the partitions of \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " into at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " boxes. To understand why, consider the partition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2", ",", "1"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Think about placing the 8 objects in boxes according to the partition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "2", ",", "2", ",", "1"}], "}"}], TraditionalForm]]],
 ". (The diagram shown below is a Ferrers diagram. A version of this diagram \
can be created with the function ",
 StyleBox[ButtonBox["FerrersDiagram",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/FerrersDiagram"], "FunctionLink"],
 " in the ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " package.)"
}], "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {"X", "X", "X", "X"},
   {"X", "X", "X", " "},
   {"X", " ", " ", " "}
  },
  GridBoxDividers->{
   "Columns" -> {{True}}, "ColumnsIndexed" -> {}, 
    "Rows" -> {True, {False}, True}, "RowsIndexed" -> {}}]]]], "Text"],

Cell["\<\
Instead of thinking about the columns as the boxes, we can instead consider \
the rows as boxes. \
\>", "Text"],

Cell[TextData[Cell[BoxData[GridBox[{
   {"X", "X", "X", "X"},
   {"X", "X", "X", " "},
   {"X", " ", " ", " "}
  },
  GridBoxDividers->{
   "Columns" -> {True, {False}, True}, "ColumnsIndexed" -> {}, 
    "Rows" -> {{True}}, "RowsIndexed" -> {}}]]]], "Text"],

Cell[TextData[{
 "Now the 8 objects are contained in three boxes. One box (the top row) has 4 \
objects, another (the middle row) has 3 objects, and the last box (the bottom \
row) has one object. In other words, we've partitioned 8 as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"4", ",", "3", ",", "1"}], "}"}], TraditionalForm]]],
 ". This partition is said to be the transpose of the first. (Note that you \
can also think about forming the transpose by reflecting it across its \
diagonal.)"
}], "Text"],

Cell[TextData[{
 "The ",
 StyleBox["Combinatorica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["TransposePartition",
  BaseStyle->"Link",
  ButtonData->"paclet:Combinatorica/ref/TransposePartition"], 
  "FunctionLink"],
 " will compute the transpose of a given partition."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TransposePartition", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "2", ",", "2", ",", "1"}], "}"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "3", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "We began with a partition whose maximum entry was 3 and found that its \
transpose was a partition into 3 boxes. It is always the case that the \
transpose of a partition with maximum ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " is a partition into ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " boxes. Moreover, this correspondence is one-to-one. We leave it to the \
reader to prove these facts."
}], "Text"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Generating Partitions", "Subsubsection"],

Cell[TextData[{
 "Here we will describe how to generate partitions and use this description \
to generalize the ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function ",
 StyleBox[ButtonBox["PartitionsP",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PartitionsP"], "FunctionLink"],
 " with a second argument limiting the maximum number in the partition."
}], "Text"],

Cell[TextData[{
 "We will describe how to recursively form the partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " objects when each box can hold at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects. The basis cases are: when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "1"}], TraditionalForm]]],
 ", there is only one partition of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", the partition consisting of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " 1s; when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 ", there is only one partition, the empty partition ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "To determine the partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " when each box can hold at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects, with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "1"}], TraditionalForm]]],
 ", proceed as follows. First, determine all partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " when each box can hold at most ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " objects. These partitions are also partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " satisfying the requirement that each box holds at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects."
}], "Text"],

Cell[TextData[{
 "Second, provided that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "-", "k"}], "\[GreaterEqual]", "0"}], TraditionalForm]]],
 ", determine all partitions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "k"}], TraditionalForm]]],
 " when each box can hold at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects, and prepend ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " to each partition. For example, with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "7"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "3"}], TraditionalForm]]],
 ", we have ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "-", "k"}], "=", "4"}], TraditionalForm]]],
 " and the partitions of 4 with each box holding at most 3 are:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegerPartitions", "[", 
  RowBox[{"4", ",", "All", ",", 
   RowBox[{"Range", "[", "3", "]"}]}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"3", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "By prepending 3 to each of these partitions, we obtain ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"{", 
     RowBox[{"3", ",", "3", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "2", ",", "2"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"3", ",", "1", ",", "1", ",", "1", ",", "1"}], "}"}]}], 
   TraditionalForm]]],
 ", which are each partitions of 7 with each box having at most 3 objects."
}], "Text"],

Cell[TextData[{
 "Combining the partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " with each box holding at most ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "-", "1"}], TraditionalForm]]],
 " objects with the partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " formed by appending ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " to the partitions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "-", "k"}], TraditionalForm]]],
 " with each box holding at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects produces all partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " with each box holding at most ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " objects. Setting ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "n"}], TraditionalForm]]],
 " produces all partitions of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 "."
}], "Text"],

Cell["\<\
It is an exercise to implement this algorithm to generate partitions.\
\>", "Text"],

Cell[TextData[{
 "We can use the above to generalize ",
 StyleBox[ButtonBox["PartitionsP",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/PartitionsP"], "FunctionLink"],
 ". We will define a function ",
 StyleBox["myPartitionsP", "Input"],
 ". This function will be defined recursively. The basis cases are when ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "=", "1"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "0"}], TraditionalForm]]],
 ", there is one partition."
}], "Text"],

Cell[BoxData[{
 RowBox[{
  RowBox[{
   RowBox[{"myPartitionsP", "[", 
    RowBox[{"_", ",", "1"}], "]"}], ":=", "1"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"myPartitionsP", "[", 
   RowBox[{"0", ",", "_"}], "]"}], ":=", "1"}]}], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "<", "0"}], TraditionalForm]]],
 " or ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "<", "1"}], TraditionalForm]]],
 ", there are no partitions."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myPartitionsP", "[", 
    RowBox[{"n_", ",", "k_"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", "<", "0"}], "||", 
    RowBox[{"k", "<", "1"}]}]}], ":=", "0"}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Provided that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", ">", "0"}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "1"}], TraditionalForm]]],
 ", the discussion above leads us to the following recursive definition."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"myPartitionsP", "[", 
    RowBox[{"n_Integer", ",", "k_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", 
    RowBox[{"k", ">", "1"}]}]}], ":=", 
  RowBox[{
   RowBox[{"myPartitionsP", "[", 
    RowBox[{"n", ",", 
     RowBox[{"k", "-", "1"}]}], "]"}], "+", 
   RowBox[{"myPartitionsP", "[", 
    RowBox[{
     RowBox[{"n", "-", "k"}], ",", "k"}], "]"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We see that the value of this function coincides with the number of \
partitions produced by ",
 StyleBox[ButtonBox["IntegerPartitions",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/IntegerPartitions"], "FunctionLink"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", 
  RowBox[{"IntegerPartitions", "[", 
   RowBox[{"20", ",", "5"}], "]"}], "]"}]], "Input"],

Cell[BoxData["192"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"myPartitionsP", "[", 
  RowBox[{"20", ",", "5"}], "]"}]], "Input"],

Cell[BoxData["192"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We can easily extend our function to compute all partitions of an integer \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", as follows."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"myPartitionsP", "[", "n_", "]"}], ":=", 
  RowBox[{"myPartitionsP", "[", 
   RowBox[{"n", ",", "n"}], "]"}]}]], "Input"],

Cell["Note that the results agree with the built-in function.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"myPartitionsP", "[", "15", "]"}]], "Input"],

Cell[BoxData["176"], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartitionsP", "[", "15", "]"}]], "Input"],

Cell[BoxData["176"], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["6.6 Generating Permutations and Combinations", "Section",
 CellTags->"Section6"],

Cell["\<\
In this section, we will implement Algorithm 1 from Section 6.6 of the text \
for generating the next permutation in lexicographic order. Implementing \
Algorithms 2 and 3 will be left as exercises for the reader. \
\>", "Text"],

Cell[CellGroupData[{

Cell[TextData[StyleBox["interchange", "Input"]], "Subsection",
 CellTags->"Section6a"],

Cell["\<\
Before implementing Algorithm 1, we will first write a function to \
interchange two elements in a list. This will be called by the function for \
generating permutations. \
\>", "Text"],

Cell[TextData[{
 "The ",
 StyleBox["interchange", "Input"],
 " function will require three parameters: the list and two integers \
representing the indices to be swapped."
}], "Text"],

Cell["\<\
The function operates as follows. A copy of the list is made. The element in \
the list at the first position to be swapped is assigned to a local variable. \
Then the first position is assigned to the value in the second position. \
Finally, the second position is assigned to the value stored in the temporary \
name and the list is returned.\
\>", "Text"],

Cell["Here is the implementation and an example of applying it. ", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"interchange", "[", 
    RowBox[{"L_List", ",", "i_Integer", ",", "j_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"0", "<", "i", "\[LessEqual]", 
     RowBox[{"Length", "[", "L", "]"}]}], "&&", 
    RowBox[{"0", "<", "j", "\[LessEqual]", 
     RowBox[{"Length", "[", "L", "]"}]}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"l", "=", "L"}], ",", "temp"}], "}"}], ",", 
    "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"temp", "=", 
      RowBox[{"l", "[", 
       RowBox[{"[", "i", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"l", "[", 
       RowBox[{"[", "i", "]"}], "]"}], "=", 
      RowBox[{"l", "[", 
       RowBox[{"[", "j", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"l", "[", 
       RowBox[{"[", "j", "]"}], "]"}], "=", "temp"}], ";", 
     "\[IndentingNewLine]", "l"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"interchange", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\"", ",", "\"\<d\>\"", ",", 
     "\"\<e\>\"", ",", "\"\<f\>\""}], "}"}], ",", "2", ",", "5"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"a\"\>", ",", "\<\"e\"\>", ",", "\<\"c\"\>", ",", "\<\"d\"\>", 
   ",", "\<\"b\"\>", ",", "\<\"f\"\>"}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell[TextData[StyleBox["nextPermutation", "Input"]], "Subsection",
 CellTags->"Section6b"],

Cell[TextData[{
 "The input to the function will be a permutation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["a", "1"], ",", 
     SubscriptBox["a", "2"], ",", "\[Ellipsis]", ",", 
     SubscriptBox["a", "n"]}], "}"}], TraditionalForm]]],
 " of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 ". Algorithm 1 consists of three steps: finding the largest ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], "<", 
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]]}], TraditionalForm]]],
 "; finding the smallest ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " to the right of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " and interchanging ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 "; and putting the elements in positions ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "+", "1"}], TraditionalForm]]],
 " and beyond in increasing order."
}], "Text"],

Cell[TextData[{
 "The first step comprises the first four lines of the body of Algorithm 1 in \
the text, ending with the first comment. The index ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " is initialized to the next to last index in the permutation. A ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop is used to conduct the search. The body of the loop decreases the \
value of ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " by one, and it is controlled by the condition ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], ">", 
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]]}], TraditionalForm]]],
 ". When the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop terminates, it will be the case that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], "<", 
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]]}], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " is the largest index for which that is true. Consequently, ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]], ">", 
    SubscriptBox["a", 
     RowBox[{"j", "+", "2"}]], ">", "\[CenterEllipsis]", ">", 
    SubscriptBox["a", "n"]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "The second step is to find the smallest ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " to the right of and larger than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " and interchange the two. Since we are guaranteed that the elements to the \
right of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " are in increasing order, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 " is the smallest element to the right of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 " is the next smallest, and so on. We are again searching from the right. \
Initialize ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ". A ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop is used to decrease ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " by one so long as ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], ">", 
    SubscriptBox["a", "k"]}], TraditionalForm]]],
 ". When the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop terminates, ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " will be such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], "<", 
    SubscriptBox["a", "k"]}], TraditionalForm]]],
 ". Note that the loop is guaranteed to stop with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", ">", "j"}], TraditionalForm]]],
 " since ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", "j"], "<", 
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]]}], TraditionalForm]]],
 ". Once ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " has been identified, we interchange ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " using the ",
 StyleBox["interchange", "Input"],
 " function."
}], "Text"],

Cell[TextData[{
 "The third step is to put the elements of the permutation to the right of \
position ",
 Cell[BoxData[
  FormBox["j", TraditionalForm]]],
 " in increasing order. Note that before the interchange, ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"j", "+", "1"}]], TraditionalForm]]],
 " through ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 " were in decreasing order. After the interchange of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 ", the tail end of the permutation remains in decreasing order. This is \
because ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " was smaller than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 ", but ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " was the smallest of the entries bigger than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 ". Thus all of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"k", "+", "1"}]], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", "n"]}], TraditionalForm]]],
 " are smaller than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 ", and all of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["a", 
     RowBox[{"j", "+", "1"}]], ",", "\[Ellipsis]", ",", 
    SubscriptBox["a", 
     RowBox[{"k", "-", "1"}]]}], TraditionalForm]]],
 " are larger than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 " which is larger than ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 ". Therefore, "
}], "Text"],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["a", 
    RowBox[{"j", "+", "1"}]], ",", "\[Ellipsis]", ",", 
   SubscriptBox["a", 
    RowBox[{"k", "-", "1"}]], ",", 
   SubscriptBox["a", "j"], ",", 
   SubscriptBox["a", 
    RowBox[{"k", "+", "1"}]], ",", "\[Ellipsis]", ",", 
   SubscriptBox["a", "n"]}], TraditionalForm]], "Equation"],

Cell["is in decreasing order.", "Text"],

Cell[TextData[{
 "To put the tail in increasing order, we follow the instructions in the \
pseudocode in the textbook that follows the interchange of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "j"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "k"], TraditionalForm]]],
 ". Variables ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " are initialized to ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   RowBox[{"j", "+", "1"}], TraditionalForm]]],
 ", respectively. Provided that ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " remains larger than ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 ", we interchange ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "r"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "s"], TraditionalForm]]],
 " and then decrease ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " by 1 and increase ",
 Cell[BoxData[
  FormBox["s", TraditionalForm]]],
 " by 1. This has the effect of swapping ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"j", "+", "1"}]], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", "n"], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"j", "+", "2"}]], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"n", "-", "1"}]], TraditionalForm]]],
 ", then ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"j", "+", "3"}]], TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["a", 
    RowBox[{"n", "-", "2"}]], TraditionalForm]]],
 ", etc."
}], "Text"],

Cell["\<\
Once the tail is in increasing order, the result is the new permutation and \
it is returned. \
\>", "Text"],

Cell[TextData[{
 "We need to add to the function two tests to ensure that the input is valid. \
First, as a ",
 StyleBox[ButtonBox["Condition",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Condition"], "FunctionLink"],
 " (",
 StyleBox["/;", "Input"],
 ") to the function definition, we ensure that the input is a permutation of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 ". We do this by sorting the input, using the ",
 StyleBox[ButtonBox["Sort",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sort"], "FunctionLink"],
 " function, and comparing it with the result of applying ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " to the ",
 StyleBox[ButtonBox["Length",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Length"], "FunctionLink"],
 " of the input list. Second, as the first expression in the function, we \
check to see if the input is the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"n", ",", 
     RowBox[{"n", "-", "1"}], ",", "\[Ellipsis]", ",", "2", ",", "1"}], "}"}],
    TraditionalForm]]],
 ", which is the final permutation in the canonical order. The ",
 StyleBox[ButtonBox["Reverse",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reverse"], "FunctionLink"],
 " function, applied to a list, simply reverses the order of the list. This \
is used in conjunction with ",
 StyleBox[ButtonBox["Range",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Range"], "FunctionLink"],
 " to produce the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"n", ",", 
     RowBox[{"n", "-", "1"}], ",", "\[Ellipsis]", ",", "2", ",", "1"}], "}"}],
    TraditionalForm]]],
 ", which is compared to the input. If they agree, ",
 StyleBox[ButtonBox["Return",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Return"], "FunctionLink"],
 " is used to terminate the function and cause the output to be ",
 StyleBox[ButtonBox["Null",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Null"], "FunctionLink"],
 "."
}], "Text"],

Cell["Here is the implementation. ", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"nextPermutation", "[", "A_List", "]"}], "/;", 
   RowBox[{
    RowBox[{"Sort", "[", "A", "]"}], "\[Equal]", 
    RowBox[{"Range", "[", 
     RowBox[{"Length", "[", "A", "]"}], "]"}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", "=", "A"}], ",", 
      RowBox[{"n", "=", 
       RowBox[{"Length", "[", "A", "]"}]}], ",", "i", ",", "j", ",", "k", ",",
       "r", ",", "s"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"a", "\[Equal]", 
        RowBox[{"Reverse", "[", 
         RowBox[{"Range", "[", "n", "]"}], "]"}]}], ",", 
       RowBox[{"Return", "[", "Null", "]"}]}], "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"step", " ", "1"}], ":", " ", 
       RowBox[{"find", " ", "j"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"j", "=", 
      RowBox[{"n", "-", "1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"a", "[", 
         RowBox[{"[", "j", "]"}], "]"}], ">", 
        RowBox[{"a", "[", 
         RowBox[{"[", 
          RowBox[{"j", "+", "1"}], "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"j", "=", 
        RowBox[{"j", "-", "1"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"step", " ", "2"}], ":", " ", 
       RowBox[{
       "find", " ", "k", " ", "and", " ", "interchange", " ", "aj", " ", 
        "and", " ", "ak"}]}], " ", "*)"}], "\[IndentingNewLine]", 
     RowBox[{"k", "=", "n"}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{
        RowBox[{"a", "[", 
         RowBox[{"[", "j", "]"}], "]"}], ">", 
        RowBox[{"a", "[", 
         RowBox[{"[", "k", "]"}], "]"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{"k", "=", 
        RowBox[{"k", "-", "1"}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"a", "=", 
      RowBox[{"interchange", "[", 
       RowBox[{"a", ",", "j", ",", "k"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"(*", " ", 
      RowBox[{
       RowBox[{"step", " ", "3"}], ":", " ", 
       RowBox[{"sort", " ", "the", " ", "tail"}]}], " ", "*)"}], 
     "\[IndentingNewLine]", 
     RowBox[{"r", "=", "n"}], ";", "\[IndentingNewLine]", 
     RowBox[{"s", "=", 
      RowBox[{"j", "+", "1"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"r", ">", "s"}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"a", "=", 
         RowBox[{"interchange", "[", 
          RowBox[{"a", ",", "r", ",", "s"}], "]"}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"r", "=", 
         RowBox[{"r", "-", "1"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"s", "=", 
         RowBox[{"s", "+", "1"}]}]}]}], "\[IndentingNewLine]", "]"}], ";", 
     "\[IndentingNewLine]", "a"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",\

 InitializationCell->True],

Cell["\<\
Example 2 of Section 6.6 finds that the permutation after 362541 is 364125. \
Let's use that example to confirm that our function is working. \
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"nextPermutation", "[", 
  RowBox[{"{", 
   RowBox[{"3", ",", "6", ",", "2", ",", "5", ",", "4", ",", "1"}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "6", ",", "4", ",", "1", ",", "2", ",", "5"}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "To generate all permutations of a set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 " we use a ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[{
 RowBox[{
  RowBox[{"aperm", "=", 
   RowBox[{"Range", "[", "4", "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{"While", "[", 
  RowBox[{
   RowBox[{"aperm", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
   RowBox[{
    RowBox[{"Print", "[", "aperm", "]"}], ";", "\[IndentingNewLine]", 
    RowBox[{"aperm", "=", 
     RowBox[{"nextPermutation", "[", "aperm", "]"}]}]}]}], 
  "\[IndentingNewLine]", "]"}]}], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "2", ",", "4", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "2", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "3", ",", "4", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "4", ",", "2", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"1", ",", "4", ",", "3", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1", ",", "3", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1", ",", "4", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "1", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "3", ",", "4", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "4", ",", "1", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "4", ",", "3", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "1", ",", "2", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "1", ",", "4", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "2", ",", "1", ",", "4"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "2", ",", "4", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4", ",", "1", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"3", ",", "4", ",", "2", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "1", ",", "2", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "1", ",", "3", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "2", ",", "1", ",", "3"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "2", ",", "3", ",", "1"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "3", ",", "1", ",", "2"}], "}"}]], "Print"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"4", ",", "3", ",", "2", ",", "1"}], "}"}]], "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Finding Permutations of Other Sets", "Subsection",
 CellTags->"Section6c"],

Cell[TextData[{
 "As mentioned in the text, any set with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements can be put in one-to-one correspondence with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 ". Consequently, any permutation of a set with ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " elements can be obtained from a permutation of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 " and the correspondence."
}], "Text"],

Cell[TextData[{
 "In ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", applying a permutation, represented as an arrangement of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 ", to a list is particularly easy. The ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator can be applied to a list of integers and will return the list \
ordered by the argument. For example, consider the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 " and the permutation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}], TraditionalForm]]],
 ". To arrange ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 " by the permutation, you simply apply the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 " with argument ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"3", ",", "1", ",", "2"}], "}"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "As another example, consider the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"2", ",", "10", ",", "13", ",", "19"}], "}"}], 
   TraditionalForm]]],
 " and the permutation ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "3", ",", "1"}], "}"}], TraditionalForm]]],
 ". Then the following expression applies the permutation to the set."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"{", 
   RowBox[{"2", ",", "10", ",", "13", ",", "19"}], "}"}], "[", 
  RowBox[{"[", 
   RowBox[{"{", 
    RowBox[{"4", ",", "2", ",", "3", ",", "1"}], "}"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"19", ",", "10", ",", "13", ",", "2"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that the braces inside the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator are required. Without them, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will interpret it as representing nested levels rather than a \
rearrangement. "
}], "Text"],

Cell[CellGroupData[{

Cell["A Function to Permute a General List", "Subsubsection"],

Cell[TextData[{
 "We can use the ",
 StyleBox["nextPermutation", "Input"],
 " function and the above information about the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator to write a function that outputs all permutations of any list."
}], "Text"],

Cell[TextData[{
 "The input to the function will be the list to be permuted. The bulk of the \
function will be contained in a ",
 StyleBox[ButtonBox["Reap",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Reap"], "FunctionLink"],
 ", with ",
 StyleBox[ButtonBox["Sow",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Sow"], "FunctionLink"],
 " applied to each permutation. Otherwise, the function will mirror the ",
 StyleBox[ButtonBox["While",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/While"], "FunctionLink"],
 " loop above that was used to list the permutations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4"}], "}"}], TraditionalForm]]],
 ". The main difference is that, rather than just printing those \
permutations, they are used as the argument to the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator to permute the given list."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"permuteList", "[", "L_List", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", "perm", "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"perm", "=", 
      RowBox[{"Range", "[", 
       RowBox[{"Length", "[", "L", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{
      RowBox[{"Reap", "[", "\[IndentingNewLine]", 
       RowBox[{"While", "[", 
        RowBox[{
         RowBox[{"perm", "=!=", "Null"}], ",", "\[IndentingNewLine]", 
         RowBox[{
          RowBox[{"Sow", "[", 
           RowBox[{"L", "[", 
            RowBox[{"[", "perm", "]"}], "]"}], "]"}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"perm", "=", 
           RowBox[{"nextPermutation", "[", "perm", "]"}]}]}]}], 
        "\[IndentingNewLine]", "]"}], "\[IndentingNewLine]", "]"}], "[", 
      RowBox[{"[", 
       RowBox[{"2", ",", "1"}], "]"}], "]"}]}]}], "\[IndentingNewLine]", 
   "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "Below, we list all the permutations of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
   TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"permuteList", "[", 
  RowBox[{"{", 
   RowBox[{"\"\<a\>\"", ",", "\"\<b\>\"", ",", "\"\<c\>\""}], "}"}], 
  "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"b\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"a\"\>", ",", "\<\"c\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"a\"\>", ",", "\<\"c\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"b\"\>", ",", "\<\"c\"\>", ",", "\<\"a\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"a\"\>", ",", "\<\"b\"\>"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\<\"c\"\>", ",", "\<\"b\"\>", ",", "\<\"a\"\>"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["\<\
Solutions to Computer Projects and Computations and Explorations\
\>", "Section",
 CellTags->"Solutions"],

Cell[CellGroupData[{

Cell["Computer Projects 10", "Subsection",
 CellTags->"SolutionsCP10"],

Cell[TextData[{
 "Given positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 ", list all the ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combinations, with repetition allowed, of the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 "."
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " In Section 6.5 of this manual, we showed that the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function could be used to generate combinations with repetition by \
repeating the elements in the list given as the first argument to ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 "."
}], "Text"],

Cell[TextData[{
 "To generate the 2-combinations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 ", for example, we apply the ",
 StyleBox[ButtonBox["Subsets",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Subsets"], "FunctionLink"],
 " function to the list consisting of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], TraditionalForm]]],
 ", each repeated twice. Note that the number of repetitions must be the same \
as ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " in order to choose ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " all of the same object."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeleteDuplicates", "[", "\[IndentingNewLine]", 
  RowBox[{"Subsets", "[", 
   RowBox[{
    RowBox[{"Join", "@@", 
     RowBox[{"Table", "[", 
      RowBox[{
       RowBox[{"ConstantArray", "[", 
        RowBox[{"i", ",", "2"}], "]"}], ",", 
       RowBox[{"{", 
        RowBox[{"i", ",", 
         RowBox[{"{", 
          RowBox[{"1", ",", "2", ",", "3"}], "}"}]}], "}"}]}], "]"}]}], ",", 
    RowBox[{"{", "2", "}"}]}], "]"}], "\[IndentingNewLine]", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Refer back to Section 6.5, in the subsection titled \
\[OpenCurlyDoubleQuote]Combinations with Repetition\[CloseCurlyDoubleQuote] \
for information about the use of ",
 StyleBox[ButtonBox["DeleteDuplicates",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/DeleteDuplicates"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Join",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Join"], "FunctionLink"],
 ", ",
 StyleBox[ButtonBox["Table",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Table"], "FunctionLink"],
 ", and ",
 StyleBox[ButtonBox["ConstantArray",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/ConstantArray"], "FunctionLink"],
 " in this application."
}], "Text"],

Cell[TextData[{
 "We now write a function that accepts ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 " as input and produces all ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combinations with repetition allowed."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"subsetsRepetition", "[", 
    RowBox[{"n_Integer", ",", "r_Integer"}], "]"}], "/;", 
   RowBox[{
    RowBox[{"n", ">", "0"}], "&&", 
    RowBox[{"r", ">", "0"}]}]}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"L", ",", "i"}], "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"L", "=", 
      RowBox[{"Range", "[", "n", "]"}]}], ";", "\[IndentingNewLine]", 
     RowBox[{"DeleteDuplicates", "[", "\[IndentingNewLine]", 
      RowBox[{"Subsets", "[", 
       RowBox[{
        RowBox[{"Join", "@@", 
         RowBox[{"Table", "[", 
          RowBox[{
           RowBox[{"ConstantArray", "[", 
            RowBox[{"i", ",", "r"}], "]"}], ",", 
           RowBox[{"{", 
            RowBox[{"i", ",", "L"}], "}"}]}], "]"}]}], ",", 
        RowBox[{"{", "r", "}"}]}], "]"}], "\[IndentingNewLine]", "]"}]}]}], 
   "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "We can obtain all of the 3-combinations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3", ",", "4", ",", "5"}], "}"}], 
   TraditionalForm]]],
 " by"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"subsetsRepetition", "[", 
  RowBox[{"5", ",", "3"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "5", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "5", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "3", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"3", ",", "5", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "4", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "4", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"4", ",", "5", ",", "5"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "5", ",", "5"}], "}"}]}], "}"}]], "Output"]
}, Open  ]]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 1", "Subsection",
 CellTags->"SolutionsCE1"],

Cell["\<\
Find the number of possible outcomes in a two-team playoff when the winner is \
the first team to win 5 out of 9, 6 out of 11, 7 out of 13, and 8 out of 15. \
\
\>", "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will designate the two teams as 1 and 2 and model a playoff as a list \
of 1s and 2s. For example, ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "1", ",", "2", ",", "2", ",", "1", ",", "1", ",", "1", ",", "2", ",", 
     "1"}], "}"}], TraditionalForm]]],
 " is a playoff in which team 1 wins the first game, team 2 wins games 2 and \
3, team 1 wins games 4, 5, and 6, team 2 wins game 7, and them team 1 wins \
game 8. If the winner is the first team to win 5 out of 9 games, then team 1 \
has won the tournament after 8 games."
}], "Text"],

Cell[TextData[{
 "We will write a function to produce all of the possible outcomes in a \
playoff where the winner is the first team to win ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " out of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"2", "n"}], "-", "1"}], TraditionalForm]]],
 " games."
}], "Text"],

Cell["\<\
First we will create a small function that will determine, given a list of \
the outcomes of individual games and the number of games needed to win, \
whether a team has won the playoff or not. The function will return true if \
one of the teams has won or false if neither team has reached the threshold \
for winning.\
\>", "Text"],

Cell[TextData[{
 "The function counts the 1s and 2s in the list. If either number is equal to \
",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", it returns true. We will use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "\[CloseCurlyQuote]s ",
 StyleBox[ButtonBox["Count",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Count"], "FunctionLink"],
 " function to determine the number of 1s and 2s in the list. ",
 StyleBox[ButtonBox["Count",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Count"], "FunctionLink"],
 " requires two arguments, an expression and a pattern. It returns the number \
of times the pattern appears in the expression. In this case, the expression \
will be the list representing the playoff and the pattern will be alternately \
1 and 2."
}], "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"playoffWonQ", "[", 
   RowBox[{"L_List", ",", "n_Integer"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"Count", "[", 
     RowBox[{"L", ",", "1"}], "]"}], "\[Equal]", "n"}], "||", 
   RowBox[{
    RowBox[{"Count", "[", 
     RowBox[{"L", ",", "2"}], "]"}], "\[Equal]", "n"}]}]}]], "Input",
 InitializationCell->True],

Cell[TextData[{
 "For instance, in our example ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
    "1", ",", "2", ",", "2", ",", "1", ",", "1", ",", "1", ",", "2", ",", 
     "1"}], "}"}], TraditionalForm]]],
 ", the function recognizes that the playoff has been won."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"playoffWonQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
    "1", ",", "2", ",", "2", ",", "1", ",", "1", ",", "1", ",", "2", ",", 
     "1"}], "}"}], ",", "5"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "We will construct the possible outcomes as follows. Begin with a list ",
 StyleBox["outcomes", "Input"],
 " and a list ",
 StyleBox["S", "Input"],
 ". Initialize ",
 StyleBox["outcomes", "Input"],
 " to the empty list and ",
 StyleBox["S", "Input"],
 " to the list ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", "1", "}"}], ",", 
     RowBox[{"{", "2", "}"}]}], "}"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "Consider the first element of ",
 StyleBox["S", "Input"],
 ", say ",
 StyleBox["p", "Input"],
 ". Remove ",
 StyleBox["p", "Input"],
 " from the list. Then construct the two lists formed by adding 1 and 2 \
respectively to ",
 StyleBox["p", "Input"],
 ". For each of these, use ",
 StyleBox["playoffWon", "Input"],
 " to determine whether or not they are outcomes. If so, they are added to \
the ",
 StyleBox["outcomes", "Input"],
 " list, and if not, they are added to the end of ",
 StyleBox["S", "Input"],
 ". When ",
 StyleBox["S", "Input"],
 " is empty, then ",
 StyleBox["outcomes", "Input"],
 " consists of all possible outcomes of the playoff."
}], "Text"],

Cell["Here is the function.", "Text"],

Cell[BoxData[
 RowBox[{
  RowBox[{"allPlayoffs", "[", "n_Integer", "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"outcomes", "=", 
       RowBox[{"{", "}"}]}], ",", 
      RowBox[{"S", "=", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", "1", "}"}], ",", 
         RowBox[{"{", "2", "}"}]}], "}"}]}], ",", "p", ",", "p1", ",", "p2"}],
      "}"}], ",", "\[IndentingNewLine]", 
    RowBox[{
     RowBox[{"While", "[", 
      RowBox[{
       RowBox[{"S", "\[NotEqual]", 
        RowBox[{"{", "}"}]}], ",", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"p", "=", 
         RowBox[{"S", "[", 
          RowBox[{"[", "1", "]"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"S", "=", 
         RowBox[{"Delete", "[", 
          RowBox[{"S", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"p1", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"p", ",", "1"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"p2", "=", 
         RowBox[{"Append", "[", 
          RowBox[{"p", ",", "2"}], "]"}]}], ";", "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"playoffWonQ", "[", 
           RowBox[{"p1", ",", "n"}], "]"}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"outcomes", ",", "p1"}], "]"}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"S", ",", "p1"}], "]"}]}], "]"}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"playoffWonQ", "[", 
           RowBox[{"p2", ",", "n"}], "]"}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"outcomes", ",", "p2"}], "]"}], ",", 
          RowBox[{"AppendTo", "[", 
           RowBox[{"S", ",", "p2"}], "]"}]}], "]"}], ";"}]}], 
      "\[IndentingNewLine]", "]"}], ";", "\[IndentingNewLine]", 
     "outcomes"}]}], "\[IndentingNewLine]", "]"}]}]], "Input",
 InitializationCell->True],

Cell["\<\
We now apply this function to playoffs that are best 3 out of 5.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"best3of5", "=", 
  RowBox[{"allPlayoffs", "[", "3", "]"}]}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "1", ",", "2", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "1", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "2", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1", ",", "2", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "1", ",", "2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "2", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "1", ",", "2", ",", "1", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "1", ",", "1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"2", ",", "2", ",", "1", ",", "1", ",", "2"}], "}"}]}], 
  "}"}]], "Output"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Length", "[", "best3of5", "]"}]], "Input"],

Cell[BoxData["20"], "Output"]
}, Open  ]],

Cell["\<\
The reader is left to apply the function to the cases called for in the \
problem and to conjecture a general formula.\
\>", "Text"]
}, Closed]],

Cell[CellGroupData[{

Cell["Computations and Explorations 3", "Subsection",
 CellTags->"SolutionsCE3"],

Cell[TextData[{
 "Verify that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{
     RowBox[{"2", "n"}], ",", "n"}], ")"}], TraditionalForm]]],
 " is divisible by the square of a prime, when ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[NotEqual]", "1"}], ",", "2", ",", " ", 
    RowBox[{"or", " ", "4"}]}], TraditionalForm]]],
 ", for as many positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as you can. [The theorem that tells that ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{
     RowBox[{"2", "n"}], ",", "n"}], ")"}], TraditionalForm]]],
 " is divisible by the square of a prime for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"n", "\[NotEqual]", "1"}], ",", "2", ",", 
    RowBox[{"or", " ", "4"}]}], TraditionalForm]]],
 " was proved in 1996 by Andrew Granville and Olivier Ramar\[EAcute]. Their \
proof settled a conjecture made in 1980 by Paul Erd\[ODoubleDot]s and Ron \
Graham.]"
}], "Problem"],

Cell[TextData[{
 StyleBox["Solution:",
  FontSlant->"Italic"],
 " We will first consider one example to see exactly what we need to do. Then \
we will write a general function. Consider ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "3"}], TraditionalForm]]],
 ", the smallest ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for which the theorem is true."
}], "Text"],

Cell[TextData[{
 "First, compute ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{
     RowBox[{"2", "n"}], ",", "n"}], ")"}], TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "3"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"c3", "=", 
  RowBox[{"Binomial", "[", 
   RowBox[{"6", ",", "3"}], "]"}]}]], "Input",
 InitializationCell->True],

Cell[BoxData["20"], "Output"]
}, Open  ]],

Cell[TextData[{
 "To determine whether or not ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{
     RowBox[{"2", "n"}], ",", "n"}], ")"}], TraditionalForm]]],
 " is divisible by the square of a prime, we could look at its prime \
factorization. If any of the exponents in the prime factorization are 2 or \
greater, then we know the number is divisible by the square of the \
corresponding prime."
}], "Text"],

Cell[TextData[{
 "We can use the function ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " (first discussed in Section 4.3 of this manual). The ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " function requires one argument, an integer. Its output is a list of the \
form ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["p", "1"], ",", 
       SubscriptBox["e", "1"]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["p", "2"], ",", 
       SubscriptBox["e", "2"]}], "}"}], ",", "\[Ellipsis]", ",", 
     RowBox[{"{", 
      RowBox[{
       SubscriptBox["p", "m"], ",", 
       SubscriptBox["e", "m"]}], "}"}]}], "}"}], TraditionalForm]]],
 ", where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["p", "m"]}], TraditionalForm]]],
 " are the primes in the prime factorization and the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["e", "1"], ",", 
    SubscriptBox["e", "2"], ",", "\[Ellipsis]", ",", 
    SubscriptBox["e", "m"]}], TraditionalForm]]],
 " are the corresponding exponents."
}], "Text"],

Cell[TextData[{
 "Apply ",
 StyleBox[ButtonBox["FactorInteger",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/FactorInteger"], "FunctionLink"],
 " to ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"6", ",", "3"}], ")"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FactorInteger", "[", "c3", "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"2", ",", "2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"5", ",", "1"}], "}"}]}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "The result tells us that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"C", "(", 
     RowBox[{"6", ",", "3"}], ")"}], "=", 
    RowBox[{
     SuperscriptBox["2", "2"], "\[CenterDot]", 
     SuperscriptBox["5", "1"]}]}], TraditionalForm]]],
 "."
}], "Text"],

Cell[TextData[{
 "We are interested in the exponents of the primes. To extract the second \
element of each pair, we can apply the ",
 StyleBox[ButtonBox["Part",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/Part"], "FunctionLink"],
 " (",
 StyleBox["[[\[Ellipsis]]]", "Input"],
 ") operator with the part specification ",
 StyleBox["All,2", "Input"],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FactorInteger", "[", "c3", "]"}], "[", 
  RowBox[{"[", 
   RowBox[{"All", ",", "2"}], "]"}], "]"}]], "Input"],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"2", ",", "1"}], "}"}]], "Output"]
}, Open  ]],

Cell[TextData[{
 "Given the list of exponents, determining whether or not ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"6", ",", "3"}], ")"}], TraditionalForm]]],
 " is divisible by the square of a prime just amounts to checking whether the \
list contains a value greater than 1. "
}], "Text"],

Cell[TextData[{
 "As is so often the case, ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " provides a shortcut. The function ",
 StyleBox[ButtonBox["SquareFreeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SquareFreeQ"], "FunctionLink"],
 ", applied to an integer, outputs ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " if the integer is not divisible by the square of a number and it returns \
",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " if the integer is divisible by a square. "
}], "Text"],

Cell[TextData[{
 "For the example ",
 Cell[BoxData[
  FormBox[
   RowBox[{"C", "(", 
    RowBox[{"6", ",", "3"}], ")"}], TraditionalForm]]],
 ", ",
 StyleBox[ButtonBox["SquareFreeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SquareFreeQ"], "FunctionLink"],
 " will output ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " since 20 is divisible by ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["2", "2"], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SquareFreeQ", "[", "c3", "]"}]], "Input"],

Cell[BoxData["False"], "Output"]
}, Open  ]],

Cell[TextData[{
 "Note that ",
 StyleBox[ButtonBox["SquareFreeQ",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/SquareFreeQ"], "FunctionLink"],
 " returns ",
 StyleBox[ButtonBox["False",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/False"], "FunctionLink"],
 " when the assertion is verified and ",
 StyleBox[ButtonBox["True",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/True"], "FunctionLink"],
 " when the assertion fails, such as for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"n", "=", "4"}], TraditionalForm]]],
 "."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SquareFreeQ", "[", 
  RowBox[{"Binomial", "[", 
   RowBox[{"8", ",", "4"}], "]"}], "]"}]], "Input"],

Cell[BoxData["True"], "Output"]
}, Open  ]],

Cell[TextData[{
 "The problem was to verify the conjecture for \[OpenCurlyDoubleQuote]as many \
positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " as you can.\[CloseCurlyDoubleQuote] We will use the ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " function to run the test for a specified amount of time. The second \
argument of ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " is an amount of time, in seconds, that bounds the time that ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " will spend executing the first argument. For the first argument, we will \
use an infinite loop that will display values of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for which the statement is found to be false. ",
 StyleBox[ButtonBox["TimeConstrained",
  BaseStyle->"Link",
  ButtonData->"paclet:ref/TimeConstrained"], "FunctionLink"],
 " takes an optional third argument that is evaluated if the allotted time \
expires. We will use this argument to display the maximum value of ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " for which the statement was checked. Note that this maximum ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " must be one less than the index used in the loop."
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeConstrained", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{
    RowBox[{"n", "=", "1"}], ";", "\[IndentingNewLine]", 
    RowBox[{"While", "[", 
     RowBox[{"True", ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"If", "[", 
        RowBox[{
         RowBox[{"SquareFreeQ", "[", 
          RowBox[{"Binomial", "[", 
           RowBox[{
            RowBox[{"2", "*", "n"}], ",", "n"}], "]"}], "]"}], ",", 
         RowBox[{"Print", "[", 
          RowBox[{"\"\<Found counterexample: \>\"", ",", "n"}], "]"}]}], 
        "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"n", "++"}]}]}], "\[IndentingNewLine]", "]"}]}], ",", 
   "\[IndentingNewLine]", "1", " ", ",", 
   RowBox[{"(*", " ", 
    RowBox[{"1", " ", "second", " ", "limit"}], " ", "*)"}], 
   "\[IndentingNewLine]", 
   RowBox[{"Print", "[", 
    RowBox[{"\"\<Checked through n=\>\"", ",", 
     RowBox[{"n", "-", "1"}]}], "]"}]}], "\[IndentingNewLine]", 
  "]"}]], "Input"],

Cell[CellGroupData[{

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Found counterexample: \"\>", "\[InvisibleSpace]", "1"}],
  SequenceForm["Found counterexample: ", 1],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Found counterexample: \"\>", "\[InvisibleSpace]", "2"}],
  SequenceForm["Found counterexample: ", 2],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Found counterexample: \"\>", "\[InvisibleSpace]", "4"}],
  SequenceForm["Found counterexample: ", 4],
  Editable->False]], "Print"],

Cell[BoxData[
 InterpretationBox[
  RowBox[{"\<\"Checked through n=\"\>", "\[InvisibleSpace]", "8825"}],
  SequenceForm["Checked through n=", 8825],
  Editable->False]], "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],

Cell[CellGroupData[{

Cell["Exercises", "Section",
 CellTags->"Exercises"],

Cell[CellGroupData[{

Cell[TextData[{
 "Build a recursive version of ",
 StyleBox["subsetSumCount", "Input"],
 ", using the ideas of ",
 StyleBox["findBitStrings", "Input"],
 ". Your function should determine all subsets of a given set whose sum is \
less than a target value. Rather than considering all sets, it should build \
potential sets recursively using the fact that once a set has sum larger than \
the target no larger set of positive integers can have smaller sum. Compare \
the performance of your procedure with ",
 StyleBox["subsetSumCount", "Input"],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "Create a function ",
 StyleBox["findDecreasing", "Input"],
 " by modifying ",
 StyleBox["findIncreasing", "Input"],
 " in order to determine a strictly decreasing subsequence of maximal \
length."
}], "Item1Exercise"],

Cell[TextData[{
 "Modify the Patience algorithm to find ",
 StyleBox["all",
  FontSlant->"Italic"],
 " of the strictly increasing subsequences of maximal length. "
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to show that, in Example 11 of Section 6.2, ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " positive integers not exceeding ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "n"}], TraditionalForm]]],
 " are not sufficient to guarantee that one integer divides one of the others."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to determine how many different strings can be made from the word \
\[OpenCurlyDoubleQuote]PAPARAZZI\[CloseCurlyDoubleQuote] when all the letters \
are used, when any number of letters are used, when all the letters are used \
and the string begins and ends with the letter \[OpenCurlyDoubleQuote]Z\
\[CloseCurlyDoubleQuote], and when all the letters are used and the three \
\[OpenCurlyDoubleQuote]A\[CloseCurlyDoubleQuote]s are consecutive."
}], "Item1Exercise"],

Cell[TextData[{
 "Suppose that a certain Mathematics Department has ",
 Cell[BoxData[
  FormBox["m", TraditionalForm]]],
 " male faculty and ",
 Cell[BoxData[
  FormBox["f", TraditionalForm]]],
 " female faculty. Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to find all committees with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"2", "k"}], TraditionalForm]]],
 " members in which both sexes are represented equally."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to prove the identity"
}], "Item1Exercise"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {
       RowBox[{"n", "+", "1"}]},
      {"k"}
     }], ")"}], "=", 
   RowBox[{
    FractionBox[
     RowBox[{"n", "+", "1"}], "k"], 
    RowBox[{"(", GridBox[{
       {"n"},
       {
        RowBox[{"k", "-", "1"}]}
      }], ")"}]}]}], TraditionalForm]], "ExerciseEquation"],

Cell[TextData[{
 "for positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[LessEqual]", "n"}], TraditionalForm]]],
 "."
}], "ExerciseContinuation"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to prove Pascal's identity:"
}], "Item1Exercise"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", GridBox[{
      {
       RowBox[{"n", "+", "1"}]},
      {"k"}
     }], ")"}], "=", 
   RowBox[{
    RowBox[{"(", GridBox[{
       {"n"},
       {
        RowBox[{"k", "-", "1"}]}
      }], ")"}], "+", 
    RowBox[{"(", GridBox[{
       {"n"},
       {"k"}
      }], ")"}]}]}], TraditionalForm]], "ExerciseEquation"],

Cell[TextData[{
 "for all positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 " with ",
 Cell[BoxData[
  FormBox[
   RowBox[{"k", "\[LessEqual]", "n"}], TraditionalForm]]],
 "."
}], "ExerciseContinuation"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to generate many rows of Pascal's triangle. See if you can formulate any \
conjectures involving identities satisfied by the binomial coefficients. Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to help you verify that your conjecture is true by using the techniques at \
the end of Section 6.4 of this manual."
}], "Item1Exercise"],

Cell[TextData[{
 "Write a function that mixes the techniques used in ",
 StyleBox["binomialF", "Input"],
 " and ",
 StyleBox["binomialR", "Input"],
 " to generate the rows of Pascal's triangle from row ",
 Cell[BoxData[
  FormBox["a", TraditionalForm]]],
 " to row ",
 Cell[BoxData[
  FormBox["b", TraditionalForm]]],
 " for ",
 Cell[BoxData[
  FormBox[
   RowBox[{"b", ">", "a", ">", "0"}], TraditionalForm]]],
 "."
}], "Item1Exercise"],

Cell[TextData[{
 "Use ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " to count and list all solutions to the equation"
}], "Item1Exercise"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["x", "1"], "+", 
    SubscriptBox["x", "2"], "+", 
    SubscriptBox["x", "3"], "+", 
    SubscriptBox["x", "4"]}], "=", "25"}], 
  TraditionalForm]], "ExerciseEquation"],

Cell[TextData[{
 "where ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["x", "1"], ",", 
    SubscriptBox["x", "2"], ",", 
    SubscriptBox["x", "3"], ",", " ", 
    RowBox[{"and", " ", 
     SubscriptBox["x", "4"]}]}], TraditionalForm]]],
 " are non-negative integers. Also count and list all solutions such that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["x", "1"], "\[GreaterEqual]", "1"}], ",", 
    RowBox[{
     SubscriptBox["x", "2"], "\[GreaterEqual]", "2"}], ",", 
    RowBox[{
     SubscriptBox["x", "3"], "\[GreaterEqual]", "3"}], ",", 
    RowBox[{
     RowBox[{"and", " ", 
      SubscriptBox["x", "4"]}], "\[GreaterEqual]", "4"}]}], TraditionalForm]]],
 "."
}], "ExerciseContinuation"],

Cell["\<\
Generate a large triangle of Stirling numbers of the second kind and look for \
patterns that suggest identities among the Stirling numbers. Also see if you \
can make any conjectures about the relationship between Stirling numbers and \
the binomial coefficients. \
\>", "Item1Exercise"],

Cell["\<\
Implement the algorithm described in the \[OpenCurlyDoubleQuote]Generating \
partitions\[CloseCurlyDoubleQuote] subsection of Section 6.5 of this manual.\
\>", "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function that takes as input three positive integers ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox["k", TraditionalForm]]],
 ", and ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 ", and returns the ",
 Cell[BoxData[
  FormBox["i", TraditionalForm]]],
 "th multinomial, in lexicographic order, of the polynomial ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["x", "1"], "+", 
      SubscriptBox["x", "2"], "+", "\[CenterEllipsis]", "+", 
      SubscriptBox["x", "k"]}], ")"}], "n"], TraditionalForm]]],
 ". Write its inverse; that is, given a multinomial, the inverse should \
return its index (position) in the sorted polynomial."
}], "Item1Exercise"],

Cell["\<\
Implement Algorithm 2 of Section 6.6 for generating the next largest bit \
string. \
\>", "Item1Exercise"],

Cell[TextData[{
 "Implement Algorithm 3 of Section 6.6 for generating the next ",
 Cell[BoxData[
  FormBox["r", TraditionalForm]]],
 "-combination. "
}], "Item1Exercise"],

Cell[TextData[{
 "Write a ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " function to compute the Cantor expansion of an integer. (See the prelude \
to Exercise 14 of Section 6.6 of the text.)"
}], "Item1Exercise"],

Cell[TextData[{
 "Implement the algorithm for generating the set of all permutations of the \
first ",
 Cell[BoxData[
  FormBox["n", TraditionalForm]]],
 " integers using the bijection from the collection of all permutations of \
the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", "n"}], "}"}], 
   TraditionalForm]]],
 " to the set ",
 Cell[BoxData[
  FormBox[
   RowBox[{"{", 
    RowBox[{"1", ",", "2", ",", "\[Ellipsis]", ",", 
     RowBox[{"n", "!"}]}], "}"}], TraditionalForm]]],
 " described prior to Exercise 14 of Section 6.6 of the textbook. \:fffc"
}], "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
},
WindowSize->{725, 850},
WindowMargins->{{4, Automatic}, {Automatic, 4}},
DockedCells->Cell[
  BoxData[
   TagBox[
    GridBox[{{"\"Exploring Discrete Mathematics with Mathematica\"", 
       ActionMenuBox[
       "\"Navigation\"", {
        "\"CloseAll\"" :> Module[{$CellContext`DJSection$}, SelectionMove[
             SelectedNotebook[], All, Notebook]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken["OpenCloseGroup"]], {$CellContext`DJSection$, {
              "Section1a", "Section1b", "Section1c", "Section1d", "Section2a",
                "Section2b", "Section3a", "Section3b", "Section3c", 
               "Section4a", "Section4b", "Section4c", "Section5a", 
               "Section5b", "Section5c", "Section5d", "Section5e", 
               "Section5f", "Section5g", "Section6a", "Section6b", 
               "Section6c", "SolutionsCE1", "SolutionsCE3", "SolutionsCP10", 
               "Section1", "Section2", "Section3", "Section4", "Section5", 
               "Section6", "Solutions", "Exercises"}}]; SelectionMove[
             SelectedNotebook[], Before, Notebook]], "\"Section1\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section1"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section2\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section2"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section3\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section3"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section4\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section4"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section5\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section5"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Section6\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Section6"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Solutions\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Solutions"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Exercises\"" :> 
         Module[{}, Do[NotebookLocate[$CellContext`DJSection]; FrontEndExecute[
                FrontEndToken[
                "SelectionCloseAllGroups"]], {$CellContext`DJSection, {
               "Section1", "Section2", "Section3", "Section4", "Section5", 
                "Section6", "Solutions", "Exercises"}}] 
            NotebookLocate["Exercises"]; FrontEndExecute[
             FrontEndToken["SelectionCloseAllGroups"]]; FrontEndExecute[
             FrontEndToken["SelectionOpenAllGroups"]]; SelectionMove[
             SelectedNotebook[], Before, Cell]], "\"Open All\"" :> 
         Module[{$CellContext`DJSection$}, 
           Do[NotebookLocate[$CellContext`DJSection$]; FrontEndExecute[
               FrontEndToken[
               "SelectionOpenAllGroups"]], {$CellContext`DJSection$, {
              "Exercises", "Section1", "Section1a", "Section1b", "Section1c", 
               "Section1d", "Section2", "Section2a", "Section2b", "Section3", 
               "Section3a", "Section3b", "Section3c", "Section4", "Section4a",
                "Section4b", "Section4c", "Section5", "Section5a", 
               "Section5b", "Section5c", "Section5d", "Section5e", 
               "Section5f", "Section5g", "Section6", "Section6a", "Section6b",
                "Section6c", "Solutions", "SolutionsCE1", "SolutionsCE3", 
               "SolutionsCP10"}}] SelectionMove[
             SelectedNotebook[], Before, Notebook]]}]}}, 
     GridBoxAlignment -> {
      "Columns" -> {Left, Baseline}, "Rows" -> {Right, Baseline}}, AutoDelete -> 
     False, GridBoxItemSize -> {"Columns" -> {
         Scaled[0.8], 
         Scaled[0.2]}}], "Grid"]], "DockedCell", FontFamily -> "Times"],
PrintingCopies->1,
PrintingPageRange->{1, Automatic},
Magnification->1,
FrontEndVersion->"9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, \
2013)",
StyleDefinitions->Notebook[{
   Cell[
    StyleData[
    StyleDefinitions -> 
     FrontEnd`FileName[{"Book"}, "Textbook.nb", CharacterEncoding -> 
       "UTF-8"]]], 
   Cell[
    StyleData[All, "Printout"], PageWidth -> PaperWidth, 
    CellHorizontalScrolling -> True, 
    CellLabelMargins -> {{2, Inherited}, {Inherited, Inherited}}, 
    ShowAutoStyles -> False, 
    AutoStyleOptions -> {"HighlightMisspelledWords" -> False}, ScriptMinSize -> 
    9, MenuSortingValue -> 1500, 
    PrivateFontOptions -> {"FontType" -> "Outline"}, Magnification -> 1], 
   Cell[
    StyleData["Title"], Editable -> False, ShowCellBracket -> Automatic, 
    CellGroupingRules -> {"TitleGrouping", 0}, PageBreakBelow -> False, 
    DefaultNewInlineCellStyle -> {}, LanguageCategory -> "NaturalLanguage", 
    FontFamily -> "Helvetica", FontSize -> 25, FontWeight -> "Bold", 
    FontColor -> GrayLevel[0.137255]], 
   Cell[
    StyleData["BookChapterTitle", StyleDefinitions -> StyleData["Title"]], 
    CellFrame -> {{0, 0}, {1, 1}}, CellDingbat -> Cell[
      BoxData[
       GraphicsBox[{{
          RGBColor[0.09375, 0.570313, 0.816406], 
          RectangleBox[{0, 0}]}, {
          GrayLevel[1], FontSize -> 35, FontWeight -> Bold, FontFamily -> 
          "Helvetica", 
          InsetBox[
           CounterBox["ChapterNumber"], {0.5, 0.45}, {Center, Center}]}}, 
        ImageSize -> {70, 70}]], "Output"], CellMargins -> {{75, 0}, {20, 5}},
     CellFrameColor -> GrayLevel[1]], 
   Cell[
    StyleData["DockedCell"], CellMargins -> {{0, 12}, {0, 0}}, 
    CellGroupingRules -> "TitleGrouping", 
    ConversionRules :> {"TeX" -> {"\n\\chapter{", "}"}}, 
    TaggingRules :> {"CounterSettings" -> {"CounterTarget" -> "CellGroup"}}, 
    PageBreakAbove -> False, PageBreakBelow -> False, LineSpacing -> {1, 2}, 
    MenuSortingValue -> 1105, FontFamily -> "Helvetica", FontSize -> 20, 
    FontWeight -> "Bold", Background -> 
    RGBColor[0.09375, 0.570313, 0.816406]], 
   Cell[
    StyleData["FunctionLink"], ConversionRules :> {"TeX" -> {{"", 
         $CellContext`ExportAsHyperRef[], ""}, {"", Automatic, ""}}}, 
    MenuSortingValue -> 1105, ButtonStyleMenuListing -> None, FontFamily -> 
    "Courier", FontVariations -> {"Underline" -> True}, FontColor -> 
    RGBColor[0, 0, 1], 
    ButtonBoxOptions -> {Active -> True, ButtonFunction :> (FrontEndExecute[{
         FrontEnd`NotebookLocate[#2]}]& ), ButtonNote -> ButtonData}], 
   Cell[
    StyleData["Problem"], CellMargins -> {{40, 12}, {15, 15}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 15010}, 
    ConversionRules :> {
     "TeX" -> {{"", Automatic, ""}, {"\n\n", Automatic, ""}}}, 
    TaggingRules :> {
     "ExtraConversionRules" -> {"General" -> {"RestartList" -> False}}}], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item1Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {30, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item1Exercise", 
       CounterAssignments -> {{"Item2Exercise", 0}}, MenuSortingValue -> 
       4100], 
      Cell[
       StyleData["Item1Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item1Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{35, 12}, {3, 5}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["ExerciseContinuation"], CellMargins -> {{50, 12}, {1, 3}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17000}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, MenuSortingValue -> 4101], 
      Cell[
       StyleData["ExerciseContinuation", "Printout"], 
       CellMargins -> {{35, 12}, {1, 3}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Item2Exercise"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {32, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{75, 12}, {1, 2}}, 
       CellGroupingRules -> {"GroupTogetherNestedGrouping", 17100}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterIncrements -> "Item2Exercise", MenuSortingValue -> 4200, 
       CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}], 
      Cell[
       StyleData["Item2Exercise", "Printout"], CellDingbat -> Cell[
         TextData[{
           CounterBox["Item2Exercise"], "."}], CellSize -> {20, Inherited}, 
         TextAlignment -> Right, FontWeight -> "Bold"], 
       CellMargins -> {{50, 12}, {1, 2}}, 
       CellFrameLabels -> {{None, None}, {None, None}}, CellFrameLabelMargins -> 
       6, CounterBoxOptions -> {CounterFunction :> (Part[
           CharacterRange["a", "z"], #]& )}]}, Open]], 
   Cell[
    StyleData["ExerciseEquation"], CellMargins -> {{36, 12}, {7, 7}}, 
    CellGroupingRules -> {"GroupTogetherNestedGrouping", 17200}, 
    CellHorizontalScrolling -> True, 
    CellFrameLabels -> {{None, None}, {None, None}}, DefaultFormatType -> 
    TraditionalForm, InputAutoReplacements -> {}, TextAlignment -> Center, 
    MenuSortingValue -> 4300, 
    ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["myCode"], CellMargins -> {{66, 10}, {5, 10}}, 
    StyleKeyMapping -> {"=" -> "WolframAlphaShort"}, Evaluatable -> False, 
    CellGroupingRules -> "InputGrouping", TextClipboardType -> "InputText", 
    PageBreakWithin -> False, GroupPageBreakWithin -> False, 
    DefaultFormatType -> DefaultInputFormatType, ContextMenu -> 
    FEPrivate`FrontEndResource["ContextMenus", "myCode"], ShowAutoStyles -> 
    True, "TwoByteSyntaxCharacterAutoReplacement" -> True, 
    HyphenationOptions -> {"HyphenationCharacter" -> "\[Continuation]"}, 
    AutoItalicWords -> {}, LanguageCategory -> "Mathematica", FormatType -> 
    InputForm, NumberMarks -> True, 
    LinebreakAdjustments -> {0.85, 2, 10, 0, 1}, CounterIncrements -> "Code", 
    MenuSortingValue -> 1495, MenuCommandKey -> "8", FontWeight -> 
    "Bold", $CellContext`StripStyleOnPaste -> 
    True, $CellContext`AutoQuoteCharacters -> {}, \
$CellContext`PasteAutoQuoteCharacters -> {}], 
   Cell[
    StyleData["CodeVar"], DefaultFormatType -> DefaultInputFormatType, 
    LineBreakWithin -> False, MenuSortingValue -> 10000, FontWeight -> "Bold",
     FontSlant -> "Italic", FontColor -> GrayLevel[0.4]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Section"], CellFrame -> {{6, 0}, {0, 1}}, ShowCellBracket -> 
       Automatic, CellMargins -> {{12, 12}, {3, 24}}, 
       CellGroupingRules -> {"SectionGrouping", 30}, 
       ConversionRules :> {
        "TeX" -> {"\n\\section{", "}"}, 
         "HTML" -> {"<h3 class=\"Section\">", "</h3>"}}, 
       TaggingRules :> {
        "CounterSettings" -> {
          "CounterFormat" -> {"BookChapterNumber", ".", "Section"}, 
           "CounterTarget" -> "Cell", "XRefPrefix" -> "Section"}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{8, 8}, {4, 6}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], TabSpacings -> 
       2.1, CounterIncrements -> {
        "Section", "SectionHeader", "SubsectionHeader", "SubsubsectionHeader",
          "SubsubsubsectionHeader"}, FontFamily -> "Helvetica", FontSize -> 
       16, FontWeight -> "Bold", Background -> GrayLevel[0.999985]], 
      Cell[
       StyleData["Section", "Printout"], CellFrame -> {{4, 0}, {0, 0.5}}, 
       CellMargins -> {{2, 2}, {3, 21}}, PageBreakBelow -> False, 
       CellFrameMargins -> {{6, 6}, {3, 5}}, CellFrameColor -> 
       RGBColor[0.09375, 0.570313, 0.816406]], 
      Cell[
       StyleData["Section", "DoubleSpacedPrintout"], 
       CellFrame -> {{4, 0}, {0, 0.5}}, CellMargins -> {{2, 2}, {3, 21}}, 
       PageBreakBelow -> False, CellFrameMargins -> {{6, 6}, {3, 5}}, 
       CellFrameColor -> RGBColor[0.09375, 0.570313, 0.816406], FontSize -> 
       14]}, Open]], 
   Cell[
    CellGroupData[{
      Cell[
       StyleData["Subsubsection"], CellDingbat -> None, 
       CellMargins -> {{12, 12}, {0, 6}}, 
       CellElementSpacings -> {"ClosedGroupTopMargin" -> 12}, 
       CellGroupingRules -> {"SectionGrouping", 50}, 
       ConversionRules :> {
        "TeX" -> {"\n\\subsubsection{", "}"}, 
         "HTML" -> {"<h5 class=\"Subsubsection\">", "</h5>"}}, PageBreakBelow -> 
       False, CellFrameLabelMargins -> 4, 
       CounterIncrements -> {
        "Subsubsection", "SubsubsectionHeader", "SubsubsubsectionHeader"}, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"], 
      Cell[
       StyleData["Subsubsection", "Printout"], PageBreakBelow -> False, 
       FontFamily -> "Helvetica", FontSize -> 11, FontWeight -> "Bold", 
       FontSlant -> "Italic"]}, Open]], 
   Cell[
    StyleData["Equation"], CellMargins -> {{26, 10}, {7, 7}}, 
    ConversionRules :> {
     "HTML" -> {{"", ""}, {"<p class=\"Equation\">", "</p>"}}}, 
    CellHorizontalScrolling -> True, DefaultFormatType -> TraditionalForm, 
    InputAutoReplacements -> {}, TextAlignment -> Center, CounterIncrements -> 
    "Equation", ButtonBoxOptions -> {Appearance -> {Automatic, None}}], 
   Cell[
    StyleData["Subsubsubsection"], CellDingbat -> None, 
    CellMargins -> {{12, 12}, {0, 6}}, 
    CellGroupingRules -> {"SectionGrouping", 60}, 
    ConversionRules :> {
     "TeX" -> {"\n\\subsubsubsection{", "}"}, 
      "HTML" -> {"<h6 class=\"Subsubsubsection\">", "</h6>"}}, PageBreakBelow -> 
    False, CellFrameLabelMargins -> 4, 
    CounterIncrements -> {"Subsubsubsection", "SubsubsubsectionHeader"}, 
    FontFamily -> "Helvetica", FontSize -> 11, FontSlant -> "Italic"]}, 
  Visible -> False, FrontEndVersion -> 
  "9.0 for Mac OS X x86 (32-bit, 64-bit Kernel) (January 25, 2013)", 
  StyleDefinitions -> "PrivateStylesheetFormatting.nb"]
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Section1"->{
  Cell[1156, 42, 68, 1, 55, "Section",
   CellTags->"Section1"]},
 "Section1a"->{
  Cell[1885, 64, 60, 1, 32, "Subsection",
   CellTags->"Section1a"]},
 "Section1b"->{
  Cell[8524, 320, 73, 1, 26, "Subsection",
   CellTags->"Section1b"]},
 "Section1c"->{
  Cell[18632, 645, 79, 1, 26, "Subsection",
   CellTags->"Section1c"]},
 "Section1d"->{
  Cell[22886, 760, 83, 1, 26, "Subsection",
   CellTags->"Section1d"]},
 "Section2"->{
  Cell[40748, 1348, 70, 1, 35, "Section",
   CellTags->"Section2"]},
 "Section2a"->{
  Cell[42013, 1391, 82, 1, 32, "Subsection",
   CellTags->"Section2a"]},
 "Section2b"->{
  Cell[51856, 1702, 78, 1, 26, "Subsection",
   CellTags->"Section2b"]},
 "Section3"->{
  Cell[80316, 2533, 75, 1, 35, "Section",
   CellTags->"Section3"]},
 "Section3a"->{
  Cell[80589, 2545, 58, 1, 32, "Subsection",
   CellTags->"Section3a"]},
 "Section3b"->{
  Cell[97003, 3095, 58, 1, 26, "Subsection",
   CellTags->"Section3b"]},
 "Section3c"->{
  Cell[117600, 3793, 67, 1, 26, "Subsection",
   CellTags->"Section3c"]},
 "Section4"->{
  Cell[132485, 4268, 82, 1, 35, "Section",
   CellTags->"Section4"]},
 "Section4a"->{
  Cell[132801, 4281, 66, 1, 32, "Subsection",
   CellTags->"Section4a"]},
 "Section4b"->{
  Cell[138736, 4519, 80, 1, 26, "Subsection",
   CellTags->"Section4b"]},
 "Section4c"->{
  Cell[159049, 5219, 66, 1, 26, "Subsection",
   CellTags->"Section4c"]},
 "Section5"->{
  Cell[172457, 5740, 87, 1, 35, "Section",
   CellTags->"Section5"]},
 "Section5a"->{
  Cell[172895, 5754, 74, 1, 32, "Subsection",
   CellTags->"Section5a"]},
 "Section5b"->{
  Cell[193581, 6384, 74, 1, 26, "Subsection",
   CellTags->"Section5b"]},
 "Section5c"->{
  Cell[202778, 6687, 89, 1, 26, "Subsection",
   CellTags->"Section5c"]},
 "Section5d"->{
  Cell[210932, 6920, 95, 1, 26, "Subsection",
   CellTags->"Section5d"]},
 "Section5e"->{
  Cell[222215, 7302, 97, 1, 26, "Subsection",
   CellTags->"Section5e"]},
 "Section5f"->{
  Cell[231403, 7647, 97, 1, 26, "Subsection",
   CellTags->"Section5f"]},
 "Section5g"->{
  Cell[254503, 8434, 99, 1, 26, "Subsection",
   CellTags->"Section5g"]},
 "Section6"->{
  Cell[276863, 9239, 86, 1, 35, "Section",
   CellTags->"Section6"]},
 "Section6a"->{
  Cell[277215, 9250, 86, 1, 32, "Subsection",
   CellTags->"Section6a"]},
 "Section6b"->{
  Cell[279647, 9328, 90, 1, 26, "Subsection",
   CellTags->"Section6b"]},
 "Section6c"->{
  Cell[297610, 9974, 80, 1, 26, "Subsection",
   CellTags->"Section6c"]},
 "Solutions"->{
  Cell[304911, 10226, 115, 3, 35, "Section",
   CellTags->"Solutions"]},
 "SolutionsCP10"->{
  Cell[305051, 10233, 70, 1, 32, "Subsection",
   CellTags->"SolutionsCP10"]},
 "SolutionsCE1"->{
  Cell[312528, 10487, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE1"]},
 "SolutionsCE3"->{
  Cell[321257, 10764, 80, 1, 26, "Subsection",
   CellTags->"SolutionsCE3"]},
 "Exercises"->{
  Cell[331983, 11147, 52, 1, 35, "Section",
   CellTags->"Exercises"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Section1", 358261, 11794},
 {"Section1a", 358343, 11797},
 {"Section1b", 358429, 11800},
 {"Section1c", 358516, 11803},
 {"Section1d", 358604, 11806},
 {"Section2", 358691, 11809},
 {"Section2a", 358776, 11812},
 {"Section2b", 358865, 11815},
 {"Section3", 358953, 11818},
 {"Section3a", 359038, 11821},
 {"Section3b", 359127, 11824},
 {"Section3c", 359216, 11827},
 {"Section4", 359305, 11830},
 {"Section4a", 359391, 11833},
 {"Section4b", 359481, 11836},
 {"Section4c", 359571, 11839},
 {"Section5", 359660, 11842},
 {"Section5a", 359746, 11845},
 {"Section5b", 359836, 11848},
 {"Section5c", 359926, 11851},
 {"Section5d", 360016, 11854},
 {"Section5e", 360106, 11857},
 {"Section5f", 360196, 11860},
 {"Section5g", 360286, 11863},
 {"Section6", 360375, 11866},
 {"Section6a", 360461, 11869},
 {"Section6b", 360551, 11872},
 {"Section6c", 360641, 11875},
 {"Solutions", 360731, 11878},
 {"SolutionsCP10", 360824, 11881},
 {"SolutionsCE1", 360922, 11884},
 {"SolutionsCE3", 361019, 11887},
 {"Exercises", 361113, 11890}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 81, 1, 96, "BookChapterTitle",
 CounterAssignments->{{"ChapterNumber", 6}}],
Cell[CellGroupData[{
Cell[685, 27, 31, 0, 55, "Section"],
Cell[719, 29, 400, 8, 50, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1156, 42, 68, 1, 55, "Section",
 CellTags->"Section1"],
Cell[1227, 45, 633, 15, 65, "Text"],
Cell[CellGroupData[{
Cell[1885, 64, 60, 1, 32, "Subsection",
 CellTags->"Section1a"],
Cell[1948, 67, 167, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[2140, 76, 54, 0, 18, "Subsubsection"],
Cell[2197, 78, 307, 9, 35, "Text"],
Cell[2507, 89, 264, 8, 26, "Equation"],
Cell[2774, 99, 71, 0, 20, "Text"],
Cell[2848, 101, 258, 9, 54, "Equation"],
Cell[3109, 112, 330, 9, 35, "Text"],
Cell[CellGroupData[{
Cell[3464, 125, 73, 1, 28, "Input"],
Cell[3540, 128, 33, 0, 28, "Output"]
}, Open  ]],
Cell[3588, 131, 1234, 35, 83, "Text"],
Cell[4825, 168, 151, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[5001, 175, 167, 5, 28, "Input"],
Cell[5171, 182, 33, 0, 28, "Output"]
}, Open  ]],
Cell[5219, 185, 319, 7, 36, "Text"],
Cell[5541, 194, 134, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[5700, 201, 169, 5, 28, "Input"],
Cell[5872, 208, 57, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[5978, 214, 43, 0, 18, "Subsubsection"],
Cell[6024, 216, 271, 5, 35, "Text"],
Cell[6298, 223, 152, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[6475, 230, 123, 4, 28, "Input"],
Cell[6601, 236, 37, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6675, 241, 123, 4, 28, "Input"],
Cell[6801, 247, 38, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[6876, 252, 123, 4, 28, "Input"],
Cell[7002, 258, 40, 0, 28, "Output"]
}, Open  ]],
Cell[7057, 261, 63, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[7145, 265, 85, 2, 28, "Input"],
Cell[7233, 269, 40, 0, 28, "Output"]
}, Open  ]],
Cell[7288, 272, 199, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[7512, 282, 214, 7, 28, "Input"],
Cell[7729, 291, 40, 0, 28, "Output"]
}, Open  ]],
Cell[7784, 294, 384, 6, 50, "Text"],
Cell[CellGroupData[{
Cell[8193, 304, 215, 7, 28, "Input"],
Cell[8411, 313, 52, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[8524, 320, 73, 1, 26, "Subsection",
 CellTags->"Section1b"],
Cell[8600, 323, 231, 5, 35, "Text"],
Cell[CellGroupData[{
Cell[8856, 332, 28, 0, 18, "Subsubsection"],
Cell[8887, 334, 310, 8, 42, "Text"],
Cell[9200, 344, 190, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[9415, 354, 108, 3, 29, "Input"],
Cell[9526, 359, 4463, 126, 183, "Output"]
}, Open  ]],
Cell[14004, 488, 462, 10, 50, "Text"],
Cell[14469, 500, 273, 6, 35, "Text"],
Cell[CellGroupData[{
Cell[14767, 510, 93, 2, 29, "Input"],
Cell[14863, 514, 30, 0, 28, "Output"]
}, Open  ]],
Cell[14908, 517, 329, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[15262, 528, 77, 1, 29, "Input"],
Cell[15342, 531, 35, 0, 28, "Output"]
}, Open  ]],
Cell[15392, 534, 321, 9, 42, "Text"],
Cell[15716, 545, 236, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[15977, 553, 180, 5, 29, "Input"],
Cell[16160, 560, 45, 0, 28, "Output"]
}, Open  ]],
Cell[16220, 563, 49, 0, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[16306, 568, 52, 0, 18, "Subsubsection"],
Cell[16361, 570, 176, 3, 35, "Text"],
Cell[16540, 575, 540, 12, 65, "Text"],
Cell[17083, 589, 245, 6, 35, "Text"],
Cell[17331, 597, 504, 16, 40, "Text"],
Cell[CellGroupData[{
Cell[17860, 617, 190, 6, 28, "Input"],
Cell[18053, 625, 54, 0, 28, "Output"]
}, Open  ]],
Cell[18122, 628, 461, 11, 50, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[18632, 645, 79, 1, 26, "Subsection",
 CellTags->"Section1c"],
Cell[18714, 648, 431, 9, 51, "Text"],
Cell[19148, 659, 351, 7, 51, "Text"],
Cell[19502, 668, 1381, 36, 154, "Input",
 InitializationCell->True],
Cell[20886, 706, 829, 17, 81, "Text"],
Cell[21718, 725, 1131, 30, 93, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[22886, 760, 83, 1, 26, "Subsection",
 CellTags->"Section1d"],
Cell[22972, 763, 569, 10, 65, "Text"],
Cell[CellGroupData[{
Cell[23566, 777, 32, 0, 18, "Subsubsection"],
Cell[23601, 779, 217, 4, 35, "Text"],
Cell[23821, 785, 136, 3, 20, "Text"],
Cell[23960, 790, 407, 12, 36, "Text"],
Cell[24370, 804, 505, 16, 37, "Text"],
Cell[CellGroupData[{
Cell[24900, 824, 107, 3, 28, "Input"],
Cell[25010, 829, 209, 7, 28, "Output"]
}, Open  ]],
Cell[25234, 839, 575, 17, 37, "Text"],
Cell[25812, 858, 703, 16, 52, "Text"],
Cell[26518, 876, 406, 14, 21, "Text"],
Cell[CellGroupData[{
Cell[26949, 894, 153, 4, 29, "Input"],
Cell[27105, 900, 29, 0, 28, "Output"]
}, Open  ]],
Cell[27149, 903, 1209, 35, 70, "Text"],
Cell[28361, 940, 107, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[28493, 947, 183, 6, 29, "Input"],
Cell[28679, 955, 31, 0, 28, "Output"]
}, Open  ]],
Cell[28725, 958, 432, 13, 37, "Text"],
Cell[29160, 973, 130, 4, 29, "myCode"],
Cell[29293, 979, 573, 16, 36, "Text"],
Cell[CellGroupData[{
Cell[29891, 999, 287, 9, 29, "Input"],
Cell[30181, 1010, 302, 10, 28, "Output"]
}, Open  ]],
Cell[30498, 1023, 661, 15, 51, "Text"],
Cell[31162, 1040, 472, 16, 45, "Input",
 InitializationCell->True],
Cell[31637, 1058, 238, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[31900, 1070, 125, 3, 29, "Input"],
Cell[32028, 1075, 29, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[32106, 1081, 36, 0, 18, "Subsubsection"],
Cell[32145, 1083, 168, 3, 35, "Text"],
Cell[32316, 1088, 480, 7, 65, "Text"],
Cell[32799, 1097, 435, 7, 50, "Text"],
Cell[33237, 1106, 602, 9, 80, "Text"],
Cell[33842, 1117, 259, 5, 35, "Text"],
Cell[34104, 1124, 1359, 38, 98, "Text"],
Cell[35466, 1164, 43, 0, 20, "Text"],
Cell[35512, 1166, 1099, 31, 123, "Input",
 InitializationCell->True],
Cell[36614, 1199, 702, 25, 35, "Text"],
Cell[CellGroupData[{
Cell[37341, 1228, 126, 3, 28, "Input"],
Cell[37470, 1233, 130, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[37637, 1242, 139, 4, 28, "Input"],
Cell[37779, 1248, 258, 8, 28, "Output"]
}, Open  ]],
Cell[38052, 1259, 829, 23, 67, "Text"],
Cell[38884, 1284, 43, 0, 20, "Text"],
Cell[38930, 1286, 1296, 37, 137, "Input",
 InitializationCell->True],
Cell[40229, 1325, 287, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[40541, 1336, 101, 2, 29, "Input"],
Cell[40645, 1340, 30, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[40748, 1348, 70, 1, 35, "Section",
 CellTags->"Section2"],
Cell[40821, 1351, 311, 7, 35, "Text"],
Cell[41135, 1360, 689, 18, 63, "Text"],
Cell[CellGroupData[{
Cell[41849, 1382, 96, 2, 29, "Input"],
Cell[41948, 1386, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[42013, 1391, 82, 1, 32, "Subsection",
 CellTags->"Section2a"],
Cell[42098, 1394, 310, 5, 50, "Text"],
Cell[42411, 1401, 504, 16, 35, "Text"],
Cell[42918, 1419, 315, 8, 35, "Text"],
Cell[43236, 1429, 651, 19, 51, "Text"],
Cell[43890, 1450, 1268, 38, 68, "Text"],
Cell[CellGroupData[{
Cell[45183, 1492, 281, 7, 28, "Input"],
Cell[45467, 1501, 155, 5, 28, "Output"]
}, Open  ]],
Cell[45637, 1509, 591, 9, 65, "Text"],
Cell[CellGroupData[{
Cell[46253, 1522, 326, 9, 28, "Input"],
Cell[46582, 1533, 146, 5, 28, "Output"]
}, Open  ]],
Cell[46743, 1541, 501, 14, 37, "Text"],
Cell[47247, 1557, 723, 25, 35, "Text"],
Cell[47973, 1584, 386, 11, 37, "Text"],
Cell[48362, 1597, 2093, 59, 186, "Input",
 InitializationCell->True],
Cell[50458, 1658, 162, 3, 35, "Text"],
Cell[50623, 1663, 379, 7, 26, "Equation"],
Cell[51005, 1672, 115, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[51145, 1679, 445, 9, 44, "Input"],
Cell[51593, 1690, 214, 6, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[51856, 1702, 78, 1, 26, "Subsection",
 CellTags->"Section2b"],
Cell[51937, 1705, 473, 12, 38, "Text"],
Cell[CellGroupData[{
Cell[52435, 1721, 47, 0, 18, "Subsubsection"],
Cell[52485, 1723, 625, 9, 65, "Text"],
Cell[53113, 1734, 613, 9, 65, "Text"],
Cell[53729, 1745, 744, 11, 80, "Text"],
Cell[54476, 1758, 107, 3, 20, "Text"],
Cell[54586, 1763, 6756, 200, 479, "Grid"],
Cell[61345, 1965, 192, 4, 20, "Text"],
Cell[61540, 1971, 642, 9, 80, "Text"],
Cell[62185, 1982, 567, 9, 66, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[62789, 1996, 51, 0, 18, "Subsubsection"],
Cell[62843, 1998, 200, 4, 35, "Text"],
Cell[63046, 2004, 357, 8, 36, "Text"],
Cell[63406, 2014, 627, 15, 54, "Text"],
Cell[64036, 2031, 1170, 25, 99, "Text"],
Cell[65209, 2058, 423, 11, 37, "Text"],
Cell[65635, 2071, 339, 8, 37, "Text"],
Cell[CellGroupData[{
Cell[65999, 2083, 349, 12, 17, "Item1Numbered"],
Cell[66351, 2097, 962, 27, 58, "Item1Numbered"],
Cell[67316, 2126, 613, 15, 45, "Item1Numbered"]
}, Open  ]],
Cell[67944, 2144, 235, 8, 21, "Text"],
Cell[CellGroupData[{
Cell[68204, 2156, 264, 9, 31, "Item1"],
Cell[68471, 2167, 682, 19, 58, "Item1"],
Cell[69156, 2188, 181, 7, 18, "Item1"]
}, Open  ]],
Cell[69352, 2198, 339, 7, 36, "Text"],
Cell[69694, 2207, 1153, 33, 71, "Text"],
Cell[70850, 2242, 800, 27, 36, "Text"],
Cell[71653, 2271, 47, 0, 20, "Text"],
Cell[71703, 2273, 4535, 115, 457, "Input",
 InitializationCell->True],
Cell[76241, 2390, 250, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[76516, 2402, 206, 5, 29, "Input"],
Cell[76725, 2409, 93, 2, 28, "Output"]
}, Open  ]],
Cell[76833, 2414, 68, 0, 20, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76938, 2419, 63, 0, 18, "Subsubsection"],
Cell[77004, 2421, 496, 13, 53, "Text"],
Cell[77503, 2436, 758, 24, 53, "Text"],
Cell[78264, 2462, 1490, 44, 100, "Text"],
Cell[79757, 2508, 498, 18, 35, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[80316, 2533, 75, 1, 35, "Section",
 CellTags->"Section3"],
Cell[80394, 2536, 170, 5, 20, "Text"],
Cell[CellGroupData[{
Cell[80589, 2545, 58, 1, 32, "Subsection",
 CellTags->"Section3a"],
Cell[80650, 2548, 92, 2, 20, "Text"],
Cell[80745, 2552, 106, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[80876, 2559, 44, 1, 28, "Input"],
Cell[80923, 2562, 30, 0, 28, "Output"]
}, Open  ]],
Cell[80968, 2565, 284, 9, 21, "Text"],
Cell[81255, 2576, 1059, 35, 64, "Text"],
Cell[82317, 2613, 345, 11, 29, "Input",
 InitializationCell->True],
Cell[82665, 2626, 262, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[82952, 2638, 83, 2, 28, "Input"],
Cell[83038, 2642, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83105, 2647, 45, 0, 18, "Subsubsection"],
Cell[83153, 2649, 268, 9, 21, "Text"],
Cell[83424, 2660, 421, 10, 35, "Text"],
Cell[CellGroupData[{
Cell[83870, 2674, 149, 4, 28, "Input"],
Cell[84022, 2680, 614, 15, 28, "Output"]
}, Open  ]],
Cell[84651, 2698, 499, 12, 37, "Text"],
Cell[CellGroupData[{
Cell[85175, 2714, 97, 2, 29, "Input"],
Cell[85275, 2718, 1949, 50, 74, "Output"]
}, Open  ]],
Cell[87239, 2771, 872, 25, 66, "Text"],
Cell[CellGroupData[{
Cell[88136, 2800, 121, 3, 29, "Input"],
Cell[88260, 2805, 1432, 48, 59, "Output"]
}, Open  ]],
Cell[89707, 2856, 521, 18, 35, "Text"],
Cell[CellGroupData[{
Cell[90253, 2878, 145, 4, 29, "Input"],
Cell[90401, 2884, 4193, 122, 134, "Output"]
}, Open  ]],
Cell[94609, 3009, 289, 10, 21, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[94935, 3024, 44, 0, 18, "Subsubsection"],
Cell[94982, 3026, 278, 8, 21, "Text"],
Cell[95263, 3036, 496, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[95784, 3050, 191, 5, 29, "Input"],
Cell[95978, 3057, 147, 3, 28, "Output"]
}, Open  ]],
Cell[96140, 3063, 435, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[96600, 3076, 122, 3, 29, "Input"],
Cell[96725, 3081, 83, 2, 28, "Output"]
}, Open  ]],
Cell[96823, 3086, 131, 3, 20, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[97003, 3095, 58, 1, 26, "Subsection",
 CellTags->"Section3b"],
Cell[97064, 3098, 107, 3, 20, "Text"],
Cell[97174, 3103, 396, 11, 35, "Text"],
Cell[CellGroupData[{
Cell[97595, 3118, 115, 3, 28, "Input"],
Cell[97713, 3123, 29, 0, 28, "Output"]
}, Open  ]],
Cell[97757, 3126, 531, 20, 36, "Text"],
Cell[CellGroupData[{
Cell[98313, 3150, 85, 2, 28, "Input"],
Cell[98401, 3154, 34, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[98472, 3159, 45, 0, 18, "Subsubsection"],
Cell[98520, 3161, 307, 10, 21, "Text"],
Cell[98830, 3173, 221, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[99076, 3183, 166, 5, 28, "Input"],
Cell[99245, 3190, 1189, 30, 44, "Output"]
}, Open  ]],
Cell[100449, 3223, 264, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[100738, 3237, 92, 2, 29, "Input"],
Cell[100833, 3241, 430, 14, 28, "Output"]
}, Open  ]],
Cell[101278, 3258, 813, 25, 51, "Text"],
Cell[CellGroupData[{
Cell[102116, 3287, 194, 6, 28, "Input"],
Cell[102313, 3295, 700, 19, 28, "Output"]
}, Open  ]],
Cell[103028, 3317, 444, 12, 35, "Text"],
Cell[CellGroupData[{
Cell[103497, 3333, 140, 4, 29, "Input"],
Cell[103640, 3339, 743, 22, 44, "Output"]
}, Open  ]],
Cell[104398, 3364, 452, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[104875, 3379, 165, 5, 29, "Input"],
Cell[105043, 3386, 1898, 57, 74, "Output"]
}, Open  ]],
Cell[106956, 3446, 837, 20, 67, "Text"],
Cell[CellGroupData[{
Cell[107818, 3470, 129, 3, 29, "Input"],
Cell[107950, 3475, 488, 16, 28, "Output"]
}, Open  ]],
Cell[108453, 3494, 634, 20, 52, "Text"],
Cell[CellGroupData[{
Cell[109112, 3518, 152, 4, 29, "Input"],
Cell[109267, 3524, 97, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[109401, 3532, 177, 5, 29, "Input"],
Cell[109581, 3539, 248, 8, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[109878, 3553, 44, 0, 18, "Subsubsection"],
Cell[109925, 3555, 815, 25, 51, "Text"],
Cell[CellGroupData[{
Cell[110765, 3584, 155, 4, 29, "Input"],
Cell[110923, 3590, 83, 2, 28, "Output"]
}, Open  ]],
Cell[111021, 3595, 1030, 31, 67, "Text"],
Cell[112054, 3628, 1032, 27, 68, "Text"],
Cell[CellGroupData[{
Cell[113111, 3659, 183, 5, 29, "Input"],
Cell[113297, 3666, 83, 2, 28, "Output"]
}, Open  ]],
Cell[113395, 3671, 2104, 59, 130, "Text"],
Cell[CellGroupData[{
Cell[115524, 3734, 449, 14, 45, "Input",
 InitializationCell->True],
Cell[115976, 3750, 107, 3, 28, "Output"]
}, Open  ]],
Cell[116098, 3756, 1453, 31, 128, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[117600, 3793, 67, 1, 26, "Subsection",
 CellTags->"Section3c"],
Cell[117670, 3796, 526, 16, 35, "Text"],
Cell[118199, 3814, 462, 11, 50, "Text"],
Cell[CellGroupData[{
Cell[118686, 3829, 47, 0, 18, "Subsubsection"],
Cell[118736, 3831, 434, 7, 50, "Text"],
Cell[119173, 3840, 740, 20, 37, "Text"],
Cell[CellGroupData[{
Cell[119938, 3864, 143, 4, 28, "Input"],
Cell[120084, 3870, 103, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[120224, 3877, 144, 4, 29, "Input"],
Cell[120371, 3883, 103, 2, 28, "Output"]
}, Open  ]],
Cell[120489, 3888, 229, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[120743, 3896, 168, 5, 28, "Input"],
Cell[120914, 3903, 103, 2, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[121066, 3911, 58, 0, 18, "Subsubsection"],
Cell[121127, 3913, 425, 11, 35, "Text"],
Cell[CellGroupData[{
Cell[121577, 3928, 160, 4, 28, "Input"],
Cell[121740, 3934, 122, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[121899, 3942, 63, 1, 28, "Input"],
Cell[121965, 3945, 122, 3, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122124, 3953, 63, 1, 28, "Input"],
Cell[122190, 3956, 122, 3, 28, "Output"]
}, Open  ]],
Cell[122327, 3962, 667, 17, 51, "Text"],
Cell[122997, 3981, 1195, 33, 137, "Input",
 InitializationCell->True],
Cell[124195, 4016, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[124329, 4023, 286, 8, 29, "Input"],
Cell[124618, 4033, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[124698, 4039, 58, 0, 18, "Subsubsection"],
Cell[124759, 4041, 841, 20, 67, "Text"],
Cell[125603, 4063, 616, 13, 67, "Text"],
Cell[126222, 4078, 315, 7, 35, "Text"],
Cell[126540, 4087, 2133, 55, 229, "Input",
 InitializationCell->True],
Cell[128676, 4144, 196, 6, 21, "Text"],
Cell[128875, 4152, 305, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[129205, 4165, 243, 6, 28, "Input"],
Cell[129451, 4173, 2210, 62, 104, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[131698, 4240, 284, 8, 29, "Input"],
Cell[131985, 4250, 29, 0, 28, "Output"]
}, Open  ]],
Cell[132029, 4253, 395, 8, 50, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[132485, 4268, 82, 1, 35, "Section",
 CellTags->"Section4"],
Cell[132570, 4271, 206, 6, 20, "Text"],
Cell[CellGroupData[{
Cell[132801, 4281, 66, 1, 32, "Subsection",
 CellTags->"Section4a"],
Cell[132870, 4284, 1002, 29, 85, "Text"],
Cell[133875, 4315, 118, 3, 20, "Text"],
Cell[133996, 4320, 673, 21, 54, "Text"],
Cell[CellGroupData[{
Cell[134694, 4345, 131, 4, 29, "Input"],
Cell[134828, 4351, 422, 13, 31, "Output"]
}, Open  ]],
Cell[135265, 4367, 735, 25, 55, "Text"],
Cell[CellGroupData[{
Cell[136025, 4396, 86, 2, 28, "Input"],
Cell[136114, 4400, 35, 0, 28, "Output"]
}, Open  ]],
Cell[136164, 4403, 346, 14, 23, "Text"],
Cell[136513, 4419, 638, 23, 39, "Text"],
Cell[137154, 4444, 677, 27, 57, "Equation"],
Cell[137834, 4473, 279, 12, 23, "Text"],
Cell[138116, 4487, 244, 10, 45, "Equation"],
Cell[138363, 4499, 23, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[138411, 4503, 212, 7, 28, "Input"],
Cell[138626, 4512, 61, 1, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[138736, 4519, 80, 1, 26, "Subsection",
 CellTags->"Section4b"],
Cell[138819, 4522, 691, 19, 51, "Text"],
Cell[CellGroupData[{
Cell[139535, 4545, 203, 6, 28, "Input"],
Cell[139741, 4553, 443, 8, 59, "Output"]
}, Open  ]],
Cell[140199, 4564, 358, 6, 50, "Text"],
Cell[140560, 4572, 548, 15, 62, "Text"],
Cell[141111, 4589, 170, 5, 21, "Text"],
Cell[141284, 4596, 423, 14, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[141732, 4614, 45, 0, 18, "Subsubsection"],
Cell[141780, 4616, 147, 4, 21, "Text"],
Cell[141930, 4622, 359, 17, 45, "Equation"],
Cell[142292, 4641, 206, 9, 20, "Text"],
Cell[142501, 4652, 389, 18, 45, "Equation"],
Cell[142893, 4672, 90, 2, 20, "Text"],
Cell[142986, 4676, 235, 11, 45, "Equation"],
Cell[143224, 4689, 1034, 28, 83, "Text"],
Cell[144261, 4719, 1297, 38, 105, "Input",
 InitializationCell->True],
Cell[145561, 4759, 532, 14, 37, "Text"],
Cell[CellGroupData[{
Cell[146118, 4777, 350, 11, 28, "Input"],
Cell[146471, 4790, 674, 23, 109, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[147182, 4818, 350, 11, 28, "Input"],
Cell[147535, 4831, 674, 23, 109, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[148258, 4860, 46, 0, 18, "Subsubsection"],
Cell[148307, 4862, 71, 0, 20, "Text"],
Cell[148381, 4864, 391, 10, 37, "Text"],
Cell[CellGroupData[{
Cell[148797, 4878, 389, 11, 60, "Input"],
Cell[149189, 4891, 139, 4, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[149365, 4900, 1801, 51, 168, "Input"],
Cell[151169, 4953, 138, 4, 28, "Output"]
}, Open  ]],
Cell[151322, 4960, 119, 3, 20, "Text"],
Cell[151444, 4965, 395, 12, 35, "Text"],
Cell[151842, 4979, 652, 19, 51, "Text"],
Cell[152497, 5000, 548, 19, 37, "Text"],
Cell[153048, 5021, 202, 7, 29, "Input"],
Cell[153253, 5030, 703, 24, 37, "Text"],
Cell[153959, 5056, 297, 9, 29, "Input"],
Cell[154259, 5067, 156, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[154440, 5075, 443, 13, 60, "Input"],
Cell[154886, 5090, 141, 4, 28, "Output"]
}, Open  ]],
Cell[155042, 5097, 1057, 26, 83, "Text"],
Cell[CellGroupData[{
Cell[156124, 5127, 2055, 58, 199, "Input"],
Cell[158182, 5187, 139, 4, 28, "Output"]
}, Open  ]],
Cell[158336, 5194, 664, 19, 50, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[159049, 5219, 66, 1, 26, "Subsection",
 CellTags->"Section4c"],
Cell[159118, 5222, 550, 17, 52, "Text"],
Cell[159671, 5241, 81, 2, 28, "Input"],
Cell[159755, 5245, 252, 10, 20, "Text"],
Cell[CellGroupData[{
Cell[160032, 5259, 84, 2, 28, "Input"],
Cell[160119, 5263, 225, 8, 47, "Output"]
}, Open  ]],
Cell[160359, 5274, 85, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[160469, 5280, 84, 2, 28, "Input"],
Cell[160556, 5284, 85, 2, 28, "Output"]
}, Open  ]],
Cell[160656, 5289, 690, 18, 52, "Text"],
Cell[CellGroupData[{
Cell[161371, 5311, 126, 3, 29, "Input"],
Cell[161500, 5316, 253, 8, 48, "Output"]
}, Open  ]],
Cell[161768, 5327, 696, 18, 51, "Text"],
Cell[CellGroupData[{
Cell[162489, 5349, 285, 9, 29, "Input"],
Cell[162777, 5360, 200, 8, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[163014, 5373, 43, 0, 18, "Subsubsection"],
Cell[163060, 5375, 336, 11, 20, "Text"],
Cell[163399, 5388, 87, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[163511, 5394, 138, 4, 28, "Input",
 InitializationCell->True],
Cell[163652, 5400, 85, 2, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[163774, 5407, 164, 5, 29, "Input",
 InitializationCell->True],
Cell[163941, 5414, 129, 4, 28, "Output"]
}, Open  ]],
Cell[164085, 5421, 432, 14, 36, "Text"],
Cell[CellGroupData[{
Cell[164542, 5439, 63, 1, 29, "Input"],
Cell[164608, 5442, 221, 7, 28, "Output"]
}, Open  ]],
Cell[164844, 5452, 341, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[165210, 5468, 58, 1, 29, "Input"],
Cell[165271, 5471, 32, 0, 28, "Output"]
}, Open  ]],
Cell[165318, 5474, 728, 22, 52, "Text"],
Cell[166049, 5498, 1036, 31, 53, "Text"],
Cell[CellGroupData[{
Cell[167110, 5533, 102, 2, 29, "Input"],
Cell[167215, 5537, 31, 0, 28, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[167295, 5543, 42, 0, 18, "Subsubsection"],
Cell[167340, 5545, 75, 0, 20, "Text"],
Cell[167418, 5547, 696, 30, 52, "Equation"],
Cell[168117, 5579, 114, 4, 20, "Text"],
Cell[168234, 5585, 87, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[168346, 5591, 350, 12, 29, "Input",
 InitializationCell->True],
Cell[168699, 5605, 340, 12, 46, "Output"]
}, Open  ]],
Cell[169054, 5620, 140, 5, 20, "Text"],
Cell[169197, 5627, 195, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[169417, 5637, 378, 12, 28, "Input",
 InitializationCell->True],
Cell[169798, 5651, 419, 16, 59, "Output"]
}, Open  ]],
Cell[170232, 5670, 150, 4, 20, "Text"],
Cell[170385, 5676, 573, 16, 37, "Text"],
Cell[CellGroupData[{
Cell[170983, 5696, 104, 2, 29, "Input"],
Cell[171090, 5700, 31, 0, 28, "Output"]
}, Open  ]],
Cell[171136, 5703, 473, 9, 50, "Text"],
Cell[171612, 5714, 784, 19, 66, "Text"]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[172457, 5740, 87, 1, 35, "Section",
 CellTags->"Section5"],
Cell[172547, 5743, 323, 7, 35, "Text"],
Cell[CellGroupData[{
Cell[172895, 5754, 74, 1, 32, "Subsection",
 CellTags->"Section5a"],
Cell[172972, 5757, 317, 12, 20, "Text"],
Cell[173292, 5771, 148, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[173465, 5778, 50, 1, 28, "Input"],
Cell[173518, 5781, 35, 0, 28, "Output"]
}, Open  ]],
Cell[173568, 5784, 179, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[173772, 5791, 49, 1, 28, "Input"],
Cell[173824, 5794, 29, 0, 28, "Output"]
}, Open  ]],
Cell[173868, 5797, 378, 8, 36, "Text"],
Cell[174249, 5807, 59, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[174333, 5811, 122, 3, 28, "Input"],
Cell[174458, 5816, 260, 8, 28, "Output"]
}, Open  ]],
Cell[174733, 5827, 608, 17, 51, "Text"],
Cell[175344, 5846, 144, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[175513, 5853, 171, 5, 28, "Input"],
Cell[175687, 5860, 230, 8, 28, "Output"]
}, Open  ]],
Cell[175932, 5871, 199, 4, 35, "Text"],
Cell[176134, 5877, 959, 26, 66, "Text"],
Cell[177096, 5905, 638, 20, 37, "Text"],
Cell[177737, 5927, 515, 10, 51, "Text"],
Cell[CellGroupData[{
Cell[178277, 5941, 97, 2, 29, "Input"],
Cell[178377, 5945, 128, 3, 28, "Output"]
}, Open  ]],
Cell[178520, 5951, 439, 10, 36, "Text"],
Cell[CellGroupData[{
Cell[178984, 5965, 312, 8, 29, "Input"],
Cell[179299, 5975, 280, 5, 28, "Output"]
}, Open  ]],
Cell[179594, 5983, 1057, 27, 69, "Text"],
Cell[CellGroupData[{
Cell[180676, 6014, 288, 9, 29, "Input"],
Cell[180967, 6025, 404, 11, 28, "Output"]
}, Open  ]],
Cell[181386, 6039, 630, 22, 36, "Text"],
Cell[CellGroupData[{
Cell[182041, 6065, 328, 10, 29, "Input"],
Cell[182372, 6077, 280, 5, 28, "Output"]
}, Open  ]],
Cell[182667, 6085, 271, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[182963, 6096, 574, 15, 76, "Input",
 InitializationCell->True],
Cell[183540, 6113, 9530, 245, 224, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193107, 6363, 69, 1, 29, "Input"],
Cell[193179, 6366, 29, 0, 28, "Output"]
}, Open  ]],
Cell[193223, 6369, 321, 10, 21, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[193581, 6384, 74, 1, 26, "Subsection",
 CellTags->"Section5b"],
Cell[193658, 6387, 190, 5, 20, "Text"],
Cell[193851, 6394, 466, 15, 35, "Text"],
Cell[194320, 6411, 357, 11, 29, "Input",
 InitializationCell->True],
Cell[194680, 6424, 166, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[194871, 6431, 80, 2, 29, "Input"],
Cell[194954, 6435, 30, 0, 28, "Output"]
}, Open  ]],
Cell[194999, 6438, 332, 10, 21, "Text"],
Cell[195334, 6450, 364, 8, 50, "Text"],
Cell[195701, 6460, 107, 4, 21, "Text"],
Cell[CellGroupData[{
Cell[195833, 6468, 80, 2, 29, "Input"],
Cell[195916, 6472, 29, 0, 28, "Output"]
}, Open  ]],
Cell[195960, 6475, 890, 24, 52, "Text"],
Cell[CellGroupData[{
Cell[196875, 6503, 144, 4, 28, "Input"],
Cell[197022, 6509, 529, 15, 28, "Output"]
}, Open  ]],
Cell[197566, 6527, 1014, 29, 51, "Text"],
Cell[CellGroupData[{
Cell[198605, 6560, 189, 5, 29, "Input"],
Cell[198797, 6567, 412, 12, 28, "Output"]
}, Open  ]],
Cell[199224, 6582, 95, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[199344, 6588, 575, 16, 91, "Input"],
Cell[199922, 6606, 2048, 47, 125, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202007, 6658, 59, 1, 29, "Input"],
Cell[202069, 6661, 29, 0, 28, "Output"]
}, Open  ]],
Cell[202113, 6664, 628, 18, 37, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[202778, 6687, 89, 1, 26, "Subsection",
 CellTags->"Section5c"],
Cell[202870, 6690, 542, 14, 51, "Text"],
Cell[203415, 6706, 759, 20, 52, "Text"],
Cell[CellGroupData[{
Cell[204199, 6730, 267, 7, 29, "Input"],
Cell[204469, 6739, 30, 0, 28, "Output"]
}, Open  ]],
Cell[204514, 6742, 99, 2, 20, "Text"],
Cell[204616, 6746, 527, 16, 36, "Text"],
Cell[205146, 6764, 142, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[205313, 6771, 278, 7, 28, "Input"],
Cell[205594, 6780, 4055, 89, 104, "Output"]
}, Open  ]],
Cell[209664, 6872, 185, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[209874, 6882, 59, 1, 29, "Input"],
Cell[209936, 6885, 29, 0, 28, "Output"]
}, Open  ]],
Cell[209980, 6888, 501, 11, 51, "Text"],
Cell[CellGroupData[{
Cell[210506, 6903, 343, 9, 29, "Input"],
Cell[210852, 6914, 31, 0, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[210932, 6920, 95, 1, 26, "Subsection",
 CellTags->"Section5d"],
Cell[211030, 6923, 257, 7, 35, "Text"],
Cell[211290, 6932, 527, 18, 21, "Text"],
Cell[CellGroupData[{
Cell[211842, 6954, 309, 9, 28, "Input"],
Cell[212154, 6965, 52, 0, 28, "Output"]
}, Open  ]],
Cell[212221, 6968, 382, 13, 32, "Text"],
Cell[CellGroupData[{
Cell[212628, 6985, 246, 9, 28, "Input"],
Cell[212877, 6996, 52, 0, 28, "Output"]
}, Open  ]],
Cell[212944, 6999, 1630, 53, 94, "Text"],
Cell[CellGroupData[{
Cell[214599, 7056, 118, 2, 28, "Input"],
Cell[214720, 7060, 52, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[214809, 7065, 102, 4, 18, "Subsubsection"],
Cell[214914, 7071, 579, 10, 66, "Text"],
Cell[215496, 7083, 468, 9, 50, "Text"],
Cell[215967, 7094, 434, 11, 36, "Text"],
Cell[216404, 7107, 121, 2, 29, "myCode"],
Cell[216528, 7111, 166, 3, 35, "Text"],
Cell[216697, 7116, 534, 13, 52, "Text"],
Cell[217234, 7131, 66, 0, 20, "Text"],
Cell[217303, 7133, 127, 3, 29, "myCode"],
Cell[217433, 7138, 458, 12, 37, "Text"],
Cell[217894, 7152, 149, 4, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[218068, 7160, 106, 2, 29, "Input"],
Cell[218177, 7164, 451, 9, 24, "Message"],
Cell[218631, 7175, 103, 2, 29, "Output"]
}, Open  ]],
Cell[218749, 7180, 138, 4, 21, "Text"],
Cell[218890, 7186, 174, 5, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[219089, 7195, 107, 2, 29, "Input"],
Cell[219199, 7199, 28, 0, 28, "Output"]
}, Open  ]],
Cell[219242, 7202, 325, 7, 36, "Text"],
Cell[219570, 7211, 144, 4, 29, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[219739, 7219, 103, 2, 29, "Input"],
Cell[219845, 7223, 29, 0, 28, "Output"]
}, Open  ]],
Cell[219889, 7226, 1057, 30, 69, "Text"],
Cell[220949, 7258, 823, 23, 105, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[221797, 7285, 120, 2, 29, "Input"],
Cell[221920, 7289, 52, 0, 28, "Output"]
}, Open  ]],
Cell[221987, 7292, 179, 4, 20, "Text"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[222215, 7302, 97, 1, 26, "Subsection",
 CellTags->"Section5e"],
Cell[222315, 7305, 199, 4, 35, "Text"],
Cell[222517, 7311, 188, 5, 21, "Text"],
Cell[CellGroupData[{
Cell[222730, 7320, 81, 2, 29, "Input"],
Cell[222814, 7324, 32, 0, 28, "Output"]
}, Open  ]],
Cell[222861, 7327, 224, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[223110, 7335, 59, 0, 18, "Subsubsection"],
Cell[223172, 7337, 1041, 36, 51, "Text"],
Cell[224216, 7375, 359, 11, 35, "Text"],
Cell[224578, 7388, 768, 30, 35, "Text"],
Cell[225349, 7420, 432, 15, 26, "Equation"],
Cell[225784, 7437, 569, 21, 35, "Text"],
Cell[226356, 7460, 986, 36, 50, "Text"],
Cell[227345, 7498, 634, 19, 37, "Text"],
Cell[CellGroupData[{
Cell[228004, 7521, 106, 2, 28, "Input",
 InitializationCell->True],
Cell[228113, 7525, 355, 8, 58, "Message"]
}, Open  ]],
Cell[228483, 7536, 376, 9, 35, "Text"],
Cell[228862, 7547, 1169, 38, 52, "Text"],
Cell[CellGroupData[{
Cell[230056, 7589, 96, 2, 29, "Input"],
Cell[230155, 7593, 28, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[230220, 7598, 88, 2, 29, "Input"],
Cell[230311, 7602, 348, 12, 28, "Output"]
}, Open  ]],
Cell[230674, 7617, 511, 15, 35, "Text"],
Cell[CellGroupData[{
Cell[231210, 7636, 97, 2, 29, "Input"],
Cell[231310, 7640, 32, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[231403, 7647, 97, 1, 26, "Subsection",
 CellTags->"Section5f"],
Cell[231503, 7650, 413, 14, 35, "Text"],
Cell[231919, 7666, 455, 9, 51, "Text"],
Cell[CellGroupData[{
Cell[232399, 7679, 86, 2, 29, "Input"],
Cell[232488, 7683, 30, 0, 28, "Output"]
}, Open  ]],
Cell[232533, 7686, 250, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[232808, 7694, 201, 6, 29, "Input"],
Cell[233012, 7702, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[233079, 7707, 96, 1, 18, "Subsubsection",
 PageBreakWithin->False],
Cell[233178, 7710, 662, 17, 52, "Text",
 PageBreakAbove->False,
 PageBreakWithin->False],
Cell[233843, 7729, 811, 19, 65, "Text"],
Cell[234657, 7750, 752, 26, 50, "Text"],
Cell[235412, 7778, 924, 32, 35, "Text"],
Cell[236339, 7812, 679, 26, 35, "Text"],
Cell[237021, 7840, 807, 25, 26, "Equation"],
Cell[237831, 7867, 170, 3, 35, "Text"],
Cell[238004, 7872, 3423, 88, 359, "Input",
 InitializationCell->True],
Cell[241430, 7962, 396, 11, 36, "Text"],
Cell[241829, 7975, 882, 26, 67, "Text"],
Cell[242714, 8003, 241, 9, 21, "Text"],
Cell[242958, 8014, 572, 20, 36, "Text"],
Cell[243533, 8036, 789, 29, 37, "Text"],
Cell[244325, 8067, 635, 21, 26, "Equation"],
Cell[244963, 8090, 128, 6, 21, "Text"],
Cell[245094, 8098, 856, 30, 51, "Text"],
Cell[245953, 8130, 1179, 42, 72, "Equation"],
Cell[247135, 8174, 1448, 43, 86, "Text"],
Cell[248586, 8219, 282, 7, 36, "Text"],
Cell[CellGroupData[{
Cell[248893, 8230, 119, 3, 29, "Input"],
Cell[249015, 8235, 1335, 48, 109, "Output"]
}, Open  ]],
Cell[250365, 8286, 215, 4, 35, "Text"],
Cell[250583, 8292, 901, 22, 68, "Text"],
Cell[CellGroupData[{
Cell[251509, 8318, 313, 10, 29, "Input"],
Cell[251825, 8330, 2617, 97, 237, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[254503, 8434, 99, 1, 26, "Subsection",
 CellTags->"Section5g"],
Cell[254605, 8437, 1038, 36, 35, "Text"],
Cell[255646, 8475, 1044, 32, 53, "Text"],
Cell[256693, 8509, 134, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[256852, 8516, 64, 1, 28, "Input"],
Cell[256919, 8519, 29, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[256985, 8524, 70, 1, 29, "Input"],
Cell[257058, 8527, 1172, 33, 59, "Output"]
}, Open  ]],
Cell[258245, 8563, 580, 13, 52, "Text"],
Cell[CellGroupData[{
Cell[258850, 8580, 93, 2, 29, "Input"],
Cell[258946, 8584, 540, 17, 28, "Output"]
}, Open  ]],
Cell[259501, 8604, 897, 29, 51, "Text"],
Cell[260401, 8635, 116, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[260542, 8642, 142, 4, 29, "Input"],
Cell[260687, 8648, 747, 21, 44, "Output"]
}, Open  ]],
Cell[261449, 8672, 605, 13, 52, "Text"],
Cell[262057, 8687, 109, 3, 20, "Text"],
Cell[CellGroupData[{
Cell[262191, 8694, 165, 4, 29, "Input"],
Cell[262359, 8700, 719, 19, 44, "Output"]
}, Open  ]],
Cell[263093, 8722, 501, 11, 52, "Text"],
Cell[CellGroupData[{
Cell[263619, 8737, 136, 3, 29, "Input"],
Cell[263758, 8742, 737, 21, 44, "Output"]
}, Open  ]],
Cell[264510, 8766, 209, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[264744, 8776, 171, 4, 29, "Input"],
Cell[264918, 8782, 28, 0, 28, "Output"]
}, Open  ]],
Cell[264961, 8785, 208, 6, 21, "Text"],
Cell[CellGroupData[{
Cell[265194, 8795, 72, 0, 18, "Subsubsection"],
Cell[265269, 8797, 548, 20, 35, "Text"],
Cell[265820, 8819, 548, 15, 36, "Text"],
Cell[266371, 8836, 258, 7, 58, "Text"],
Cell[266632, 8845, 121, 3, 20, "Text"],
Cell[266756, 8850, 258, 7, 60, "Text"],
Cell[267017, 8859, 522, 11, 50, "Text"],
Cell[267542, 8872, 300, 10, 21, "Text"],
Cell[CellGroupData[{
Cell[267867, 8886, 138, 3, 29, "Input"],
Cell[268008, 8891, 83, 2, 28, "Output"]
}, Open  ]],
Cell[268106, 8896, 448, 11, 50, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[268591, 8912, 46, 0, 18, "Subsubsection"],
Cell[268640, 8914, 377, 10, 36, "Text"],
Cell[269020, 8926, 743, 26, 50, "Text"],
Cell[269766, 8954, 830, 29, 50, "Text"],
Cell[270599, 8985, 820, 30, 35, "Text"],
Cell[CellGroupData[{
Cell[271444, 9019, 138, 3, 29, "Input"],
Cell[271585, 9024, 319, 10, 28, "Output"]
}, Open  ]],
Cell[271919, 9037, 535, 15, 35, "Text"],
Cell[272457, 9054, 932, 35, 50, "Text"],
Cell[273392, 9091, 93, 2, 20, "Text"],
Cell[273488, 9095, 512, 16, 36, "Text"],
Cell[274003, 9113, 278, 8, 45, "Input",
 InitializationCell->True],
Cell[274284, 9123, 223, 10, 20, "Text"],
Cell[274510, 9135, 244, 8, 29, "Input",
 InitializationCell->True],
Cell[274757, 9145, 278, 10, 20, "Text"],
Cell[275038, 9157, 465, 15, 45, "Input",
 InitializationCell->True],
Cell[275506, 9174, 256, 7, 21, "Text"],
Cell[CellGroupData[{
Cell[275787, 9185, 128, 3, 29, "Input"],
Cell[275918, 9190, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[275985, 9195, 90, 2, 29, "Input"],
Cell[276078, 9199, 30, 0, 28, "Output"]
}, Open  ]],
Cell[276123, 9202, 175, 6, 20, "Text"],
Cell[276301, 9210, 155, 4, 29, "Input"],
Cell[276459, 9216, 71, 0, 20, "Text"],
Cell[CellGroupData[{
Cell[276555, 9220, 67, 1, 29, "Input"],
Cell[276625, 9223, 30, 0, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[276692, 9228, 65, 1, 28, "Input"],
Cell[276760, 9231, 30, 0, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[276863, 9239, 86, 1, 35, "Section",
 CellTags->"Section6"],
Cell[276952, 9242, 238, 4, 35, "Text"],
Cell[CellGroupData[{
Cell[277215, 9250, 86, 1, 32, "Subsection",
 CellTags->"Section6a"],
Cell[277304, 9253, 196, 4, 35, "Text"],
Cell[277503, 9259, 183, 5, 21, "Text"],
Cell[277689, 9266, 368, 6, 50, "Text"],
Cell[278060, 9274, 74, 0, 20, "Text"],
Cell[278137, 9276, 1020, 30, 124, "Input",
 InitializationCell->True],
Cell[CellGroupData[{
Cell[279182, 9310, 248, 7, 29, "Input"],
Cell[279433, 9319, 165, 3, 28, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[279647, 9328, 90, 1, 26, "Subsection",
 CellTags->"Section6b"],
Cell[279740, 9331, 1243, 46, 53, "Text"],
Cell[280986, 9379, 1398, 44, 73, "Text"],
Cell[282387, 9425, 2235, 85, 93, "Text"],
Cell[284625, 9512, 1795, 67, 73, "Text"],
Cell[286423, 9581, 341, 10, 29, "Equation"],
Cell[286767, 9593, 39, 0, 20, "Text"],
Cell[286809, 9595, 1767, 73, 55, "Text"],
Cell[288579, 9670, 118, 3, 20, "Text"],
Cell[288700, 9675, 2102, 59, 101, "Text"],
Cell[290805, 9736, 44, 0, 20, "Text"],
Cell[290852, 9738, 3121, 83, 392, "Input",
 InitializationCell->True],
Cell[293976, 9823, 166, 3, 35, "Text"],
Cell[CellGroupData[{
Cell[294167, 9830, 158, 4, 28, "Input"],
Cell[294328, 9836, 116, 3, 28, "Output"]
}, Open  ]],
Cell[294459, 9842, 330, 12, 21, "Text"],
Cell[CellGroupData[{
Cell[294814, 9858, 433, 11, 93, "Input"],
Cell[CellGroupData[{
Cell[295272, 9873, 92, 2, 21, "Print"],
Cell[295367, 9877, 92, 2, 21, "Print"],
Cell[295462, 9881, 92, 2, 21, "Print"],
Cell[295557, 9885, 92, 2, 21, "Print"],
Cell[295652, 9889, 92, 2, 21, "Print"],
Cell[295747, 9893, 92, 2, 21, "Print"],
Cell[295842, 9897, 92, 2, 21, "Print"],
Cell[295937, 9901, 92, 2, 21, "Print"],
Cell[296032, 9905, 92, 2, 21, "Print"],
Cell[296127, 9909, 92, 2, 21, "Print"],
Cell[296222, 9913, 92, 2, 21, "Print"],
Cell[296317, 9917, 92, 2, 21, "Print"],
Cell[296412, 9921, 92, 2, 21, "Print"],
Cell[296507, 9925, 92, 2, 21, "Print"],
Cell[296602, 9929, 92, 2, 21, "Print"],
Cell[296697, 9933, 92, 2, 21, "Print"],
Cell[296792, 9937, 92, 2, 21, "Print"],
Cell[296887, 9941, 92, 2, 21, "Print"],
Cell[296982, 9945, 92, 2, 21, "Print"],
Cell[297077, 9949, 92, 2, 21, "Print"],
Cell[297172, 9953, 92, 2, 21, "Print"],
Cell[297267, 9957, 92, 2, 21, "Print"],
Cell[297362, 9961, 92, 2, 21, "Print"],
Cell[297457, 9965, 92, 2, 21, "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[297610, 9974, 80, 1, 26, "Subsection",
 CellTags->"Section6c"],
Cell[297693, 9977, 629, 20, 35, "Text"],
Cell[298325, 9999, 1515, 52, 68, "Text"],
Cell[CellGroupData[{
Cell[299865, 10055, 217, 6, 28, "Input"],
Cell[300085, 10063, 107, 2, 28, "Output"]
}, Open  ]],
Cell[300207, 10068, 411, 13, 35, "Text"],
Cell[CellGroupData[{
Cell[300643, 10085, 216, 6, 28, "Input"],
Cell[300862, 10093, 96, 2, 28, "Output"]
}, Open  ]],
Cell[300973, 10098, 387, 12, 36, "Text"],
Cell[CellGroupData[{
Cell[301385, 10114, 61, 0, 18, "Subsubsection"],
Cell[301449, 10116, 360, 10, 36, "Text"],
Cell[301812, 10128, 1014, 27, 68, "Text"],
Cell[302829, 10157, 995, 26, 155, "Input",
 InitializationCell->True],
Cell[303827, 10185, 221, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[304073, 10197, 148, 4, 29, "Input"],
Cell[304224, 10203, 614, 15, 28, "Output"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[304911, 10226, 115, 3, 35, "Section",
 CellTags->"Solutions"],
Cell[CellGroupData[{
Cell[305051, 10233, 70, 1, 32, "Subsection",
 CellTags->"SolutionsCP10"],
Cell[305124, 10236, 443, 17, 42, "Problem"],
Cell[305570, 10255, 488, 13, 37, "Text"],
Cell[306061, 10270, 703, 23, 36, "Text"],
Cell[CellGroupData[{
Cell[306789, 10297, 491, 13, 61, "Input"],
Cell[307283, 10312, 407, 14, 28, "Output"]
}, Open  ]],
Cell[307705, 10329, 691, 20, 36, "Text"],
Cell[308399, 10351, 303, 11, 20, "Text"],
Cell[308705, 10364, 967, 27, 108, "Input",
 InitializationCell->True],
Cell[309675, 10393, 215, 8, 20, "Text"],
Cell[CellGroupData[{
Cell[309915, 10405, 93, 2, 29, "Input"],
Cell[310011, 10409, 2468, 72, 89, "Output"]
}, Open  ]]
}, Closed]],
Cell[CellGroupData[{
Cell[312528, 10487, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE1"],
Cell[312611, 10490, 186, 4, 55, "Problem"],
Cell[312800, 10496, 626, 15, 50, "Text"],
Cell[313429, 10513, 322, 11, 35, "Text"],
Cell[313754, 10526, 343, 6, 50, "Text"],
Cell[314100, 10534, 788, 20, 67, "Text"],
Cell[314891, 10556, 358, 11, 29, "Input",
 InitializationCell->True],
Cell[315252, 10569, 291, 9, 20, "Text"],
Cell[CellGroupData[{
Cell[315568, 10582, 207, 6, 29, "Input"],
Cell[315778, 10590, 31, 0, 28, "Output"]
}, Open  ]],
Cell[315824, 10593, 453, 17, 37, "Text"],
Cell[316280, 10612, 682, 22, 53, "Text"],
Cell[316965, 10636, 37, 0, 20, "Text"],
Cell[317005, 10638, 1974, 52, 187, "Input",
 InitializationCell->True],
Cell[318982, 10692, 88, 2, 20, "Text"],
Cell[CellGroupData[{
Cell[319095, 10698, 94, 2, 29, "Input"],
Cell[319192, 10702, 1736, 43, 74, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[320965, 10750, 66, 1, 29, "Input"],
Cell[321034, 10753, 29, 0, 28, "Output"]
}, Open  ]],
Cell[321078, 10756, 142, 3, 20, "Text"]
}, Closed]],
Cell[CellGroupData[{
Cell[321257, 10764, 80, 1, 26, "Subsection",
 CellTags->"SolutionsCE3"],
Cell[321340, 10767, 988, 31, 68, "Problem"],
Cell[322331, 10800, 379, 12, 35, "Text"],
Cell[322713, 10814, 261, 12, 20, "Text"],
Cell[CellGroupData[{
Cell[322999, 10830, 136, 4, 28, "Input",
 InitializationCell->True],
Cell[323138, 10836, 29, 0, 28, "Output"]
}, Open  ]],
Cell[323182, 10839, 424, 11, 35, "Text"],
Cell[323609, 10852, 1344, 42, 51, "Text"],
Cell[324956, 10896, 274, 11, 21, "Text"],
Cell[CellGroupData[{
Cell[325255, 10911, 67, 1, 29, "Input"],
Cell[325325, 10914, 171, 6, 28, "Output"]
}, Open  ]],
Cell[325511, 10923, 282, 11, 23, "Text"],
Cell[325796, 10936, 369, 11, 37, "Text"],
Cell[CellGroupData[{
Cell[326190, 10951, 144, 4, 29, "Input"],
Cell[326337, 10957, 73, 2, 28, "Output"]
}, Open  ]],
Cell[326425, 10962, 312, 8, 35, "Text"],
Cell[326740, 10972, 627, 18, 37, "Text"],
Cell[327370, 10992, 504, 19, 23, "Text"],
Cell[CellGroupData[{
Cell[327899, 11015, 65, 1, 29, "Input"],
Cell[327967, 11018, 32, 0, 28, "Output"]
}, Open  ]],
Cell[328014, 11021, 536, 18, 21, "Text"],
Cell[CellGroupData[{
Cell[328575, 11043, 123, 3, 29, "Input"],
Cell[328701, 11048, 31, 0, 28, "Output"]
}, Open  ]],
Cell[328747, 11051, 1404, 33, 98, "Text"],
Cell[CellGroupData[{
Cell[330176, 11088, 982, 25, 154, "Input"],
Cell[CellGroupData[{
Cell[331183, 11117, 180, 4, 22, "Print"],
Cell[331366, 11123, 180, 4, 22, "Print"],
Cell[331549, 11129, 180, 4, 22, "Print"],
Cell[331732, 11135, 178, 4, 22, "Print"]
}, Open  ]]
}, Open  ]]
}, Closed]]
}, Closed]],
Cell[CellGroupData[{
Cell[331983, 11147, 52, 1, 35, "Section",
 CellTags->"Exercises"],
Cell[CellGroupData[{
Cell[332060, 11152, 570, 12, 61, "Item1Exercise"],
Cell[332633, 11166, 235, 7, 34, "Item1Exercise"],
Cell[332871, 11175, 184, 5, 20, "Item1Exercise"],
Cell[333058, 11182, 381, 12, 33, "Item1Exercise"],
Cell[333442, 11196, 541, 10, 46, "Item1Exercise"],
Cell[333986, 11208, 462, 15, 33, "Item1Exercise"],
Cell[334451, 11225, 120, 5, 20, "Item1Exercise"],
Cell[334574, 11232, 344, 15, 46, "ExerciseEquation"],
Cell[334921, 11249, 281, 12, 16, "ExerciseContinuation"],
Cell[335205, 11263, 126, 5, 20, "Item1Exercise"],
Cell[335334, 11270, 367, 17, 45, "ExerciseEquation"],
Cell[335704, 11289, 285, 12, 16, "ExerciseContinuation"],
Cell[335992, 11303, 421, 10, 46, "Item1Exercise"],
Cell[336416, 11315, 437, 16, 34, "Item1Exercise"],
Cell[336856, 11333, 146, 5, 20, "Item1Exercise"],
Cell[337005, 11340, 233, 8, 26, "ExerciseEquation"],
Cell[337241, 11350, 738, 24, 29, "ExerciseContinuation"],
Cell[337982, 11376, 298, 5, 46, "Item1Exercise"],
Cell[338283, 11383, 186, 3, 20, "Item1Exercise"],
Cell[338472, 11388, 830, 27, 46, "Item1Exercise"],
Cell[339305, 11417, 116, 3, 20, "Item1Exercise"],
Cell[339424, 11422, 170, 5, 20, "Item1Exercise"],
Cell[339597, 11429, 221, 6, 33, "Item1Exercise"],
Cell[339821, 11437, 635, 19, 33, "Item1Exercise"]
}, Open  ]]
}, Closed]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
